@@ -26,14 +26,19 @@
 
 #include <opm/material/fluidsystems/BlackOilFluidSystem.hpp>
 
-#include <opm/input/eclipse/Deck/Deck.hpp>
 #include <opm/input/eclipse/EclipseState/EclipseState.hpp>
-#include <opm/input/eclipse/Schedule/Schedule.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/OverburdTable.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/RockwnodTable.hpp>
 
+#include <opm/input/eclipse/Schedule/Schedule.hpp>
+
+#include <opm/input/eclipse/Units/Units.hpp>
+
 #include <opm/grid/CpGrid.hpp>
 #include <opm/grid/polyhedralgrid.hh>
+
+#include <dune/common/parametertree.hh>
+
 #if HAVE_DUNE_ALUGRID
 #include <dune/alugrid/grid.hh>
 #include <dune/alugrid/3d/gridview.hh>
@@ -48,12 +53,46 @@
 
 #include <boost/date_time.hpp>
 
+#include <algorithm>
 #include <limits>
 #include <stdexcept>
 #include <iostream>
 
 namespace Opm {
 
+int eclPositionalParameter(Dune::ParameterTree& tree,
+                           std::set<std::string>& seenParams,
+                           std::string& errorMsg,
+                           const char** argv,
+                           int paramIdx)
+{
+    std::string param  = argv[paramIdx];
+    size_t i = param.find('=');
+    if (i != std::string::npos) {
+        std::string oldParamName = param.substr(0, i);
+        std::string oldParamValue = param.substr(i+1);
+        std::string newParamName = "--" + oldParamName;
+        std::replace(newParamName.begin(),
+                     newParamName.end(), '_' , '-');
+        errorMsg =
+          "The old syntax to specify parameters on the command line is no longer supported: "
+          "Try replacing '" + oldParamName + "=" + oldParamValue + "' with "+
+          "'" + newParamName + "=" + oldParamValue + "'!";
+        return 0;
+    }
+
+    if (seenParams.count("EclDeckFileName") > 0) {
+        errorMsg =
+            "Parameter 'EclDeckFileName' specified multiple times"
+            " as a command line parameter";
+        return 0;
+    }
+
+    tree["EclDeckFileName"] = argv[paramIdx];
+    seenParams.insert("EclDeckFileName");
+    return 1;
+}
+
 template<class GridView, class FluidSystem, class Scalar>
 EclGenericProblem<GridView,FluidSystem,Scalar>::
 EclGenericProblem(const EclipseState& eclState,
@@ -65,6 +104,37 @@ EclGenericProblem(const EclipseState& eclState,
 {
 }
 
+template<class GridView, class FluidSystem, class Scalar>
+EclGenericProblem<GridView,FluidSystem,Scalar>
+EclGenericProblem<GridView,FluidSystem,Scalar>::
+serializationTestObject(const EclipseState& eclState,
+                        const Schedule& schedule,
+                        const GridView& gridView)
+{
+    EclGenericProblem result(eclState, schedule, gridView);
+    result.maxOilSaturation_ = {1.0, 2.0};
+    result.maxPolymerAdsorption_ = {3.0, 4.0, 5.0};
+    result.maxWaterSaturation_ = {6.0};
+    result.minOilPressure_ = {7.0, 8.0, 9.0, 10.0};
+    result.overburdenPressure_ = {11.0};
+    result.polymerConcentration_ = {12.0};
+    result.polymerMoleWeight_ = {13.0, 14.0};
+    result.solventSaturation_ = {15.0};
+    result.microbialConcentration_ = {16.0};
+    result.oxygenConcentration_ = {17.0};
+    result.ureaConcentration_ = {18.0};
+    result.biofilmConcentration_ = {19.0};
+    result.calciteConcentration_ = {20.0};
+    result.lastRv_ = {21.0};
+    result.maxDRv_ = {22.0, 23.0};
+    result.convectiveDrs_ = {24.0, 25.0, 26.0};
+    result.lastRs_ = {27.0};
+    result.maxDRs_ = {28.0};
+    result.dRsDtOnlyFreeGas_ = {false, true};
+
+    return result;
+}
+
 template<class GridView, class FluidSystem, class Scalar>
 std::string
 EclGenericProblem<GridView,FluidSystem,Scalar>::
@@ -277,6 +347,20 @@ porosity(unsigned globalSpaceIdx, unsigned timeIdx) const
     return this->referencePorosity_[timeIdx][globalSpaceIdx];
 }
 
+template<class GridView, class FluidSystem, class Scalar>
+Scalar EclGenericProblem<GridView,FluidSystem,Scalar>::
+rockFraction(unsigned elementIdx, unsigned timeIdx) const
+{
+    const auto& fp = eclState_.fieldProps();
+    const std::vector<double>& poro  = fp.get_double("PORO");
+    // the reference porosity is defined as the accumulated pore volume divided by the
+    // geometric volume of the element. Note that it can
+    // be larger than 1.0 if porevolume multipliers are used
+    // to for instance implement larger boundary cells
+    Scalar porosity = poro[elementIdx];
+    return referencePorosity(elementIdx, timeIdx) / porosity * (1 - porosity);
+}
+
 template<class GridView, class FluidSystem, class Scalar>
 template<class T>
 void EclGenericProblem<GridView,FluidSystem,Scalar>::
@@ -331,6 +415,15 @@ updateKrnum_()
     updateNum("KRNUMZ", krnumz_);
 }
 
+template<class GridView, class FluidSystem, class Scalar>
+void EclGenericProblem<GridView,FluidSystem,Scalar>::
+updateImbnum_()
+{
+    updateNum("IMBNUMX", imbnumx_);
+    updateNum("IMBNUMY", imbnumy_);
+    updateNum("IMBNUMZ", imbnumz_);
+}
+
 template<class GridView, class FluidSystem, class Scalar>
 bool EclGenericProblem<GridView,FluidSystem,Scalar>::
 vapparsActive(int episodeIdx) const
@@ -446,80 +539,6 @@ beginTimeStep_(bool enableExperiments,
             maxDRv_[pvtRegionIdx] = oilVaporizationControl.getMaxDRVDT(pvtRegionIdx)*timeStepSize;
 }
 
-template<class GridView, class FluidSystem, class Scalar>
-void EclGenericProblem<GridView,FluidSystem,Scalar>::
-checkDeckCompatibility_(const Deck& deck,
-                        bool enableApiTracking,
-                        bool enableSolvent,
-                        bool enablePolymer,
-                        bool enableExtbo,
-                        bool enableEnergy,
-                        int numPhases,
-                        bool indicesGasEnabled,
-                        bool indicesOilEnabled,
-                        bool indicesWaterEnabled,
-                        bool enableMICP) const
-{
-    if (enableApiTracking)
-        throw std::logic_error("API tracking is not yet implemented but requested at compile time.");
-    if (!enableApiTracking && deck.hasKeyword("API"))
-        throw std::logic_error("The simulator is build with API tracking disabled, but API tracking is requested by the deck.");
-
-    if (enableSolvent && !deck.hasKeyword("SOLVENT"))
-        throw std::runtime_error("The simulator requires the solvent option to be enabled, but the deck does not.");
-    else if (!enableSolvent && deck.hasKeyword("SOLVENT"))
-        throw std::runtime_error("The deck enables the solvent option, but the simulator is compiled without it.");
-
-    if (enablePolymer && !deck.hasKeyword("POLYMER"))
-        throw std::runtime_error("The simulator requires the polymer option to be enabled, but the deck does not.");
-    else if (!enablePolymer && deck.hasKeyword("POLYMER"))
-        throw std::runtime_error("The deck enables the polymer option, but the simulator is compiled without it.");
-
-    if (enableMICP && !deck.hasKeyword("MICP"))
-        throw std::runtime_error("The simulator requires the MICP option to be enabled, but the deck does not.");
-    else if (!enableMICP && deck.hasKeyword("MICP"))
-        throw std::runtime_error("The deck enables the MICP option, but the simulator is compiled without it.");
-
-    if (enableExtbo && !deck.hasKeyword("PVTSOL"))
-        throw std::runtime_error("The simulator requires the extendedBO option to be enabled, but the deck does not.");
-    else if (!enableExtbo && deck.hasKeyword("PVTSOL"))
-        throw std::runtime_error("The deck enables the extendedBO option, but the simulator is compiled without it.");
-
-    if (deck.hasKeyword("TEMP") && deck.hasKeyword("THERMAL"))
-        throw std::runtime_error("The deck enables both, the TEMP and the THERMAL options, but they are mutually exclusive.");
-
-    bool deckEnergyEnabled = (deck.hasKeyword("TEMP") || deck.hasKeyword("THERMAL"));
-    if (enableEnergy && !deckEnergyEnabled)
-        throw std::runtime_error("The simulator requires the TEMP or the THERMAL option to be enabled, but the deck activates neither.");
-    else if (!enableEnergy && deckEnergyEnabled)
-        throw std::runtime_error("The deck enables the TEMP or the THERMAL option, but the simulator is not compiled to support either.");
-
-    if (deckEnergyEnabled && deck.hasKeyword("TEMP"))
-        std::cerr << "WARNING: The deck requests the TEMP option, i.e., treating energy "
-                  << "conservation as a post processing step. This is currently unsupported, "
-                  << "i.e., energy conservation is always handled fully implicitly." << std::endl;
-
-    int numDeckPhases = FluidSystem::numActivePhases();
-    if (numDeckPhases < numPhases)
-        std::cerr << "WARNING: The number of active phases specified by the deck ("
-                  << numDeckPhases << ") is smaller than the number of compiled-in phases ("
-                  << numPhases << "). This usually results in a significant "
-                  << "performance degradation compared to using a specialized simulator."  << std::endl;
-    else if (numDeckPhases < numPhases)
-        throw std::runtime_error("The deck enables "+std::to_string(numDeckPhases)+" phases "
-                                 "while this simulator can only handle "+
-                                 std::to_string(numPhases)+".");
-
-    // make sure that the correct phases are active
-    if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx) && !indicesOilEnabled)
-        throw std::runtime_error("The deck enables oil, but this simulator cannot handle it.");
-    if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx) && !indicesGasEnabled)
-        throw std::runtime_error("The deck enables gas, but this simulator cannot handle it.");
-    if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx) && !indicesWaterEnabled)
-        throw std::runtime_error("The deck enables water, but this simulator cannot handle it.");
-    // the opposite cases should be fine (albeit a bit slower than what's possible)
-}
-
 template<class GridView, class FluidSystem, class Scalar>
 void EclGenericProblem<GridView,FluidSystem,Scalar>::
 initFluidSystem_()
@@ -781,6 +800,31 @@ initDRSDT_(size_t numDof,
     }
 }
 
+template<class GridView, class FluidSystem, class Scalar>
+bool EclGenericProblem<GridView,FluidSystem,Scalar>::
+operator==(const EclGenericProblem& rhs) const
+{
+    return this->maxOilSaturation_ == rhs.maxOilSaturation_ &&
+           this->maxPolymerAdsorption_ == rhs.maxPolymerAdsorption_ &&
+           this->maxWaterSaturation_ == rhs.maxWaterSaturation_ &&
+           this->minOilPressure_ == rhs.minOilPressure_ &&
+           this->overburdenPressure_ == rhs.overburdenPressure_ &&
+           this->polymerConcentration_ == rhs.polymerConcentration_ &&
+           this->polymerMoleWeight_ == rhs.polymerMoleWeight_ &&
+           this->solventSaturation_ == rhs.solventSaturation_ &&
+           this->microbialConcentration_ == rhs.microbialConcentration_ &&
+           this->oxygenConcentration_ == rhs.oxygenConcentration_ &&
+           this->ureaConcentration_ == rhs.ureaConcentration_ &&
+           this->biofilmConcentration_ == rhs.biofilmConcentration_ &&
+           this->calciteConcentration_ == rhs.calciteConcentration_ &&
+           this->lastRv_ == rhs.lastRv_ &&
+           this->maxDRv_ == rhs.maxDRv_ &&
+           this->convectiveDrs_ == rhs.convectiveDrs_ &&
+           this->lastRs_ == rhs.lastRs_ &&
+           this->maxDRs_ == rhs.maxDRs_ &&
+           this->dRsDtOnlyFreeGas_ == rhs.dRsDtOnlyFreeGas_;
+}
+
 #if HAVE_DUNE_FEM
 template class EclGenericProblem<Dune::GridView<Dune::Fem::GridPart2GridViewTraits<Dune::Fem::AdaptiveLeafGridPart<Dune::CpGrid, Dune::PartitionIteratorType(4), false>>>,
                                  BlackOilFluidSystem<double,BlackOilDefaultIndexTraits>,
