@@ -24,9 +24,6 @@
 #include <config.h>
 #include <ebos/eclgenericthresholdpressure.hh>
 
-#include <opm/material/densead/Evaluation.hpp>
-#include <opm/material/densead/Math.hpp>
-#include <opm/input/eclipse/Deck/Deck.hpp>
 #include <opm/input/eclipse/EclipseState/EclipseState.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/FieldPropsManager.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/Eqldims.hpp>
@@ -49,6 +46,8 @@
 #include <ebos/femcpgridcompat.hh>
 #endif // HAVE_DUNE_FEM
 
+#include <fmt/format.h>
+
 #include <algorithm>
 #include <cassert>
 #include <stdexcept>
@@ -60,15 +59,11 @@ EclGenericThresholdPressure<Grid,GridView,ElementMapper,Scalar>::
 EclGenericThresholdPressure(const CartesianIndexMapper& cartMapper,
                             const GridView& gridView,
                             const ElementMapper& elementMapper,
-                            const EclipseState& eclState,
-                            const Deck& deck,
-                            bool enableExperiments)
+                            const EclipseState& eclState)
     : cartMapper_(cartMapper)
     , gridView_(gridView)
     , elementMapper_(elementMapper)
     , eclState_(eclState)
-    , deck_(deck)
-    , enableExperiments_(enableExperiments)
 {
 }
 
@@ -79,28 +74,26 @@ thresholdPressure(int elem1Idx, int elem2Idx) const
     if (!enableThresholdPressure_)
         return 0.0;
 
-    if (enableExperiments_) {
-        // threshold pressure accross faults
-        if (!thpresftValues_.empty()) {
-            int cartElem1Idx = cartMapper_.cartesianIndex(elem1Idx);
-            int cartElem2Idx = cartMapper_.cartesianIndex(elem2Idx);
+    // threshold pressure accross faults
+    if (!thpresftValues_.empty()) {
+        int cartElem1Idx = cartMapper_.cartesianIndex(elem1Idx);
+        int cartElem2Idx = cartMapper_.cartesianIndex(elem2Idx);
 
-            assert(0 <= cartElem1Idx && static_cast<int>(cartElemFaultIdx_.size()) > cartElem1Idx);
-            assert(0 <= cartElem2Idx && static_cast<int>(cartElemFaultIdx_.size()) > cartElem2Idx);
+        assert(0 <= cartElem1Idx && static_cast<int>(cartElemFaultIdx_.size()) > cartElem1Idx);
+        assert(0 <= cartElem2Idx && static_cast<int>(cartElemFaultIdx_.size()) > cartElem2Idx);
 
-            int fault1Idx = cartElemFaultIdx_[cartElem1Idx];
-            int fault2Idx = cartElemFaultIdx_[cartElem2Idx];
-            if (fault1Idx != -1 && fault1Idx == fault2Idx)
-                // inside a fault there's no threshold pressure, even accross EQUIL
-                // regions.
-                return 0.0;
-            if (fault1Idx != fault2Idx) {
-                // TODO: which value if a cell is part of multiple faults? we take
-                // the maximum here.
-                Scalar val1 = (fault1Idx >= 0) ? thpresftValues_[fault1Idx] : 0.0;
-                Scalar val2 = (fault2Idx >= 0) ? thpresftValues_[fault2Idx] : 0.0;
-                return std::max(val1, val2);
-            }
+        int fault1Idx = cartElemFaultIdx_[cartElem1Idx];
+        int fault2Idx = cartElemFaultIdx_[cartElem2Idx];
+        if (fault1Idx != -1 && fault1Idx == fault2Idx)
+            // inside a fault there's no threshold pressure, even accross EQUIL
+            // regions.
+            return 0.0;
+        if (fault1Idx != fault2Idx) {
+            // TODO: which value if a cell is part of multiple faults? we take
+            // the maximum here.
+            Scalar val1 = (fault1Idx >= 0) ? thpresftValues_[fault1Idx] : 0.0;
+            Scalar val2 = (fault2Idx >= 0) ? thpresftValues_[fault2Idx] : 0.0;
+            return std::max(val1, val2);
         }
     }
 
@@ -199,44 +192,109 @@ applyExplicitThresholdPressures_()
         }
     }
 
-    if (enableExperiments_) {
-        // apply threshold pressures accross faults (experimental!)
-        if (deck_.hasKeyword("THPRESFT"))
-            extractThpresft_(deck_["THPRESFT"].back());
-    }
+    // apply threshold pressures across faults
+    if (thpres.ftSize() > 0)
+        configureThpresft_();
 }
 
 template<class Grid, class GridView, class ElementMapper, class Scalar>
 void EclGenericThresholdPressure<Grid,GridView,ElementMapper,Scalar>::
-extractThpresft_(const DeckKeyword& thpresftKeyword)
+configureThpresft_()
 {
     // retrieve the faults collection.
     const FaultCollection& faults = eclState_.getFaults();
+    const SimulationConfig& simConfig = eclState_.getSimulationConfig();
+    const auto& thpres = simConfig.getThresholdPressure();
 
-    // extract the multipliers from the deck keyword
+    // extract the multipliers
     int numFaults = faults.size();
     int numCartesianElem = eclState_.getInputGrid().getCartesianSize();
     thpresftValues_.resize(numFaults, -1.0);
     cartElemFaultIdx_.resize(numCartesianElem, -1);
-    for (size_t recordIdx = 0; recordIdx < thpresftKeyword.size(); ++ recordIdx) {
-        const DeckRecord& record = thpresftKeyword.getRecord(recordIdx);
+    for (size_t faultIdx = 0; faultIdx < faults.size(); faultIdx++) {
+        auto& fault = faults.getFault(faultIdx);
+        thpresftValues_[faultIdx] = thpres.getThresholdPressureFault(faultIdx);
+        for (const FaultFace& face : fault)
+            // "face" is a misnomer because the object describes a set of cell
+            // indices, but we go with the conventions of the parser here...
+            for (size_t cartElemIdx : face)
+                cartElemFaultIdx_[cartElemIdx] = faultIdx;
+    }
+}
 
-        const std::string& faultName = record.getItem("FAULT_NAME").getTrimmedString(0);
-        Scalar thpresValue = record.getItem("VALUE").getSIDouble(0);
+template<class Grid, class GridView, class ElementMapper, class Scalar>
+std::vector<Scalar>
+EclGenericThresholdPressure<Grid,GridView,ElementMapper,Scalar>::
+getRestartVector() const
+{
+    if (!enableThresholdPressure_)
+        return {};
 
-        for (size_t faultIdx = 0; faultIdx < faults.size(); faultIdx++) {
-            auto& fault = faults.getFault(faultIdx);
-            if (fault.getName() != faultName)
-                continue;
+    std::vector<Scalar> result(numEquilRegions_ * numEquilRegions_, 0.0);
+    const auto& simConfig = eclState_.getSimulationConfig();
+    const auto& thpres = simConfig.getThresholdPressure();
 
-            thpresftValues_[faultIdx] = thpresValue;
-            for (const FaultFace& face: fault)
-                // "face" is a misnomer because the object describes a set of cell
-                // indices, but we go with the conventions of the parser here...
-                for (size_t cartElemIdx: face)
-                    cartElemFaultIdx_[cartElemIdx] = faultIdx;
+    std::size_t idx = 0;
+    for (unsigned j = 1; j <= numEquilRegions_; ++j) {
+        for (unsigned i = 1; i <= numEquilRegions_; ++i, ++idx) {
+            if (thpres.hasRegionBarrier(i, j)) {
+                if (thpres.hasThresholdPressure(i, j)) {
+                    result[idx] = thpres.getThresholdPressure(i, j);
+                } else {
+                    result[idx] = this->thpresDefault_[idx];
+                }
+            }
         }
     }
+
+    return result;
+}
+
+template<class Grid, class GridView, class ElementMapper, class Scalar>
+void
+EclGenericThresholdPressure<Grid,GridView,ElementMapper,Scalar>::
+logPressures()
+{
+    if (!enableThresholdPressure_)
+        return;
+
+    auto lineFormat = [this](unsigned i, unsigned j, double val)
+    {
+        const auto& units = eclState_.getUnits();
+        return fmt::format("{:4}{:>6}{:23}{:>6}{:24}{:>11.07}{:7}{}\n",
+                           " ", i,
+                           " ", j,
+                           " ", units.from_si(UnitSystem::measure::pressure, val),
+                           " ", units.name(UnitSystem::measure::pressure));
+    };
+    auto lineFormatS = [](auto s1, auto s2, auto s3)
+    {
+        return fmt::format("{:4}{:^16}{:13}{:^9}{:21}{:^18}\n",
+                           " ", s1, " ", s2, " ", s3);
+    };
+
+    std::string str = "\nLIST OF ALL NON-ZERO THRESHOLD PRESSURES\n"
+                      "----------------------------------------\n"
+                      "\n";
+    str += lineFormatS("FLOW FROM REGION", "TO REGION", "THRESHOLD PRESSURE");
+    str += lineFormatS(std::string(16, '-'), std::string(9, '-'), std::string(18, '-'));
+    const auto& simConfig = eclState_.getSimulationConfig();
+    const auto& thpres = simConfig.getThresholdPressure();
+
+    for (unsigned i = 1; i <= numEquilRegions_; ++i) {
+        for (unsigned j = (thpres.irreversible() ? 1 : i); j <= numEquilRegions_; ++j) {
+            if (thpres.hasRegionBarrier(i, j)) {
+                if (thpres.hasThresholdPressure(i, j)) {
+                    str += lineFormat(i, j, thpres.getThresholdPressure(j, i));
+                } else {
+                    std::size_t idx = (j - 1) * numEquilRegions_ + (i - 1);
+                    str += lineFormat(i, j, this->thpresDefault_[idx]);
+                }
+            }
+        }
+    }
+    str += lineFormatS(std::string(16, '-'), std::string(9, '-'), std::string(18, '-'));
+    OpmLog::note(str);
 }
 
 #if HAVE_DUNE_FEM
