@@ -43,7 +43,10 @@
 #include <opm/input/eclipse/Schedule/Action/State.hpp>
 #include <opm/input/eclipse/Schedule/Schedule.hpp>
 #include <opm/input/eclipse/Schedule/SummaryState.hpp>
+#include <opm/input/eclipse/Schedule/UDQ/UDQConfig.hpp>
 #include <opm/input/eclipse/Schedule/UDQ/UDQState.hpp>
+#include <opm/input/eclipse/Schedule/Well/PAvgCalculatorCollection.hpp>
+#include <opm/input/eclipse/Schedule/Well/WellMatcher.hpp>
 #include <opm/input/eclipse/Units/UnitSystem.hpp>
 
 #include <dune/grid/common/mcmgmapper.hh>
@@ -260,25 +263,32 @@ writeInit(const std::function<unsigned int(unsigned int)>& map)
         auto cartMap = cartesianToCompressed(equilGrid_->size(0), UgGridHelpers::globalCell(*equilGrid_));
         eclIO_->writeInitial(computeTrans_(cartMap, map), integerVectors, exportNncStructure_(cartMap, map));
     }
+#if HAVE_MPI
+    if (collectToIORank_.isParallel()) {
+        const auto& comm = grid_.comm();
+        Opm::EclMpiSerializer ser(comm);
+        ser.broadcast(outputNnc_);
+    }
+#endif
 }
 
 template<class Grid, class EquilGrid, class GridView, class ElementMapper, class Scalar>
 data::Solution EclGenericWriter<Grid,EquilGrid,GridView,ElementMapper,Scalar>::
-computeTrans_(const std::unordered_map<int,int>& cartesianToActive, const std::function<unsigned int(unsigned int)>& map) const
+computeTrans_(const std::unordered_map<int,int>& cartesianToActive,
+              const std::function<unsigned int(unsigned int)>& map) const
 {
     const auto& cartMapper = *equilCartMapper_;
     const auto& cartDims = cartMapper.cartesianDimensions();
-    const int globalSize = cartDims[0]*cartDims[1]*cartDims[2];
+    const int globalSize = cartDims[0] * cartDims[1] * cartDims[2];
 
-    data::CellData tranx = {UnitSystem::measure::transmissibility, std::vector<double>(globalSize), data::TargetType::INIT};
-    data::CellData trany = {UnitSystem::measure::transmissibility, std::vector<double>(globalSize), data::TargetType::INIT};
-    data::CellData tranz = {UnitSystem::measure::transmissibility, std::vector<double>(globalSize), data::TargetType::INIT};
+    auto tranx = data::CellData {
+        UnitSystem::measure::transmissibility,
+        std::vector<double>(globalSize, 0.0),
+        data::TargetType::INIT
+    };
 
-    for (size_t i = 0; i < tranx.data.size(); ++i) {
-        tranx.data[0] = 0.0;
-        trany.data[0] = 0.0;
-        tranz.data[0] = 0.0;
-    }
+    auto trany = tranx;
+    auto tranz = tranx;
 
     using GlobalGridView = typename EquilGrid::LeafGridView;
     const GlobalGridView& globalGridView = equilGrid_->leafGridView();
@@ -326,9 +336,11 @@ computeTrans_(const std::unordered_map<int,int>& cartesianToActive, const std::f
         }
     }
 
-    return {{"TRANX", tranx},
-            {"TRANY", trany},
-            {"TRANZ", tranz}};
+    return {
+        {"TRANX", tranx},
+        {"TRANY", trany},
+        {"TRANZ", tranz},
+    };
 }
 
 template<class Grid, class EquilGrid, class GridView, class ElementMapper, class Scalar>
@@ -339,7 +351,6 @@ exportNncStructure_(const std::unordered_map<int,int>& cartesianToActive, const
     std::size_t ny = eclState_.getInputGrid().getNY();
     auto nncData = eclState_.getInputNNC().input();
     const auto& unitSystem = eclState_.getDeckUnitSystem();
-    std::vector<NNCdata> outputNnc;
     std::size_t index = 0;
 
     for( const auto& entry : nncData ) {
@@ -352,7 +363,7 @@ exportNncStructure_(const std::unordered_map<int,int>& cartesianToActive, const
             auto tt = unitSystem.from_si(UnitSystem::measure::transmissibility, entry.trans);
             // Eclipse ignores NNCs (with EDITNNC applied) that are small. Seems like the threshold is 1.0e-6
             if ( tt >= 1.0e-6 )
-                outputNnc.emplace_back(entry.cell1, entry.cell2, entry.trans);
+                outputNnc_.emplace_back(entry.cell1, entry.cell2, entry.trans);
         }
         ++index;
     }
@@ -409,11 +420,11 @@ exportNncStructure_(const std::unordered_map<int,int>& cartesianToActive, const
                 // to zero when setting up the simulator. These will be ignored here, too.
                 auto tt = unitSystem.from_si(UnitSystem::measure::transmissibility, std::abs(t));
                 if ( tt > 1e-12 )
-                    outputNnc.push_back({cc1, cc2, t});
+                    outputNnc_.push_back({cc1, cc2, t});
             }
         }
     }
-    return outputNnc;
+    return outputNnc_;
 }
 
 template<class Grid, class EquilGrid, class GridView, class ElementMapper, class Scalar>
@@ -431,7 +442,11 @@ doWriteOutput(const int                     reportStepNum,
               const std::vector<Scalar>& thresholdPressure,
               Scalar curTime,
               Scalar nextStepSize,
-              bool doublePrecision)
+              bool doublePrecision,
+              bool isFlowsn,
+              std::array<std::pair<std::string, std::pair<std::vector<int>, std::vector<double>>>, 3>&& flowsn,
+              bool isFloresn,
+              std::array<std::pair<std::string, std::pair<std::vector<int>, std::vector<double>>>, 3>&& floresn)
 {
     const auto isParallel = this->collectToIORank_.isParallel();
     const bool needsReordering = this->collectToIORank_.doesNeedReordering();
@@ -460,6 +475,29 @@ doWriteOutput(const int                     reportStepNum,
         restartValue.addExtra("OPMEXTRA", std::vector<double>(1, nextStepSize));
     }
 
+    // Add nnc flows and flores.
+    if (isFlowsn) {
+        const auto flowsn_global = isParallel ? this->collectToIORank_.globalFlowsn() : std::move(flowsn);
+        for (const auto& flows : flowsn_global) { 
+            if (flows.first.empty())
+                continue;
+            if (flows.first == "FLOGASN+") {
+                restartValue.addExtra(flows.first, UnitSystem::measure::gas_surface_rate, flows.second.second);
+            }
+            else {
+                restartValue.addExtra(flows.first, UnitSystem::measure::liquid_surface_rate, flows.second.second);
+            }
+        }
+    }
+    if (isFloresn) {
+        const auto floresn_global = isParallel ? this->collectToIORank_.globalFloresn() : std::move(floresn);
+        for (const auto& flores : floresn_global) {
+            if (flores.first.empty())
+                continue;
+            restartValue.addExtra(flores.first, UnitSystem::measure::rate, flores.second.second);
+        }
+    }
+
     // first, create a tasklet to write the data for the current time
     // step to disk
     auto eclWriteTasklet = std::make_shared<EclWriteTasklet>(
