@@ -22,6 +22,9 @@
 #ifndef OPM_AQUIFERINTERFACE_HEADER_INCLUDED
 #define OPM_AQUIFERINTERFACE_HEADER_INCLUDED
 
+#include <opm/models/common/multiphasebaseproperties.hh>
+#include <opm/models/discretization/common/fvbaseproperties.hh>
+
 #include <opm/output/data/Aquifer.hpp>
 
 namespace Opm
@@ -79,7 +82,8 @@ protected:
 
     int phaseIdx_() const
     {
-        if (co2store_())
+        // If OIL is used to model brine the aquifer should do the same
+        if (co2store_() && FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx))
             return FluidSystem::oilPhaseIdx;
 
         return FluidSystem::waterPhaseIdx;
