@@ -40,7 +40,7 @@
 #ifdef USE_POLYHEDRALGRID
 #include <opm/grid/polyhedralgrid.hh>
 #endif
-#if HAVE_DUNE_ALUGRID
+#if USE_ALUGRID
 #include <dune/alugrid/grid.hh>
 #endif
 
@@ -48,6 +48,7 @@
 
 #include <vector>
 #include <type_traits>
+#include <string_view>
 
 namespace Opm
 {
@@ -71,7 +72,7 @@ class SupportsFaceTag<Dune::PolyhedralGrid<3, 3>>
 {};
 #endif
 
-#if HAVE_DUNE_ALUGRID
+#if USE_ALUGRID
 template<>
 class SupportsFaceTag<Dune::ALUGrid<3, 3, Dune::cube, Dune::nonconforming>>
     : public std::bool_constant<true>
@@ -112,6 +113,9 @@ public:
     template <class Restarter>
     void deserialize(Restarter& res);
 
+    template<class Serializer>
+    void serializeOp(Serializer& serializer);
+
 protected:
     // ---------      Types      ---------
     using ElementContext = GetPropType<TypeTag, Properties::ElementContext>;
@@ -119,10 +123,23 @@ protected:
 
     Simulator& simulator_;
 
+    // TODO: possibly better to use unorder_map here for aquifers
     std::vector<std::unique_ptr<AquiferInterface<TypeTag>>> aquifers;
 
-    // This initialization function is used to connect the parser objects with the ones needed by AquiferCarterTracy
+    // This initialization function is used to connect the parser objects
+    // with the ones needed by AquiferCarterTracy
     void init();
+
+private:
+    void createDynamicAquifers(const int episode_index);
+
+    void initializeStaticAquifers();
+
+    template <typename AquiferType, typename AquiferData>
+    std::unique_ptr<AquiferType>
+    createAnalyticAquiferPointer(const AquiferData& aqData,
+                                 const int          aquiferID,
+                                 std::string_view   aqType) const;
 };
 
 
