@@ -27,47 +27,44 @@
 #include <ebos/eclproblem.hh>
 #include <opm/models/utils/start.hh>
 
-#include <opm/simulators/timestepping/AdaptiveTimeSteppingEbos.hpp>
-
-#include <opm/simulators/flow/NonlinearSolverEbos.hpp>
-#include <opm/simulators/flow/BlackoilModelParametersEbos.hpp>
-#include <opm/simulators/wells/BlackoilWellModel.hpp>
-#include <opm/simulators/aquifers/BlackoilAquiferModel.hpp>
-#include <opm/simulators/wells/WellConnectionAuxiliaryModule.hpp>
-#include <opm/simulators/flow/countGlobalCells.hpp>
-#include <opm/simulators/utils/DeferredLoggingErrorHelpers.hpp>
-
-#include <opm/grid/UnstructuredGrid.h>
-#include <opm/simulators/timestepping/SimulatorReport.hpp>
-#include <opm/simulators/linalg/ParallelIstlInformation.hpp>
-#include <opm/core/props/phaseUsageFromDeck.hpp>
 #include <opm/common/ErrorMacros.hpp>
 #include <opm/common/Exceptions.hpp>
 #include <opm/common/OpmLog/OpmLog.hpp>
-#include <opm/input/eclipse/Units/Units.hpp>
-#include <opm/simulators/timestepping/SimulatorTimer.hpp>
-#include <opm/common/utility/parameters/ParameterGroup.hpp>
+
+#include <opm/core/props/phaseUsageFromDeck.hpp>
+
+#include <opm/grid/UnstructuredGrid.h>
+
 #include <opm/input/eclipse/EclipseState/EclipseState.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/TableManager.hpp>
 
+#include <opm/simulators/aquifers/BlackoilAquiferModel.hpp>
+#include <opm/simulators/flow/countGlobalCells.hpp>
+#include <opm/simulators/flow/NonlinearSolverEbos.hpp>
+#include <opm/simulators/flow/BlackoilModelParametersEbos.hpp>
 #include <opm/simulators/linalg/ISTLSolverEbos.hpp>
+#include <opm/simulators/timestepping/AdaptiveTimeSteppingEbos.hpp>
+#include <opm/simulators/timestepping/ConvergenceReport.hpp>
+#include <opm/simulators/timestepping/SimulatorReport.hpp>
+#include <opm/simulators/timestepping/SimulatorTimer.hpp>
+
+#include <opm/simulators/utils/DeferredLoggingErrorHelpers.hpp>
+#include <opm/simulators/wells/BlackoilWellModel.hpp>
+#include <opm/simulators/wells/WellConnectionAuxiliaryModule.hpp>
 
 #include <dune/istl/owneroverlapcopy.hh>
-#if DUNE_VERSION_NEWER(DUNE_COMMON, 2, 7)
 #include <dune/common/parallel/communication.hh>
-#else
-#include <dune/common/parallel/collectivecommunication.hh>
-#endif
 #include <dune/common/timer.hh>
 #include <dune/common/unused.hh>
 
+#include <algorithm>
 #include <cassert>
 #include <cmath>
-#include <iostream>
 #include <iomanip>
+#include <iostream>
 #include <limits>
+#include <utility>
 #include <vector>
-#include <algorithm>
 
 namespace Opm::Properties {
 
@@ -204,6 +201,70 @@ namespace Opm {
         typedef Dune::BlockVector<VectorBlockType>      BVector;
 
         typedef ISTLSolverEbos<TypeTag> ISTLSolverType;
+
+        class ComponentName
+        {
+        public:
+            ComponentName()
+                : names_(numEq)
+            {
+                for (unsigned phaseIdx = 0; phaseIdx < FluidSystem::numPhases; ++phaseIdx) {
+                    if (!FluidSystem::phaseIsActive(phaseIdx)) {
+                        continue;
+                    }
+
+                    const unsigned canonicalCompIdx = FluidSystem::solventComponentIndex(phaseIdx);
+                    names_[Indices::canonicalToActiveComponentIndex(canonicalCompIdx)]
+                        = FluidSystem::componentName(canonicalCompIdx);
+                }
+
+                if constexpr (has_solvent_) {
+                    names_[solventSaturationIdx] = "Solvent";
+                }
+
+                if constexpr (has_extbo_) {
+                    names_[zFractionIdx] = "ZFraction";
+                }
+
+                if constexpr (has_polymer_) {
+                    names_[polymerConcentrationIdx] = "Polymer";
+                }
+
+                if constexpr (has_polymermw_) {
+                    assert(has_polymer_);
+                    names_[polymerMoleWeightIdx] = "MolecularWeightP";
+                }
+
+                if constexpr (has_energy_) {
+                    names_[temperatureIdx] = "Energy";
+                }
+
+                if constexpr (has_foam_) {
+                    names_[foamConcentrationIdx] = "Foam";
+                }
+
+                if constexpr (has_brine_) {
+                    names_[saltConcentrationIdx] = "Brine";
+                }
+
+                if constexpr (has_micp_) {
+                    names_[microbialConcentrationIdx] = "Microbes";
+                    names_[oxygenConcentrationIdx] = "Oxygen";
+                    names_[ureaConcentrationIdx] = "Urea";
+                    names_[biofilmConcentrationIdx] = "Biofilm";
+                    names_[calciteConcentrationIdx] = "Calcite";
+                }
+            }
+
+            const std::string& name(const int compIdx) const
+            {
+                return this->names_[compIdx];
+            }
+
+        private:
+            std::vector<std::string> names_{};
+        };
+
         //typedef typename SolutionVector :: value_type            PrimaryVariables ;
 
         // ---------  Public methods  ---------
@@ -229,7 +290,7 @@ namespace Opm {
         , well_model_ (well_model)
         , terminal_output_ (terminal_output)
         , current_relaxation_(1.0)
-        , dx_old_(UgGridHelpers::numCells(grid_))
+        , dx_old_(ebosSimulator_.model().numGridDof())
         {
             // compute global sum of number of cells
             global_nc_ = detail::countGlobalCells(grid_);
@@ -334,9 +395,9 @@ namespace Opm {
 
                 // Throw if any NaN or too large residual found.
                 if (severity == ConvergenceReport::Severity::NotANumber) {
-                    OPM_THROW(NumericalIssue, "NaN residual found!");
+                    OPM_THROW(NumericalProblem, "NaN residual found!");
                 } else if (severity == ConvergenceReport::Severity::TooLarge) {
-                    OPM_THROW_NOLOG(NumericalIssue, "Too large residual found!");
+                    OPM_THROW_NOLOG(NumericalProblem, "Too large residual found!");
                 }
             }
             report.update_time += perfTimer.stop();
@@ -350,7 +411,7 @@ namespace Opm {
                 //residual_.singlePrecision = (unit::convert::to(dt, unit::day) < 20.) ;
 
                 // Compute the nonlinear update.
-                const int nc = UgGridHelpers::numCells(grid_);
+                unsigned nc = ebosSimulator_.model().numGridDof();
                 BVector x(nc);
 
                 // Solve the linear system.
@@ -455,13 +516,7 @@ namespace Opm {
 
             const auto& elemMapper = ebosSimulator_.model().elementMapper();
             const auto& gridView = ebosSimulator_.gridView();
-            auto elemIt = gridView.template begin</*codim=*/0>();
-            const auto& elemEndIt = gridView.template end</*codim=*/0>();
-            for (; elemIt != elemEndIt; ++elemIt) {
-                const auto& elem = *elemIt;
-                if (elem.partitionType() != Dune::InteriorEntity)
-                    continue;
-
+            for (const auto& elem : elements(gridView, Dune::Partitions::interior)) {
                 unsigned globalElemIdx = elemMapper.index(elem);
                 const auto& priVarsNew = ebosSimulator_.model().solution(/*timeIdx=*/0)[globalElemIdx];
 
@@ -470,14 +525,16 @@ namespace Opm {
 
                 Scalar saturationsNew[FluidSystem::numPhases] = { 0.0 };
                 Scalar oilSaturationNew = 1.0;
-                if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx) && FluidSystem::numActivePhases() > 1) {
-                    saturationsNew[FluidSystem::waterPhaseIdx] = priVarsNew[Indices::waterSaturationIdx];
+                if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx) &&
+                    FluidSystem::numActivePhases() > 1 &&
+                    priVarsNew.primaryVarsMeaningWater() == PrimaryVariables::WaterMeaning::Sw) {
+                    saturationsNew[FluidSystem::waterPhaseIdx] = priVarsNew[Indices::waterSwitchIdx];
                     oilSaturationNew -= saturationsNew[FluidSystem::waterPhaseIdx];
                 }
 
                 if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx) &&
                     FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx) &&
-                    priVarsNew.primaryVarsMeaning() == PrimaryVariables::Sw_po_Sg) {
+                    priVarsNew.primaryVarsMeaningGas() == PrimaryVariables::GasMeaning::Sg) {
                     assert(Indices::compositionSwitchIdx >= 0 );
                     saturationsNew[FluidSystem::gasPhaseIdx] = priVarsNew[Indices::compositionSwitchIdx];
                     oilSaturationNew -= saturationsNew[FluidSystem::gasPhaseIdx];
@@ -501,14 +558,12 @@ namespace Opm {
                 resultDenom += pressureNew*pressureNew;
 
                 if (FluidSystem::numActivePhases() > 1) {
-                    if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)) {
-                        saturationsOld[FluidSystem::waterPhaseIdx] = priVarsOld[Indices::waterSaturationIdx];
+                    if (priVarsOld.primaryVarsMeaningWater() == PrimaryVariables::WaterMeaning::Sw) {
+                        saturationsOld[FluidSystem::waterPhaseIdx] = priVarsOld[Indices::waterSwitchIdx];
                         oilSaturationOld -= saturationsOld[FluidSystem::waterPhaseIdx];
                     }
 
-                    if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx) &&
-                        FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx) &&
-                        priVarsOld.primaryVarsMeaning() == PrimaryVariables::Sw_po_Sg)
+                    if (priVarsOld.primaryVarsMeaningGas() == PrimaryVariables::GasMeaning::Sg)
                     {
                         assert(Indices::compositionSwitchIdx >= 0 );
                         saturationsOld[FluidSystem::gasPhaseIdx] = priVarsOld[Indices::compositionSwitchIdx];
@@ -573,6 +628,7 @@ namespace Opm {
         /// Apply an update to the primary variables.
         void updateSolution(const BVector& dx)
         {
+            OPM_TIMEBLOCK(updateSolution);
             auto& ebosNewtonMethod = ebosSimulator_.model().newtonMethod();
             SolutionVector& solution = ebosSimulator_.model().solution(/*timeIdx=*/0);
 
@@ -584,7 +640,11 @@ namespace Opm {
                                                     // residual
 
             // if the solution is updated, the intensive quantities need to be recalculated
-            ebosSimulator_.model().invalidateAndUpdateIntensiveQuantities(/*timeIdx=*/0);
+            {
+                OPM_TIMEBLOCK(invalidateAndUpdateIntensiveQuantities);
+                ebosSimulator_.model().invalidateAndUpdateIntensiveQuantities(/*timeIdx=*/0);
+                ebosSimulator_.problem().eclWriter()->mutableEclOutputModule().invalidateLocalData();
+            }
         }
 
         /// Return true if output to cout is wanted.
@@ -601,6 +661,7 @@ namespace Opm {
                                                        std::vector< Scalar >& maxCoeff,
                                                        std::vector< Scalar >& B_avg)
         {
+            OPM_TIMEBLOCK(convergenceReduction);
             // Compute total pore volume (use only owned entries)
             double pvSum = pvSumLocal;
             double numAquiferPvSum = numAquiferPvSumLocal;
@@ -660,6 +721,7 @@ namespace Opm {
                                     std::vector<Scalar>& maxCoeff,
                                     std::vector<Scalar>& B_avg)
         {
+            OPM_TIMEBLOCK(localConvergenceData);
             double pvSumLocal = 0.0;
             double numAquiferPvSumLocal = 0.0;
             const auto& ebosModel = ebosSimulator_.model();
@@ -669,14 +731,8 @@ namespace Opm {
 
             ElementContext elemCtx(ebosSimulator_);
             const auto& gridView = ebosSimulator().gridView();
-            const auto& elemEndIt = gridView.template end</*codim=*/0, Dune::Interior_Partition>();
             OPM_BEGIN_PARALLEL_TRY_CATCH();
-
-            for (auto elemIt = gridView.template begin</*codim=*/0, Dune::Interior_Partition>();
-                 elemIt != elemEndIt;
-                 ++elemIt)
-            {
-                const auto& elem = *elemIt;
+            for (const auto& elem : elements(gridView, Dune::Partitions::interior)) {
                 elemCtx.updatePrimaryStencil(elem);
                 elemCtx.updatePrimaryIntensiveQuantities(/*timeIdx=*/0);
                 const unsigned cell_idx = elemCtx.globalSpaceIndex(/*spaceIdx=*/0, /*timeIdx=*/0);
@@ -750,7 +806,7 @@ namespace Opm {
                 }
 
                 if constexpr (has_energy_) {
-                    B_avg[ contiEnergyEqIdx ] += 1.0;
+                    B_avg[ contiEnergyEqIdx ] += 1.0 / (4.182e1); // converting J -> RM3 (entalpy / (cp * deltaK * rho) assuming change of 1e-5K of water
                     const auto R2 = ebosResid[cell_idx][contiEnergyEqIdx];
                     R_sum[ contiEnergyEqIdx ] += R2;
                     maxCoeff[ contiEnergyEqIdx ] = std::max( maxCoeff[ contiEnergyEqIdx ], std::abs( R2 ) / pvValue );
@@ -796,6 +852,7 @@ namespace Opm {
         ///        of a numerical aquifer.
         double computeCnvErrorPv(const std::vector<Scalar>& B_avg, double dt)
         {
+            OPM_TIMEBLOCK(computeCnvErrorPv);
             double errorPV{};
             const auto& ebosModel = ebosSimulator_.model();
             const auto& ebosProblem = ebosSimulator_.problem();
@@ -805,7 +862,7 @@ namespace Opm {
 
             OPM_BEGIN_PARALLEL_TRY_CATCH();
 
-            for (const auto& elem: elements(gridView, Dune::Partitions::interiorBorder))
+            for (const auto& elem : elements(gridView, Dune::Partitions::interiorBorder))
             {
                 // Skip cells of numerical Aquifer
                 if (isNumericalAquiferCell(gridView.grid(), elem))
@@ -837,11 +894,13 @@ namespace Opm {
             return grid_.comm().sum(errorPV);
         }
 
-        ConvergenceReport getReservoirConvergence(const double dt,
+        ConvergenceReport getReservoirConvergence(const double reportTime,
+                                                  const double dt,
                                                   const int iteration,
                                                   std::vector<Scalar>& B_avg,
                                                   std::vector<Scalar>& residual_norms)
         {
+            OPM_TIMEBLOCK(getReservoirConvergence);
             typedef std::vector< Scalar > Vector;
 
             const int numComp = numEq;
@@ -859,11 +918,10 @@ namespace Opm {
             cnvErrorPvFraction /= (pvSum - numAquiferPvSum);
 
             const double tol_mb  = param_.tolerance_mb_;
-            // Default value of relaxed_max_pv_fraction_ is 1 and
-            // max_strict_iter_ is 8. Hence only iteration chooses
-            // whether to use relaxed or not.
-            // To activate only fraction use fraction below 1 and iter 0.
-            const bool use_relaxed = cnvErrorPvFraction < param_.relaxed_max_pv_fraction_ && iteration >= param_.max_strict_iter_;
+            // Default value of relaxed_max_pv_fraction_ is 0.03 and min_strict_cnv_iter_ is 0.
+            // For each iteration, we need to determine whether to use the relaxed CNV tolerance.
+            // To disable the usage of relaxed CNV tolerance, you can set the relaxed_max_pv_fraction_ to be 0.
+            const bool use_relaxed = cnvErrorPvFraction < param_.relaxed_max_pv_fraction_ && iteration >= param_.min_strict_cnv_iter_;
             const double tol_cnv = use_relaxed ? param_.tolerance_cnv_relaxed_ :  param_.tolerance_cnv_;
 
             // Finish computation
@@ -876,51 +934,8 @@ namespace Opm {
                 residual_norms.push_back(CNV[compIdx]);
             }
 
-            // Setup component names, only the first time the function is run.
-            static std::vector<std::string> compNames;
-            if (compNames.empty()) {
-                compNames.resize(numComp);
-                for (unsigned phaseIdx = 0; phaseIdx < FluidSystem::numPhases; ++phaseIdx) {
-                    if (!FluidSystem::phaseIsActive(phaseIdx)) {
-                        continue;
-                    }
-                    const unsigned canonicalCompIdx = FluidSystem::solventComponentIndex(phaseIdx);
-                    const unsigned compIdx = Indices::canonicalToActiveComponentIndex(canonicalCompIdx);
-                    compNames[compIdx] = FluidSystem::componentName(canonicalCompIdx);
-                }
-                if constexpr (has_solvent_) {
-                    compNames[solventSaturationIdx] = "Solvent";
-                }
-                if constexpr (has_extbo_) {
-                    compNames[zFractionIdx] = "ZFraction";
-                }
-                if constexpr (has_polymer_) {
-                    compNames[polymerConcentrationIdx] = "Polymer";
-                }
-                if constexpr (has_polymermw_) {
-                    assert(has_polymer_);
-                    compNames[polymerMoleWeightIdx] = "MolecularWeightP";
-                }
-                if constexpr (has_energy_) {
-                    compNames[temperatureIdx] = "Energy";
-                }
-                if constexpr (has_foam_) {
-                    compNames[foamConcentrationIdx] = "Foam";
-                }
-                if constexpr (has_brine_) {
-                    compNames[saltConcentrationIdx] = "Brine";
-                }
-                if constexpr (has_micp_) {
-                    compNames[microbialConcentrationIdx] = "Microbes";
-                    compNames[oxygenConcentrationIdx] = "Oxygen";
-                    compNames[ureaConcentrationIdx] = "Urea";
-                    compNames[biofilmConcentrationIdx] = "Biofilm";
-                    compNames[calciteConcentrationIdx] = "Calcite";
-            }
-            }
-
             // Create convergence report.
-            ConvergenceReport report;
+            ConvergenceReport report{reportTime};
             using CR = ConvergenceReport;
             for (int compIdx = 0; compIdx < numComp; ++compIdx) {
                 double res[2] = { mass_balance_residual[compIdx], CNV[compIdx] };
@@ -931,21 +946,22 @@ namespace Opm {
                     if (std::isnan(res[ii])) {
                         report.setReservoirFailed({types[ii], CR::Severity::NotANumber, compIdx});
                         if ( terminal_output_ ) {
-                            OpmLog::debug("NaN residual for " + compNames[compIdx] + " equation.");
+                            OpmLog::debug("NaN residual for " + this->compNames_.name(compIdx) + " equation.");
                         }
                     } else if (res[ii] > maxResidualAllowed()) {
                         report.setReservoirFailed({types[ii], CR::Severity::TooLarge, compIdx});
                         if ( terminal_output_ ) {
-                            OpmLog::debug("Too large residual for " + compNames[compIdx] + " equation.");
+                            OpmLog::debug("Too large residual for " + this->compNames_.name(compIdx) + " equation.");
                         }
                     } else if (res[ii] < 0.0) {
                         report.setReservoirFailed({types[ii], CR::Severity::Normal, compIdx});
                         if ( terminal_output_ ) {
-                            OpmLog::debug("Negative residual for " + compNames[compIdx] + " equation.");
+                            OpmLog::debug("Negative residual for " + this->compNames_.name(compIdx) + " equation.");
                         }
                     } else if (res[ii] > tol[ii]) {
                         report.setReservoirFailed({types[ii], CR::Severity::Normal, compIdx});
                     }
+                    report.setReservoirConvergenceMetric(types[ii], compIdx, res[ii]);
                 }
             }
 
@@ -957,12 +973,12 @@ namespace Opm {
                     std::string msg = "Iter";
                     for (int compIdx = 0; compIdx < numComp; ++compIdx) {
                         msg += "    MB(";
-                        msg += compNames[compIdx][0];
+                        msg += this->compNames_.name(compIdx)[0];
                         msg += ")  ";
                     }
                     for (int compIdx = 0; compIdx < numComp; ++compIdx) {
                         msg += "    CNV(";
-                        msg += compNames[compIdx][0];
+                        msg += this->compNames_.name(compIdx)[0];
                         msg += ") ";
                     }
                     OpmLog::debug(msg);
@@ -994,11 +1010,16 @@ namespace Opm {
                                          const int iteration,
                                          std::vector<double>& residual_norms)
         {
+            OPM_TIMEBLOCK(getConvergence);
             // Get convergence reports for reservoir and wells.
             std::vector<Scalar> B_avg(numEq, 0.0);
-            auto report = getReservoirConvergence(timer.currentStepLength(), iteration, B_avg, residual_norms);
-            report += wellModel().getWellConvergence(B_avg);
-
+            auto report = getReservoirConvergence(timer.simulationTimeElapsed(),
+                                                  timer.currentStepLength(),
+                                                  iteration, B_avg, residual_norms);
+            {
+                OPM_TIMEBLOCK(getWellConvergence);
+                report += wellModel().getWellConvergence(B_avg, /*checkWellGroupControls*/report.converged());
+            }
             return report;
         }
 
@@ -1021,6 +1042,7 @@ namespace Opm {
         std::vector<std::vector<double> >
         computeFluidInPlace(const std::vector<int>& /*fipnum*/) const
         {
+            OPM_TIMEBLOCK(computeFluidInPlace);
             //assert(true)
             //return an empty vector
             std::vector<std::vector<double> > regionValues(0, std::vector<double>(0,0.0));
@@ -1037,18 +1059,16 @@ namespace Opm {
         const SimulatorReportSingle& failureReport() const
         { return failureReport_; }
 
-        struct StepReport
-        {
-            int report_step;
-            int current_step;
-            std::vector<ConvergenceReport> report;
-        };
-
         const std::vector<StepReport>& stepReports() const
         {
             return convergence_reports_;
         }
 
+        std::vector<StepReport> getStepReportsDestructively() const
+        {
+            return std::move(this->convergence_reports_);
+        }
+
     protected:
         // ---------  Data members  ---------
 
@@ -1080,6 +1100,8 @@ namespace Opm {
         BVector dx_old_;
 
         std::vector<StepReport> convergence_reports_;
+        ComponentName compNames_{};
+
     public:
         /// return the StandardWells object
         BlackoilWellModel<TypeTag>&
@@ -1114,7 +1136,7 @@ namespace Opm {
 
         template<class G, class T>
         typename std::enable_if<!std::is_same<G,Dune::CpGrid>::value, bool>::type
-        isNumericalAquiferCell(const G& grid, const T& elem)
+        isNumericalAquiferCell(const G&, const T&)
         {
             return false;
         }
@@ -1124,6 +1146,7 @@ namespace Opm {
         double drMaxRel() const { return param_.dr_max_rel_; }
         double maxResidualAllowed() const { return param_.max_residual_allowed_; }
         double linear_solve_setup_time_;
+
     public:
         std::vector<bool> wasSwitched_;
     };
