@@ -42,27 +42,21 @@
 #include <flow/flow_ebos_onephase_energy.hpp>
 #include <flow/flow_ebos_oilwater_brine.hpp>
 #include <flow/flow_ebos_gaswater_brine.hpp>
+#include <flow/flow_ebos_gaswater_energy.hpp>
+#include <flow/flow_ebos_gaswater_dissolution.hpp>
+#include <flow/flow_ebos_gaswater_dissolution_diffuse.hpp>
 #include <flow/flow_ebos_energy.hpp>
 #include <flow/flow_ebos_oilwater_polymer.hpp>
 #include <flow/flow_ebos_oilwater_polymer_injectivity.hpp>
 #include <flow/flow_ebos_micp.hpp>
 
-#include <opm/input/eclipse/Deck/Deck.hpp>
-#include <opm/input/eclipse/Parser/ErrorGuard.hpp>
-#include <opm/input/eclipse/Parser/Parser.hpp>
-#include <opm/input/eclipse/Parser/ParseContext.hpp>
 #include <opm/input/eclipse/EclipseState/EclipseState.hpp>
-#include <opm/input/eclipse/EclipseState/checkDeck.hpp>
-#include <opm/input/eclipse/Schedule/ArrayDimChecker.hpp>
-#include <opm/input/eclipse/Schedule/UDQ/UDQState.hpp>
-#include <opm/input/eclipse/Schedule/Action/State.hpp>
-#include <opm/input/eclipse/Schedule/Well/WellTestState.hpp>
 
 #include <opm/models/utils/propertysystem.hh>
 #include <opm/models/utils/parametersystem.hh>
 
+#include <opm/simulators/flow/Banners.hpp>
 #include <opm/simulators/flow/FlowMainEbos.hpp>
-#include <opm/simulators/utils/readDeck.hpp>
 
 #if HAVE_DUNE_FEM
 #include <dune/fem/misc/mpimanager.hh>
@@ -74,17 +68,13 @@
 #include <opm/simulators/utils/ParallelEclipseState.hpp>
 #endif
 
-#if HAVE_DAMARIS
-#include <opm/simulators/utils/DamarisOutputModule.hpp>
-#endif
-
 #include <cassert>
 #include <cstdlib>
-#include <filesystem>
 #include <iostream>
 #include <memory>
 #include <stdexcept>
 #include <string>
+#include <string_view>
 #include <type_traits>
 #include <utility>
 
@@ -102,6 +92,10 @@ struct FlowEarlyBird {
 
 namespace Opm {
 
+namespace Action { class State; }
+class UDQState;
+class WellTestState;
+
 // ----------------- Main program -----------------
 template <class TypeTag>
 int flowEbosMain(int argc, char** argv, bool outputCout, bool outputFiles)
@@ -124,111 +118,23 @@ int flowEbosMain(int argc, char** argv, bool outputCout, bool outputFiles)
 class Main
 {
 public:
-    Main(int argc, char** argv) : argc_(argc), argv_(argv)  { initMPI();  }
+    Main(int argc, char** argv);
 
     // This constructor can be called from Python
-    Main(const std::string& filename)
-    {
-        setArgvArgc_(filename);
-        initMPI();
-    }
+    Main(const std::string& filename);
 
     // This constructor can be called from Python when Python has
     // already parsed a deck
-    Main(std::shared_ptr<Deck> deck,
+    Main(const std::string& filename,
          std::shared_ptr<EclipseState> eclipseState,
          std::shared_ptr<Schedule> schedule,
-         std::shared_ptr<SummaryConfig> summaryConfig)
-        : deck_{std::move(deck)}
-        , eclipseState_{std::move(eclipseState)}
-        , schedule_{std::move(schedule)}
-        , summaryConfig_{std::move(summaryConfig)}
-    {
-        setArgvArgc_(deck_->getDataFile());
-        initMPI();
-    }
+         std::shared_ptr<SummaryConfig> summaryConfig);
 
+    ~Main();
 
-    ~Main()
-    {
-#if HAVE_MPI
-        if (test_split_comm_) {
-            // Cannot use EclGenericVanguard::comm()
-            // to get world size here, as it may be
-            // a split communication at this point.
-            int world_size;
-            MPI_Comm_size(MPI_COMM_WORLD, &world_size);
-            if (world_size > 1) {
-                MPI_Comm new_comm = EclGenericVanguard::comm();
-                int result;
-                MPI_Comm_compare(MPI_COMM_WORLD, new_comm, &result);
-                assert(result == MPI_UNEQUAL);
-                MPI_Comm_free(&new_comm);
-            }
-        }
-#endif // HAVE_MPI
+    void setArgvArgc_(const std::string& filename);
 
-        EclGenericVanguard::setCommunication(nullptr);
-
-#if HAVE_DAMARIS
-        if (enableDamarisOutput_) {
-            int err;
-            if (isSimulationRank_) {
-                err = damaris_stop();
-                if (err != DAMARIS_OK) {
-                    std::cerr << "ERROR: Damaris library produced an error result for damaris_stop()" << std::endl;
-                }
-            }
-            err = damaris_finalize();
-            if (err != DAMARIS_OK) {
-                std::cerr << "ERROR: Damaris library produced an error result for damaris_finalize()" << std::endl;
-            }
-        }
-#endif // HAVE_DAMARIS
-
-#if HAVE_MPI && !HAVE_DUNE_FEM
-        MPI_Finalize();
-#endif
-    }
-
-    void setArgvArgc_(const std::string& filename)
-    {
-        this->deckFilename_ = filename;
-        this->flowProgName_ = "flow";
-
-        this->argc_ = 2;
-        this->saveArgs_[0] = const_cast<char *>(this->flowProgName_.c_str());
-        this->saveArgs_[1] = const_cast<char *>(this->deckFilename_.c_str());
-
-        // Note: argv[argc] must exist and be nullptr
-        assert ((sizeof this->saveArgs_) > (this->argc_ * sizeof this->saveArgs_[0]));
-        this->saveArgs_[this->argc_] = nullptr;
-
-        this->argv_ = this->saveArgs_;
-    }
-
-    void initMPI()
-    {
-#if HAVE_DUNE_FEM
-        Dune::Fem::MPIManager::initialize(argc_, argv_);
-#elif HAVE_MPI
-        MPI_Init(&argc_, &argv_);
-#endif
-        EclGenericVanguard::setCommunication(std::make_unique<Parallel::Communication>());
-
-        handleTestSplitCommunicatorCmdLine_();
-
-#if HAVE_MPI
-        if (test_split_comm_ && EclGenericVanguard::comm().size() > 1) {
-            int world_rank = EclGenericVanguard::comm().rank();
-            int color = (world_rank == 0);
-            MPI_Comm new_comm;
-            MPI_Comm_split(EclGenericVanguard::comm(), color, world_rank, &new_comm);
-            isSimulationRank_ = (world_rank > 0);
-            EclGenericVanguard::setCommunication(std::make_unique<Parallel::Communication>(new_comm));
-        }
-#endif // HAVE_MPI
-    }
+    void initMPI();
 
     int runDynamic()
     {
@@ -266,15 +172,7 @@ public:
         if (initialize_<Properties::TTag::FlowEarlyBird>(exitCode)) {
             // TODO: check that this deck really represents a blackoil
             // case. E.g. check that number of phases == 3
-            EclGenericVanguard::setParams(
-                setupTime_,
-                deck_,
-                eclipseState_,
-                schedule_,
-                std::move(udqState_),
-                std::move(this->actionState_),
-                std::move(this->wtestState_),
-                summaryConfig_);
+            this->setupVanguard();
             return flowEbosBlackoilTpfaMainInit(
                 argc_, argv_, outputCout_, outputFiles_);
         } else {
@@ -289,14 +187,7 @@ private:
         const auto& rspec = this->eclipseState_->runspec();
         const auto& phases = rspec.phases();
 
-        EclGenericVanguard::setParams(this->setupTime_,
-                                      this->deck_,
-                                      this->eclipseState_,
-                                      this->schedule_,
-                                      std::move(this->udqState_),
-                                      std::move(this->actionState_),
-                                      std::move(this->wtestState_),
-                                      this->summaryConfig_);
+        this->setupVanguard();
 
         // run the actual simulator
         //
@@ -310,12 +201,12 @@ private:
         }
 
         // water-only case
-        else if(phases.size() == 1 && phases.active(Phase::WATER) && !thermal) {
+        else if (phases.size() == 1 && phases.active(Phase::WATER) && !thermal) {
             return this->runWaterOnly(phases);
         }
 
         // water-only case with energy
-        else if(phases.size() == 2 && phases.active(Phase::WATER) && thermal) {
+        else if (phases.size() == 2 && phases.active(Phase::WATER) && thermal) {
             return this->runWaterOnlyEnergy(phases);
         }
 
@@ -373,14 +264,7 @@ private:
     template <class TypeTag>
     int dispatchStatic_()
     {
-        EclGenericVanguard::setParams(this->setupTime_,
-                                      this->deck_,
-                                      this->eclipseState_,
-                                      this->schedule_,
-                                      std::move(this->udqState_),
-                                      std::move(this->actionState_),
-                                      std::move(this->wtestState_),
-                                      this->summaryConfig_);
+        this->setupVanguard();
         return flowEbosMain<TypeTag>(argc_, argv_, outputCout_, outputFiles_);
     }
 
@@ -396,7 +280,7 @@ private:
         Dune::Timer externalSetupTimer;
         externalSetupTimer.start();
 
-        handleVersionCmdLine_(argc_, argv_);
+        handleVersionCmdLine_(argc_, argv_, Opm::moduleVersionName());
 
         // we always want to use the default locale, and thus spare us the trouble
         // with incorrect locale settings.
@@ -439,32 +323,12 @@ private:
 #if HAVE_DAMARIS
         enableDamarisOutput_ = EWOMS_GET_PARAM(PreTypeTag, bool, EnableDamarisOutput);
         if (enableDamarisOutput_) {
-            if (!outputDir.empty()) {
-                ensureOutputDirExists(outputDir);
-            }
-            // By default EnableDamarisOutputCollective is true so all simulation results will
-            // be written into one single file for each iteration using Parallel HDF5.
-            // It set to false, FilePerCore mode is used in Damaris, then simulation results in each
-            // node are aggregated by dedicated Damaris cores and stored to separate files per Damaris core.
-            // Irrespective of mode, output is written asynchronously at the end of each timestep.
-            const bool enableDamarisOutputCollective = EWOMS_GET_PARAM(PreTypeTag, bool, EnableDamarisOutputCollective);
-            // Using the ModifyModel class to set the XML file for Damaris.
-            DamarisOutput::initializeDamaris(EclGenericVanguard::comm(), EclGenericVanguard::comm().rank(), outputDir, enableDamarisOutputCollective);
-            int is_client;
-            MPI_Comm new_comm;
-            int err = damaris_start(&is_client);
-            isSimulationRank_ = (is_client > 0);
-            if (isSimulationRank_ && err == DAMARIS_OK) {
-                damaris_client_comm_get(&new_comm);
-                EclGenericVanguard::setCommunication(std::make_unique<Parallel::Communication>(new_comm));
-            } else {
-                return false;
-            }
+            this->setupDamaris(outputDir,
+                               EWOMS_GET_PARAM(PreTypeTag, bool, EnableDamarisOutputCollective));
         }
 #endif // HAVE_DAMARIS
 
         int mpiRank = EclGenericVanguard::comm().rank();
-        FileOutputMode outputMode = FileOutputMode::OUTPUT_NONE;
         outputCout_ = false;
         if (mpiRank == 0)
             outputCout_ = EWOMS_GET_PARAM(PreTypeTag, bool, EnableTerminalOutput);
@@ -492,48 +356,31 @@ private:
             exitCode = EXIT_FAILURE;
             return false;
         }
+
+        std::string cmdline_params;
         if (outputCout_) {
-            FlowMainEbos<PreTypeTag>::printBanner(EclGenericVanguard::comm());
+            printFlowBanner(EclGenericVanguard::comm().size(),
+                            getNumThreads<PreTypeTag>(),
+                            Opm::moduleVersionName());
+            std::ostringstream str;
+            Parameters::printValues<PreTypeTag>(str);
+            cmdline_params = str.str();
         }
+
         // Create Deck and EclipseState.
         try {
-            auto python = std::make_shared<Python>();
-            const bool init_from_restart_file = !EWOMS_GET_PARAM(PreTypeTag, bool, SchedRestart);
-            const bool allRanksDbgPrtLog = EWOMS_GET_PARAM(PreTypeTag, bool,
-                                                      EnableLoggingFalloutWarning);
-            outputMode = setupLogging(mpiRank,
-                                      deckFilename,
-                                      outputDir,
-                                      EWOMS_GET_PARAM(PreTypeTag, std::string, OutputMode),
-                                      outputCout_, "STDOUT_LOGGER", allRanksDbgPrtLog);
-            auto parseContext =
-                std::make_unique<ParseContext>(std::vector<std::pair<std::string , InputError::Action>>
-                                               {{ParseContext::PARSE_RANDOM_SLASH, InputError::IGNORE},
-                                                {ParseContext::PARSE_MISSING_DIMS_KEYWORD, InputError::WARN},
-                                                {ParseContext::SUMMARY_UNKNOWN_WELL, InputError::WARN},
-                                                {ParseContext::SUMMARY_UNKNOWN_GROUP, InputError::WARN}});
-            if (EWOMS_GET_PARAM(PreTypeTag, bool, EclStrictParsing))
-                parseContext->update(InputError::DELAYED_EXIT1);
-
-            FlowMainEbos<PreTypeTag>::printPRTHeader(outputCout_);
-
-            if (outputCout_) {
-                OpmLog::info("Reading deck file '" + deckFilename + "'");
-            }
-
-            std::optional<int> outputInterval;
-            int output_param = EWOMS_GET_PARAM(PreTypeTag, int, EclOutputInterval);
-            if (output_param >= 0)
-                outputInterval = output_param;
-
-            readDeck(EclGenericVanguard::comm(), deckFilename, deck_, eclipseState_, schedule_, udqState_, actionState_, wtestState_,
-                     summaryConfig_, nullptr, python, std::move(parseContext),
-                     init_from_restart_file, outputCout_, outputInterval);
-
-            verifyValidCellGeometry(EclGenericVanguard::comm(), *this->eclipseState_);
-
+            this->readDeck(deckFilename,
+                           outputDir,
+                           EWOMS_GET_PARAM(PreTypeTag, std::string, OutputMode),
+                           !EWOMS_GET_PARAM(PreTypeTag, bool, SchedRestart),
+                           EWOMS_GET_PARAM(PreTypeTag, bool,  EnableLoggingFalloutWarning),
+                           EWOMS_GET_PARAM(PreTypeTag, std::string, ParsingStrictness),
+                           mpiRank,
+                           EWOMS_GET_PARAM(PreTypeTag, int, EclOutputInterval),
+                           cmdline_params,
+                           Opm::moduleVersion(),
+                           Opm::compileTimestamp());
             setupTime_ = externalSetupTimer.elapsed();
-            outputFiles_ = (outputMode != FileOutputMode::OUTPUT_NONE);
         }
         catch (const std::invalid_argument& e)
         {
@@ -552,34 +399,6 @@ private:
         return true;
     }
 
-    std::filesystem::path simulationCaseName_(const std::string& casename)
-    {
-        namespace fs = ::std::filesystem;
-
-        auto exists = [](const fs::path& f)
-        {
-            return (fs::exists(f) && fs::is_regular_file(f))
-                || (fs::is_symlink(f) &&
-                    fs::is_regular_file(fs::read_symlink(f)));
-        };
-
-        auto simcase = fs::path { casename };
-
-        if (exists(simcase)) {
-            return simcase;
-        }
-
-        for (const auto& ext : { std::string("DATA"), std::string("data") }) {
-            if (exists(simcase.replace_extension(ext))) {
-                return simcase;
-            }
-        }
-
-        throw std::invalid_argument {
-            "Cannot find input case '" + casename + '\''
-        };
-    }
-
     // This function is an extreme special case, if the program has been invoked
     // *exactly* as:
     //
@@ -587,19 +406,8 @@ private:
     //
     // the call is intercepted by this function which will print "flow $version"
     // on stdout and exit(0).
-    void handleVersionCmdLine_(int argc, char** argv)
-    {
-        auto pos = std::find_if(argv, argv + argc,
-            [](const char* arg)
-        {
-            return std::strcmp(arg, "--version") == 0;
-        });
-
-        if (pos != argv + argc) {
-            std::cout << "flow " << moduleVersionName() << std::endl;
-            std::exit(EXIT_SUCCESS);
-        }
-    }
+    void handleVersionCmdLine_(int argc, char** argv,
+                               std::string_view moduleVersionName);
 
     // This function is a special case, if the program has been invoked
     // with the argument "--test-split-communicator=true" as the FIRST
@@ -607,15 +415,7 @@ private:
     // test_split_comm_ flag to true.
     // Note: initializing the parameter system before MPI could make this
     // use the parameter system instead.
-    void handleTestSplitCommunicatorCmdLine_()
-    {
-        if (argc_ >= 2 && std::strcmp(argv_[1], "--test-split-communicator=true") == 0) {
-            test_split_comm_ = true;
-            --argc_;             // We have one less argument.
-            argv_[1] = argv_[0]; // What used to be the first proper argument now becomes the command argument.
-            ++argv_;             // Pretend this is what it always was.
-        }
-    }
+    void handleTestSplitCommunicatorCmdLine_();
 
     int runMICP(const Phases& phases)
     {
@@ -637,6 +437,7 @@ private:
     int runTwoPhase(const Phases& phases)
     {
         const bool diffusive = eclipseState_->getSimulationConfig().isDiffusive();
+        const bool disgasw = eclipseState_->getSimulationConfig().hasDISGASW();
 
         // oil-gas
         if (phases.active( Phase::OIL ) && phases.active( Phase::GAS )) {
@@ -660,12 +461,19 @@ private:
 
         // gas-water
         else if ( phases.active( Phase::GAS ) && phases.active( Phase::WATER ) ) {
+            if (disgasw) {
+                if (diffusive) {
+                    return flowEbosGasWaterDissolutionDiffuseMain(argc_, argv_, outputCout_, outputFiles_);
+                }
+                return flowEbosGasWaterDissolutionMain(argc_, argv_, outputCout_, outputFiles_);
+            }
             if (diffusive) {
                 if (outputCout_) {
-                    std::cerr << "The DIFFUSE option is not available for the two-phase gas/water model." << std::endl;
+                    std::cerr << "The DIFFUSE option is not available for the two-phase gas/water model without disgasw." << std::endl;
                 }
                 return EXIT_FAILURE;
             }
+
             return flowEbosGasWaterMain(argc_, argv_, outputCout_, outputFiles_);
         }
         else {
@@ -720,7 +528,7 @@ private:
 
         return flowEbosWaterOnlyMain(argc_, argv_, outputCout_, outputFiles_);
     }
-    
+
     int runWaterOnlyEnergy(const Phases& phases)
     {
         if (!phases.active(Phase::WATER) || phases.size() != 2) {
@@ -744,7 +552,7 @@ private:
             return EXIT_FAILURE;
         }
 
-        if (phases.size() == 3) { 
+        if (phases.size() == 3) {
 
             if (phases.active(Phase::OIL)){ // oil water brine case
                 return flowEbosOilWaterBrineMain(argc_, argv_, outputCout_, outputFiles_);
@@ -754,7 +562,7 @@ private:
                     eclipseState_->getSimulationConfig().hasVAPWAT()) {
                     //case with water vaporization into gas phase and salt precipitation
                     return flowEbosGasWaterSaltprecVapwatMain(argc_, argv_, outputCout_, outputFiles_);
-                } 
+                }
                 else {
                     return flowEbosGasWaterBrineMain(argc_, argv_, outputCout_, outputFiles_);
                 }
@@ -793,6 +601,11 @@ private:
             return flowEbosGasOilEnergyMain(argc_, argv_, outputCout_, outputFiles_);
         }
 
+        // water-gas-thermal
+        if (!phases.active( Phase::OIL ) && phases.active( Phase::WATER ) && phases.active( Phase::GAS )) {
+            return flowEbosGasWaterEnergyMain(argc_, argv_, outputCout_, outputFiles_);
+        }
+
         return flowEbosEnergyMain(argc_, argv_, outputCout_, outputFiles_);
     }
 
@@ -808,6 +621,49 @@ private:
         }
     }
 
+    void readDeck(const std::string& deckFilename,
+                  const std::string& outputDir,
+                  const std::string& outputMode,
+                  const bool init_from_restart_file,
+                  const bool allRanksDbgPrtLog,
+                  const std::string& parsingStrictness,
+                  const int mpiRank,
+                  const int output_param,
+                  const std::string& parameters,
+                  std::string_view moduleVersion,
+                  std::string_view compileTimestamp);
+
+    void setupVanguard();
+
+    template<class TypeTag>
+    static int getNumThreads()
+    {
+
+        int threads = 1;
+
+#ifdef _OPENMP
+        // This function is called before the parallel OpenMP stuff gets initialized.
+        // That initialization happends after the deck is read and we want this message.
+        // Hence we duplicate the code of setupParallelism to get the number of threads.
+        if (std::getenv("OMP_NUM_THREADS"))
+            threads =  omp_get_max_threads();
+        else
+            threads = std::min(2, omp_get_max_threads());
+
+        const int input_threads = EWOMS_GET_PARAM(TypeTag, int, ThreadsPerProcess);
+
+        if (input_threads > 0)
+            threads = std::min(input_threads, omp_get_max_threads());
+#endif
+
+        return threads;
+    }
+
+#if HAVE_DAMARIS
+    void setupDamaris(const std::string& outputDir,
+                      const bool enableDamarisOutputCollective);
+#endif
+
     int argc_{0};
     char** argv_{nullptr};
     bool outputCout_{false};
@@ -821,7 +677,6 @@ private:
     std::unique_ptr<WellTestState> wtestState_{};
 
     // These variables may be owned by both Python and the simulator
-    std::shared_ptr<Deck> deck_{};
     std::shared_ptr<EclipseState> eclipseState_{};
     std::shared_ptr<Schedule> schedule_{};
     std::shared_ptr<SummaryConfig> summaryConfig_{};
