@@ -27,6 +27,8 @@
 #include <ebos/eclbasevanguard.hh>
 
 #include <opm/common/ErrorMacros.hpp>
+#include <opm/common/Exceptions.hpp>
+#include <opm/common/TimingMacros.hpp>
 
 #include <opm/models/discretization/common/fvbaseproperties.hh>
 #include <opm/models/common/multiphasebaseproperties.hh>
@@ -84,7 +86,7 @@ public:
 namespace Opm
 {
 
-#if HAVE_CUDA || HAVE_OPENCL || HAVE_FPGA || HAVE_AMGCL
+#if COMPILE_BDA_BRIDGE
 template<class Matrix, class Vector, int block_size> class BdaBridge;
 class WellContributions;
 #endif
@@ -113,7 +115,7 @@ struct FlexibleSolverInfo
     size_t interiorCellNum_ = 0;
 };
 
-#if HAVE_CUDA || HAVE_OPENCL || HAVE_FPGA || HAVE_AMGCL
+#if COMPILE_BDA_BRIDGE
 template<class Matrix, class Vector>
 struct BdaSolverInfo
 {
@@ -121,13 +123,12 @@ struct BdaSolverInfo
   using Bridge = BdaBridge<Matrix,Vector,Matrix::block_type::rows>;
 
   BdaSolverInfo(const std::string& accelerator_mode,
-                const std::string& fpga_bitstream,
                 const int linear_solver_verbosity,
                 const int maxit,
                 const double tolerance,
                 const int platformID,
                 const int deviceID,
-                const std::string& opencl_ilu_reorder,
+                const bool opencl_ilu_parallel,
                 const std::string& linsolver);
 
   ~BdaSolverInfo();
@@ -148,6 +149,8 @@ struct BdaSolverInfo
              Vector& x,
              Dune::InverseOperatorResult& result);
 
+  bool gpuActive();
+
   int numJacobiBlocks_ = 0;
 
 private:
@@ -237,21 +240,22 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
               converged_(false),
               matrix_()
         {
+            OPM_TIMEBLOCK(IstlSolverEbos);
             const bool on_io_rank = (simulator.gridView().comm().rank() == 0);
 #if HAVE_MPI
             comm_.reset( new CommunicationType( simulator_.vanguard().grid().comm() ) );
 #endif
-            parameters_.template init<TypeTag>();
+            parameters_.template init<TypeTag>(simulator_.vanguard().eclState().getSimulationConfig().useCPR());
             prm_ = setupPropertyTree(parameters_,
                                      EWOMS_PARAM_IS_SET(TypeTag, int, LinearSolverMaxIter),
-                                     EWOMS_PARAM_IS_SET(TypeTag, int, CprMaxEllIter));
+                                     EWOMS_PARAM_IS_SET(TypeTag, double, LinearSolverReduction));
 
-#if HAVE_CUDA || HAVE_OPENCL || HAVE_FPGA || HAVE_AMGCL
+#if COMPILE_BDA_BRIDGE
             {
                 std::string accelerator_mode = EWOMS_GET_PARAM(TypeTag, std::string, AcceleratorMode);
                 if ((simulator_.vanguard().grid().comm().size() > 1) && (accelerator_mode != "none")) {
                     if (on_io_rank) {
-                        OpmLog::warning("Cannot use GPU or FPGA with MPI, GPU/FPGA are disabled");
+                        OpmLog::warning("Cannot use GPU with MPI, GPU are disabled");
                     }
                     accelerator_mode = "none";
                 }
@@ -259,23 +263,21 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
                 const int deviceID = EWOMS_GET_PARAM(TypeTag, int, BdaDeviceId);
                 const int maxit = EWOMS_GET_PARAM(TypeTag, int, LinearSolverMaxIter);
                 const double tolerance = EWOMS_GET_PARAM(TypeTag, double, LinearSolverReduction);
-                const std::string opencl_ilu_reorder = EWOMS_GET_PARAM(TypeTag, std::string, OpenclIluReorder);
+                const bool opencl_ilu_parallel = EWOMS_GET_PARAM(TypeTag, bool, OpenclIluParallel);
                 const int linear_solver_verbosity = parameters_.linear_solver_verbosity_;
-                std::string fpga_bitstream = EWOMS_GET_PARAM(TypeTag, std::string, FpgaBitstream);
                 std::string linsolver = EWOMS_GET_PARAM(TypeTag, std::string, LinearSolver);
                 bdaBridge = std::make_unique<detail::BdaSolverInfo<Matrix,Vector>>(accelerator_mode,
-                                                                                   fpga_bitstream,
                                                                                    linear_solver_verbosity,
                                                                                    maxit,
                                                                                    tolerance,
                                                                                    platformID,
                                                                                    deviceID,
-                                                                                   opencl_ilu_reorder,
+                                                                                   opencl_ilu_parallel,
                                                                                    linsolver);
             }
 #else
             if (EWOMS_GET_PARAM(TypeTag, std::string, AcceleratorMode) != "none") {
-                OPM_THROW(std::logic_error,"Cannot use accelerated solver since CUDA, OpenCL and amgcl were not found by cmake and FPGA was not enabled");
+                OPM_THROW(std::logic_error,"Cannot use accelerated solver since CUDA, OpenCL and amgcl were not found by cmake");
             }
 #endif
             extractParallelGridInformationToISTL(simulator_.vanguard().grid(), parallelInformation_);
@@ -286,12 +288,6 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
             ElementMapper elemMapper(simulator_.vanguard().gridView(), Dune::mcmgElementLayout());
             detail::findOverlapAndInterior(simulator_.vanguard().grid(), elemMapper, overlapRows_, interiorRows_);
             useWellConn_ = EWOMS_GET_PARAM(TypeTag, bool, MatrixAddWellContributions);
-#if HAVE_FPGA
-            // check usage of MatrixAddWellContributions: for FPGA they must be included
-            if (EWOMS_GET_PARAM(TypeTag, std::string, AcceleratorMode) == "fpga" && !useWellConn_) {
-                OPM_THROW(std::logic_error,"fpgaSolver needs --matrix-add-well-contributions=true");
-            }
-#endif
             const bool ownersFirst = EWOMS_GET_PARAM(TypeTag, bool, OwnerCellsFirst);
             if (!ownersFirst) {
                 const std::string msg = "The linear solver no longer supports --owner-cells-first=false.";
@@ -319,6 +315,7 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
 
         void prepare(const SparseMatrixAdapter& M, Vector& b)
         {
+            OPM_TIMEBLOCK(istlSolverEbosPrepare);        
             static bool firstcall = true;
 #if HAVE_MPI
             if (firstcall) {
@@ -347,8 +344,8 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
             } else {
                 // Pointers should not change
                 if ( &(M.istlMatrix()) != matrix_ ) {
-                        OPM_THROW(std::logic_error, "Matrix objects are expected to be reused when reassembling!"
-                                  <<" old pointer was " << matrix_ << ", new one is " << (&M.istlMatrix()) );
+                        OPM_THROW(std::logic_error,
+                                  "Matrix objects are expected to be reused when reassembling!");
                 }
             }
             rhs_ = &b;
@@ -356,7 +353,14 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
             if (isParallel() && prm_.get<std::string>("preconditioner.type") != "ParOverILU0") {
                 detail::makeOverlapRowsInvalid(getMatrix(), overlapRows_);
             }
+#if COMPILE_BDA_BRIDGE
+            if(!bdaBridge->gpuActive()){
+                prepareFlexibleSolver();
+            }
+#else
             prepareFlexibleSolver();
+#endif
+
             firstcall = false;
         }
 
@@ -378,6 +382,7 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
 
         bool solve(Vector& x)
         {
+            OPM_TIMEBLOCK(istlSolverEbosSolve);        
             calls_ += 1;
             // Write linear system if asked for.
             const int verbosity = prm_.get<int>("verbosity", 0);
@@ -392,9 +397,7 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
             // Solve system.
             Dune::InverseOperatorResult result;
 
-            // Use GPU if: available, chosen by user, and successful.
-            // Use FPGA if: support compiled, chosen by user, and successful.
-#if HAVE_CUDA || HAVE_OPENCL || HAVE_FPGA || HAVE_AMGCL
+#if COMPILE_BDA_BRIDGE
             std::function<void(WellContributions&)> getContribs =
                 [this](WellContributions& w)
                 {
@@ -406,6 +409,12 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
                                   x, result))
 #endif
             {
+                OPM_TIMEBLOCK(flexibleSolverApply);
+#if COMPILE_BDA_BRIDGE
+                if(bdaBridge->gpuActive()){
+                    prepareFlexibleSolver();
+                }
+#endif
                 assert(flexibleSolver_.solver_);
                 flexibleSolver_.solver_->apply(x, *rhs_, result);
             }
@@ -439,11 +448,19 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
             // store number of iterations
             iterations_ = result.iterations;
             converged_ = result.converged;
-
+            if(!converged_){
+                if(result.reduction < parameters_.relaxed_linear_solver_reduction_){
+                    std::stringstream ss;
+                    ss<< "Full linear solver tolerance not achived. The reduction is:" << result.reduction
+                      << " after " << result.iterations << " iterations ";
+                    OpmLog::warning(ss.str());
+                    converged_ = true;
+                }
+            }
             // Check for failure of linear solver.
-            if (!parameters_.ignoreConvergenceFailure_ && !result.converged) {
+            if (!parameters_.ignoreConvergenceFailure_ && !converged_) {
                 const std::string msg("Convergence failure for linear solver.");
-                OPM_THROW_NOLOG(NumericalIssue, msg);
+                OPM_THROW_NOLOG(NumericalProblem, msg);
             }
         }
     protected:
@@ -458,7 +475,7 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
 
         void prepareFlexibleSolver()
         {
-
+            OPM_TIMEBLOCK(flexibleSolverPrepare);
             if (shouldCreateSolver()) {
                 std::function<Vector()> trueFunc =
                     [this]
@@ -470,7 +487,7 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
                     auto wellOp = std::make_unique<WellModelOperator>(simulator_.problem().wellModel());
                     flexibleSolver_.wellOperator_ = std::move(wellOp);
                 }
-
+                OPM_TIMEBLOCK(flexibleSolverCreate);
                 flexibleSolver_.create(getMatrix(),
                                        isParallel(),
                                        prm_,
@@ -480,6 +497,7 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
             }
             else
             {
+                OPM_TIMEBLOCK(flexibleSolverUpdate);        
                 flexibleSolver_.pre_->update();
             }
         }
@@ -537,6 +555,7 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
         // conservation equations, ignoring all other terms.
         Vector getTrueImpesWeights(int pressureVarIndex) const
         {
+            OPM_TIMEBLOCK(getTrueImpesWeights);
             Vector weights(rhs_->size());
             ElementContext elemCtx(simulator_);
             Amg::getTrueImpesWeights(pressureVarIndex, weights,
@@ -566,7 +585,7 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
         Matrix* matrix_;
         Vector *rhs_;
 
-#if HAVE_CUDA || HAVE_OPENCL || HAVE_FPGA || HAVE_AMGCL
+#if COMPILE_BDA_BRIDGE
         std::unique_ptr<detail::BdaSolverInfo<Matrix, Vector>> bdaBridge;
 #endif
 
