@@ -37,14 +37,18 @@
 #include <opm/simulators/linalg/bda/opencl/openclWellContributions.hpp>
 #endif
 
-#if HAVE_FPGA
-#include <opm/simulators/linalg/bda/FPGASolverBackend.hpp>
-#endif
-
 #if HAVE_AMGCL
 #include <opm/simulators/linalg/bda/amgclSolverBackend.hpp>
 #endif
 
+#if HAVE_ROCALUTION
+#include <opm/simulators/linalg/bda/rocalutionSolverBackend.hpp>
+#endif
+
+#if HAVE_ROCSPARSE
+#include <opm/simulators/linalg/bda/rocsparseSolverBackend.hpp>
+#endif
+
 typedef Dune::InverseOperatorResult InverseOperatorResult;
 
 namespace Opm
@@ -53,16 +57,14 @@ namespace Opm
     using Opm::Accelerator::BdaResult;
     using Opm::Accelerator::BdaSolver;
     using Opm::Accelerator::SolverStatus;
-    using Opm::Accelerator::ILUReorder;
 
 template <class BridgeMatrix, class BridgeVector, int block_size>
 BdaBridge<BridgeMatrix, BridgeVector, block_size>::BdaBridge(std::string accelerator_mode_,
-                                                             [[maybe_unused]] std::string fpga_bitstream,
                                                              int linear_solver_verbosity, int maxit,
                                                              double tolerance,
                                                              [[maybe_unused]] unsigned int platformID,
                                                              unsigned int deviceID,
-                                                             [[maybe_unused]] std::string opencl_ilu_reorder,
+                                                             [[maybe_unused]] bool opencl_ilu_parallel,
                                                              [[maybe_unused]] std::string linsolver)
 : verbosity(linear_solver_verbosity), accelerator_mode(accelerator_mode_)
 {
@@ -76,38 +78,9 @@ BdaBridge<BridgeMatrix, BridgeVector, block_size>::BdaBridge(std::string acceler
     } else if (accelerator_mode.compare("opencl") == 0) {
 #if HAVE_OPENCL
         use_gpu = true;
-        ILUReorder ilu_reorder;
-        if (opencl_ilu_reorder == "") {
-            ilu_reorder = Opm::Accelerator::ILUReorder::GRAPH_COLORING;  // default when not selected by user
-        } else if (opencl_ilu_reorder == "level_scheduling") {
-            ilu_reorder = Opm::Accelerator::ILUReorder::LEVEL_SCHEDULING;
-        } else if (opencl_ilu_reorder == "graph_coloring") {
-            ilu_reorder = Opm::Accelerator::ILUReorder::GRAPH_COLORING;
-        } else if (opencl_ilu_reorder == "none") {
-            ilu_reorder = Opm::Accelerator::ILUReorder::NONE;
-        } else {
-            OPM_THROW(std::logic_error, "Error invalid argument for --opencl-ilu-reorder, usage: '--opencl-ilu-reorder=[level_scheduling|graph_coloring]'");
-        }
-        backend.reset(new Opm::Accelerator::openclSolverBackend<block_size>(linear_solver_verbosity, maxit, tolerance, platformID, deviceID, ilu_reorder, linsolver));
+        backend.reset(new Opm::Accelerator::openclSolverBackend<block_size>(linear_solver_verbosity, maxit, tolerance, platformID, deviceID, opencl_ilu_parallel, linsolver));
 #else
         OPM_THROW(std::logic_error, "Error openclSolver was chosen, but OpenCL was not found by CMake");
-#endif
-    } else if (accelerator_mode.compare("fpga") == 0) {
-#if HAVE_FPGA
-        use_fpga = true;
-        ILUReorder ilu_reorder;
-        if (opencl_ilu_reorder == "") {
-            ilu_reorder = Opm::Accelerator::ILUReorder::LEVEL_SCHEDULING;  // default when not selected by user
-        } else if (opencl_ilu_reorder == "level_scheduling") {
-            ilu_reorder = Opm::Accelerator::ILUReorder::LEVEL_SCHEDULING;
-        } else if (opencl_ilu_reorder == "graph_coloring") {
-            ilu_reorder = Opm::Accelerator::ILUReorder::GRAPH_COLORING;
-        } else {
-            OPM_THROW(std::logic_error, "Error invalid argument for --opencl-ilu-reorder, usage: '--opencl-ilu-reorder=[level_scheduling|graph_coloring]'");
-        }
-        backend.reset(new Opm::Accelerator::FpgaSolverBackend<block_size>(fpga_bitstream, linear_solver_verbosity, maxit, tolerance, ilu_reorder));
-#else
-        OPM_THROW(std::logic_error, "Error fpgaSolver was chosen, but FPGA was not enabled by CMake");
 #endif
     } else if (accelerator_mode.compare("amgcl") == 0) {
 #if HAVE_AMGCL
@@ -115,12 +88,25 @@ BdaBridge<BridgeMatrix, BridgeVector, block_size>::BdaBridge(std::string acceler
         backend.reset(new Opm::Accelerator::amgclSolverBackend<block_size>(linear_solver_verbosity, maxit, tolerance, platformID, deviceID));
 #else
         OPM_THROW(std::logic_error, "Error amgclSolver was chosen, but amgcl was not found by CMake");
+#endif
+    } else if (accelerator_mode.compare("rocalution") == 0) {
+#if HAVE_ROCALUTION
+        use_gpu = true; // should be replaced by a 'use_bridge' boolean
+        backend.reset(new Opm::Accelerator::rocalutionSolverBackend<block_size>(linear_solver_verbosity, maxit, tolerance));
+#else
+        OPM_THROW(std::logic_error, "Error rocalutionSolver was chosen, but rocalution was not found by CMake");
+#endif
+    } else if (accelerator_mode.compare("rocsparse") == 0) {
+#if HAVE_ROCSPARSE
+        use_gpu = true; // should be replaced by a 'use_bridge' boolean
+        backend.reset(new Opm::Accelerator::rocsparseSolverBackend<block_size>(linear_solver_verbosity, maxit, tolerance, platformID, deviceID));
+#else
+        OPM_THROW(std::logic_error, "Error rocsparseSolver was chosen, but rocsparse/rocblas was not found by CMake");
 #endif
     } else if (accelerator_mode.compare("none") == 0) {
         use_gpu = false;
-        use_fpga = false;
     } else {
-        OPM_THROW(std::logic_error, "Error unknown value for parameter 'AcceleratorMode', should be passed like '--accelerator-mode=[none|cusparse|opencl|fpga|amgcl]");
+        OPM_THROW(std::logic_error, "Error unknown value for parameter 'AcceleratorMode', should be passed like '--accelerator-mode=[none|cusparse|opencl|amgcl|rocalution]");
     }
 }
 
@@ -220,7 +206,7 @@ void BdaBridge<BridgeMatrix, BridgeVector, block_size>::solve_system(BridgeMatri
                                                                      WellContributions& wellContribs,
                                                                      InverseOperatorResult& res)
 {
-    if (use_gpu || use_fpga) {
+    if (use_gpu) {
         BdaResult result;
         result.converged = false;
         const int dim = (*bridgeMat)[0][0].N();
@@ -229,7 +215,7 @@ void BdaBridge<BridgeMatrix, BridgeVector, block_size>::solve_system(BridgeMatri
 
         if (dim != 3) {
             OpmLog::warning("BdaSolver only accepts blocksize = 3 at this time, will use Dune for the remainder of the program");
-            use_gpu = use_fpga = false;
+            use_gpu = false;
             return;
         }
 
@@ -301,7 +287,7 @@ void BdaBridge<BridgeMatrix, BridgeVector, block_size>::solve_system(BridgeMatri
 
 template <class BridgeMatrix, class BridgeVector, int block_size>
 void BdaBridge<BridgeMatrix, BridgeVector, block_size>::get_result([[maybe_unused]] BridgeVector& x) {
-    if (use_gpu || use_fpga) {
+    if (use_gpu) {
         backend->get_result(static_cast<double*>(&(x[0][0])));
     }
 }
