@@ -23,8 +23,6 @@
 #include "dune/istl/solver.hh" // for struct InverseOperatorResult
 
 #include <opm/simulators/linalg/bda/BdaSolver.hpp>
-#include <opm/simulators/linalg/bda/BlockedMatrix.hpp>
-#include <opm/simulators/linalg/bda/ILUReorder.hpp>
 
 namespace Opm
 {
@@ -32,7 +30,6 @@ namespace Opm
 class WellContributions;
 
 typedef Dune::InverseOperatorResult InverseOperatorResult;
-using Opm::Accelerator::ILUReorder;
 
 /// BdaBridge acts as interface between opm-simulators with the BdaSolvers
 template <class BridgeMatrix, class BridgeVector, int block_size>
@@ -41,7 +38,6 @@ class BdaBridge
 private:
     int verbosity = 0;
     bool use_gpu = false;
-    bool use_fpga = false;
     std::string accelerator_mode;
     std::unique_ptr<Opm::Accelerator::BdaSolver<block_size> > backend;
     std::shared_ptr<Opm::Accelerator::BlockedMatrix> matrix;  // 'stores' matrix, actually points to h_rows, h_cols and the received BridgeMatrix for the nonzeroes
@@ -53,17 +49,16 @@ private:
 
 public:
     /// Construct a BdaBridge
-    /// \param[in] accelerator_mode           to select if an accelerated solver is used, is passed via command-line: '--accelerator-mode=[none|cusparse|opencl|fpga]'
-    /// \param[in] fpga_bitstream             FPGA programming bitstream file name, is passed via command-line: '--fpga-bitstream=[<filename>]'
+    /// \param[in] accelerator_mode           to select if an accelerated solver is used, is passed via command-line: '--accelerator-mode=[none|cusparse|opencl|amgcl|rocalution]'
     /// \param[in] linear_solver_verbosity    verbosity of BdaSolver
     /// \param[in] maxit                      maximum number of iterations for BdaSolver
     /// \param[in] tolerance                  required relative tolerance for BdaSolver
     /// \param[in] platformID                 the OpenCL platform ID to be used
     /// \param[in] deviceID                   the device ID to be used by the cusparse- and openclSolvers, too high values could cause runtime errors
-    /// \param[in] opencl_ilu_reorder         select either level_scheduling or graph_coloring, see ILUReorder.hpp for explanation
-    /// \param[in] linsolver                  copy of cmdline argument --linear-solver
-    BdaBridge(std::string accelerator_mode, std::string fpga_bitstream, int linear_solver_verbosity, int maxit, double tolerance,
-        unsigned int platformID, unsigned int deviceID, std::string opencl_ilu_reorder, std::string linsolver);
+    /// \param[in] opencl_ilu_parallel        whether to parallelize the ILU decomposition and application in OpenCL with level_scheduling
+    /// \param[in] linsolver                  indicating the preconditioner, equal to the --linear-solver cmdline argument
+    BdaBridge(std::string accelerator_mode, int linear_solver_verbosity, int maxit, double tolerance,
+        unsigned int platformID, unsigned int deviceID, bool opencl_ilu_parallel, std::string linsolver);
 
 
     /// Solve linear system, A*x = b
@@ -98,12 +93,6 @@ public:
     /// \param[in] N              number of rows in scalar vector that wellContribs will be applied on
     void initWellContributions(WellContributions& wellContribs, unsigned N);
 
-    /// Return whether the BdaBridge will use the FPGA or not
-    /// return whether the BdaBridge will use the FPGA or not
-    bool getUseFpga(){
-        return use_fpga;
-    }
-
     /// Return the selected accelerator mode, this is input via the command-line
     std::string getAccleratorName(){
         return accelerator_mode;
