@@ -18,8 +18,7 @@
 */
 
 #include <config.h> // CMake
-#include <cstdlib>
-#include <cstring>
+
 #include <opm/common/OpmLog/OpmLog.hpp>
 #include <opm/common/ErrorMacros.hpp>
 
@@ -53,8 +52,10 @@ WellContributions::create(const std::string& accelerator_mode, bool useWellConn)
         OPM_THROW(std::runtime_error, "Cannot initialize well contributions: OpenCL is not enabled");
 #endif
     }
-    else if(accelerator_mode.compare("fpga") == 0){
-        // unused for FPGA, but must be defined to avoid error
+    else if(accelerator_mode.compare("rocsparse") == 0){
+        if (!useWellConn) {
+            OPM_THROW(std::logic_error, "Error rocsparse requires --matrix-add-well-contributions=true");
+        }
         return std::make_unique<WellContributions>();
     }
     else if(accelerator_mode.compare("amgcl") == 0){
@@ -63,6 +64,12 @@ WellContributions::create(const std::string& accelerator_mode, bool useWellConn)
         }
         return std::make_unique<WellContributions>();
     }
+    else if(accelerator_mode.compare("rocalution") == 0){
+        if (!useWellConn) {
+            OPM_THROW(std::logic_error, "Error rocalution requires --matrix-add-well-contributions=true");
+        }
+        return std::make_unique<WellContributions>();
+    }
     else{
         OPM_THROW(std::logic_error, "Invalid accelerator mode");
     }
@@ -95,9 +102,11 @@ void WellContributions::setBlockSize(unsigned int dim_, unsigned int dim_wells_)
     dim_wells = dim_wells_;
 
     if(dim != 3 || dim_wells != 4){
-        std::ostringstream oss;
-        oss << "WellContributions::setBlockSize error: dim and dim_wells must be equal to 3 and 4, repectivelly, otherwise the add well contributions kernel won't work.\n";
-        OPM_THROW(std::logic_error, oss.str());
+        OPM_THROW(std::logic_error,
+                  "WellContributions::setBlockSize error: "
+                  "dim and dim_wells must be equal to 3 and 4, "
+                  "respectively, otherwise the add well contributions "
+                  "kernel won't work.\n");
     }
 }
 
