@@ -32,10 +32,9 @@
 namespace Opm
 {
 
-/// This class serves to eliminate the need to include the WellContributions into the matrix (with --matrix-add-well-contributions=true) for the cusparseSolver
-/// If the --matrix-add-well-contributions commandline parameter is true, this class should not be used
-/// So far, StandardWell and MultisegmentWell are supported
-/// StandardWells are only supported for cusparseSolver (CUDA), MultisegmentWells are supported for both cusparseSolver and openclSolver
+/// This class serves to eliminate the need to include the WellContributions into the matrix (with --matrix-add-well-contributions=true) for the cusparseSolver or openclSolver.
+/// If the --matrix-add-well-contributions commandline parameter is true, this class should still be used, but be empty.
+/// StandardWell and MultisegmentWell are supported for both cusparseSolver and openclSolver.
 /// A single instance (or pointer) of this class is passed to the BdaSolver.
 /// For StandardWell, this class contains all the data and handles the computation. For MultisegmentWell, the vector 'multisegments' contains all the data. For more information, check the MultisegmentWellContribution class.
 
@@ -54,11 +53,7 @@ class WellContributions
 public:
     static std::unique_ptr<WellContributions> create(const std::string& accelerator_mode, bool useWellConn);
 
-#if DUNE_VERSION_NEWER(DUNE_ISTL, 2, 7)
     using UMFPackIndex = SuiteSparse_long;
-#else
-    using UMFPackIndex = int;
-#endif
     /// StandardWell has C, D and B matrices that need to be copied
     enum class MatrixType {
         C,
