@@ -70,7 +70,7 @@ private:
     cl::Buffer d_invUvals;
     cl::Buffer d_invL_x;
 
-    ILUReorder opencl_ilu_reorder;
+    bool opencl_ilu_parallel;
     std::unique_ptr<BILU0<block_size> > bilu0;
 
     /// Struct that holds the structure of the small subsystems for each column
@@ -110,12 +110,12 @@ private:
     void buildUpperSubsystemsStructures();
 
 public:
-    BISAI(ILUReorder opencl_ilu_reorder, int verbosity);
+    BISAI(bool opencl_ilu_parallel, int verbosity);
 
     // set own Opencl variables, but also that of the bilu0 preconditioner
     void setOpencl(std::shared_ptr<cl::Context>& context, std::shared_ptr<cl::CommandQueue>& queue) override;
 
-    // analysis, find reordering if specified
+    // analysis, extract parallelism
     bool analyze_matrix(BlockedMatrix *mat) override;
     bool analyze_matrix(BlockedMatrix *mat, BlockedMatrix *jacMat) override;
 
@@ -125,21 +125,6 @@ public:
 
     // apply preconditioner, x = prec(y)
     void apply(const cl::Buffer& y, cl::Buffer& x) override;
-
-    int* getToOrder() override
-    {
-        return bilu0->getToOrder();
-    }
-
-    int* getFromOrder() override
-    {
-        return bilu0->getFromOrder();
-    }
-
-    BlockedMatrix* getRMat() override
-    {
-        return bilu0->getRMat();
-    }
 };
 
 /// Similar function to csrPatternToCsc. It gives an offset map from CSR to CSC instead of the full CSR to CSC conversion.
