@@ -22,19 +22,13 @@
 
 #include <mutex>
 
-#include <dune/common/version.hh>
-#if DUNE_VERSION_NEWER(DUNE_ISTL, 2, 7)
 #include <dune/istl/paamg/matrixhierarchy.hh>
-#else
-#include <dune/istl/paamg/hierarchy.hh>
-#endif
 #include <dune/istl/umfpack.hh>
 
 #include <opm/simulators/linalg/bda/opencl/opencl.hpp>
 #include <opm/simulators/linalg/bda/opencl/BILU0.hpp>
 #include <opm/simulators/linalg/bda/Matrix.hpp>
 #include <opm/simulators/linalg/bda/opencl/OpenclMatrix.hpp>
-#include <opm/simulators/linalg/bda/ILUReorder.hpp>
 #include <opm/simulators/linalg/bda/opencl/Preconditioner.hpp>
 
 #include <opm/simulators/linalg/bda/opencl/openclSolverBackend.hpp>
@@ -98,7 +92,7 @@ private:
     unsigned num_post_smooth_steps;             // number of Jacobi smooth steps after prolongation
 
     std::unique_ptr<openclSolverBackend<1> > coarse_solver; // coarse solver is scalar
-    ILUReorder opencl_ilu_reorder;                          // reordering strategy for ILU0 in coarse solver
+    bool opencl_ilu_parallel;                   // whether ILU0 operation should be parallelized
 
     // Analyze the AMG hierarchy build by Dune
     void analyzeHierarchy();
@@ -122,7 +116,7 @@ private:
 
 public:
 
-    CPR(int verbosity, ILUReorder opencl_ilu_reorder);
+    CPR(int verbosity, bool opencl_ilu_parallel);
 
     bool analyze_matrix(BlockedMatrix *mat) override;
     bool analyze_matrix(BlockedMatrix *mat, BlockedMatrix *jacMat) override;
@@ -136,21 +130,6 @@ public:
 
     bool create_preconditioner(BlockedMatrix *mat) override;
     bool create_preconditioner(BlockedMatrix *mat, BlockedMatrix *jacMat) override;
-    
-    int* getToOrder() override
-    {
-        return bilu0->getToOrder();
-    }
-
-    int* getFromOrder() override
-    {
-        return bilu0->getFromOrder();
-    }
-
-    BlockedMatrix* getRMat() override
-    {
-        return bilu0->getRMat();
-    }
 };
 
 // solve A^T * x = b
