@@ -23,7 +23,6 @@
 #include <opm/simulators/linalg/bda/opencl/opencl.hpp>
 #include <opm/simulators/linalg/bda/BdaResult.hpp>
 #include <opm/simulators/linalg/bda/BdaSolver.hpp>
-#include <opm/simulators/linalg/bda/ILUReorder.hpp>
 #include <opm/simulators/linalg/bda/WellContributions.hpp>
 
 #include <opm/simulators/linalg/bda/opencl/Preconditioner.hpp>
@@ -51,15 +50,14 @@ class openclSolverBackend : public BdaSolver<block_size>
     using Base::initialized;
 
 private:
-    double *rb = nullptr;                 // reordered b vector, if the matrix is reordered, rb is newly allocated, otherwise it just points to b
+    double *h_b = nullptr;                // b vector, on host
     std::vector<double> vals_contiguous;  // only used if COPY_ROW_BY_ROW is true in openclSolverBackend.cpp
 
     // OpenCL variables must be reusable, they are initialized in initialize()
-    cl::Buffer d_Avals, d_Acols, d_Arows;        // (reordered) matrix in BSR format on GPU
+    cl::Buffer d_Avals, d_Acols, d_Arows;        // matrix in BSR format on GPU
     cl::Buffer d_x, d_b, d_rb, d_r, d_rw, d_p;   // vectors, used during linear solve
     cl::Buffer d_pw, d_s, d_t, d_v;              // vectors, used during linear solve
     cl::Buffer d_tmp;                            // used as tmp GPU buffer for dot() and norm()
-    cl::Buffer d_toOrder;                        // only used when reordering is used
 
     std::vector<cl::Device> devices;
 
@@ -68,65 +66,13 @@ private:
     std::unique_ptr<Preconditioner<block_size> > prec;
                                                                   // can perform blocked ILU0 and AMG on pressure component
     bool is_root;                                                 // allow for nested solvers, the root solver is called by BdaBridge
-    int *toOrder = nullptr, *fromOrder = nullptr;                 // BILU0 reorders rows of the matrix via these mappings
     bool analysis_done = false;
     std::shared_ptr<BlockedMatrix> mat = nullptr;                 // original matrix
     std::shared_ptr<BlockedMatrix> jacMat = nullptr;              // matrix for preconditioner
-    BlockedMatrix *rmat = nullptr;                                // reordered matrix (or original if no reordering), used for spmv
-    ILUReorder opencl_ilu_reorder;                                // reordering strategy
+    bool opencl_ilu_parallel;                                     // parallelize ILU operations (with level_scheduling)
     std::vector<cl::Event> events;
     cl_int err;
 
-    /// Divide A by B, and round up: return (int)ceil(A/B)
-    /// \param[in] A    dividend
-    /// \param[in] B    divisor
-    /// \return         rounded division result
-    unsigned int ceilDivision(const unsigned int A, const unsigned int B);
-
-    /// Calculate dot product between in1 and in2, partial sums are stored in out, which are summed on CPU
-    /// \param[in] in1           input vector 1
-    /// \param[in] in2           input vector 2
-    /// \param[out] out          output vector containing partial sums
-    /// \return                  dot product
-    double dot_w(cl::Buffer in1, cl::Buffer in2, cl::Buffer out);
-
-    /// Calculate the norm of in, partial sums are stored in out, which are summed on the CPU
-    /// Equal to Dune::DenseVector::two_norm()
-    /// \param[in] in          input vector
-    /// \param[out] out        output vector containing partial sums
-    /// \return                norm
-    double norm_w(cl::Buffer in, cl::Buffer out);
-
-    /// Perform axpy: out += a * in
-    /// \param[in] in         input vector
-    /// \param[in] a          scalar value to multiply input vector
-    /// \param[inout] out     output vector
-    void axpy_w(cl::Buffer in, const double a, cl::Buffer out);
-
-    /// Perform scale: vec *= a
-    /// \param[inout] vec     vector to scale
-    /// \param[in] a          scalar value to multiply vector
-    void scale_w(cl::Buffer vec, const double a);
-
-    /// Custom function that combines scale, axpy and add functions in bicgstab
-    /// p = (p - omega * v) * beta + r
-    /// \param[inout] p      output vector
-    /// \param[in] v         input vector
-    /// \param[in] r         input vector
-    /// \param[in] omega     scalar value
-    /// \param[in] beta      scalar value
-    void custom_w(cl::Buffer p, cl::Buffer v, cl::Buffer r, const double omega, const double beta);
-
-    /// Sparse matrix-vector multiply, spmv
-    /// b = A * x
-    /// Matrix A, must be in BCRS format
-    /// \param[in] vals     nnzs of matrix A
-    /// \param[in] cols     columnindices of matrix A
-    /// \param[in] rows     rowpointers of matrix A
-    /// \param[in] x        input vector
-    /// \param[out] b       output vector
-    void spmv_blocked_w(cl::Buffer vals, cl::Buffer cols, cl::Buffer rows, cl::Buffer x, cl::Buffer b);
-
     /// Solve linear system using ilu0-bicgstab
     /// \param[in] wellContribs   WellContributions, to apply them separately, instead of adding them to matrix A
     /// \param[inout] res         summary of solver result
@@ -137,17 +83,13 @@ private:
     /// \param[in] jacMatrix  matrix for preconditioner
     void initialize(std::shared_ptr<BlockedMatrix> matrix, std::shared_ptr<BlockedMatrix> jacMatrix);
 
-    /// Clean memory
-    void finalize();
-
     /// Copy linear system to GPU
     void copy_system_to_gpu();
 
-    /// Reorder the linear system so it corresponds with the coloring
+    /// Reassign pointers, in case the addresses of the Dune variables have changed
     /// \param[in] vals           array of nonzeroes, each block is stored row-wise and contiguous, contains nnz values
-    /// \param[in] b              input vectors, contains N values
-    /// \param[out] wellContribs  WellContributions, to set reordering
-    void update_system(double *vals, double *b, WellContributions &wellContribs);
+    /// \param[in] b              input vector b, contains N values
+    void update_system(double *vals, double *b);
 
     /// Update linear system on GPU, don't copy rowpointers and colindices, they stay the same
     void update_system_on_gpu();
@@ -156,12 +98,13 @@ private:
     /// \return true iff analysis was successful
     bool analyze_matrix();
 
-    /// Perform ilu0-decomposition
+    /// Create the preconditioner, only done once per linear solve
     /// \return true iff decomposition was successful
     bool create_preconditioner();
 
     /// Solve linear system
     /// \param[in] wellContribs   WellContributions, to apply them separately, instead of adding them to matrix A
+    ///                           could be empty
     /// \param[inout] res         summary of solver result
     void solve_system(WellContributions &wellContribs, BdaResult &res);
 
@@ -175,17 +118,14 @@ public:
     /// \param[in] tolerance                  required relative tolerance for openclSolver
     /// \param[in] platformID                 the OpenCL platform to be used
     /// \param[in] deviceID                   the device to be used
-    /// \param[in] opencl_ilu_reorder         select either level_scheduling or graph_coloring, see Reorder.hpp for explanation
+    /// \param[in] opencl_ilu_parallel        whether to parallelize the ILU decomposition and application in OpenCL with level_scheduling
     /// \param[in] linsolver                  indicating the preconditioner, equal to the --linear-solver cmdline argument
     ///                                       only ilu0, cpr_quasiimpes and isai are supported
     openclSolverBackend(int linear_solver_verbosity, int maxit, double tolerance, unsigned int platformID, unsigned int deviceID,
-        ILUReorder opencl_ilu_reorder, std::string linsolver);
+        bool opencl_ilu_parallel, std::string linsolver);
 
     /// For the CPR coarse solver
-    openclSolverBackend(int linear_solver_verbosity, int maxit, double tolerance, ILUReorder opencl_ilu_reorder);
-
-    /// Destroy a openclSolver, and free memory
-    ~openclSolverBackend();
+    openclSolverBackend(int linear_solver_verbosity, int maxit, double tolerance, bool opencl_ilu_parallel);
 
     /// Solve linear system, A*x = b, matrix A must be in blocked-CSR format
     /// \param[in] matrix         matrix A
