@@ -43,16 +43,21 @@
 #include <opm/input/eclipse/Deck/Deck.hpp>
 
 #include <opm/input/eclipse/EclipseState/checkDeck.hpp>
+#include <opm/input/eclipse/EclipseState/Grid/FieldData.hpp>
+#include <opm/input/eclipse/EclipseState/SummaryConfig/SummaryConfig.hpp>
+
+#include <opm/input/eclipse/Parser/ErrorGuard.hpp>
+#include <opm/input/eclipse/Parser/InputErrorAction.hpp>
+#include <opm/input/eclipse/Parser/ParseContext.hpp>
+#include <opm/input/eclipse/Parser/Parser.hpp>
+
 #include <opm/input/eclipse/Schedule/Action/State.hpp>
 #include <opm/input/eclipse/Schedule/ArrayDimChecker.hpp>
 #include <opm/input/eclipse/Schedule/Schedule.hpp>
 #include <opm/input/eclipse/Schedule/SummaryState.hpp>
+#include <opm/input/eclipse/Schedule/UDQ/UDQConfig.hpp>
 #include <opm/input/eclipse/Schedule/UDQ/UDQState.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellTestState.hpp>
-#include <opm/input/eclipse/EclipseState/SummaryConfig/SummaryConfig.hpp>
-
-#include <opm/input/eclipse/Parser/ErrorGuard.hpp>
-#include <opm/input/eclipse/Parser/Parser.hpp>
 
 #include <opm/input/eclipse/Units/UnitSystem.hpp>
 
@@ -181,15 +186,15 @@ namespace {
         wtestState = std::make_unique<Opm::WellTestState>();
     }
 
-    std::shared_ptr<Opm::Deck>
+    Opm::Deck
     readDeckFile(const std::string&       deckFilename,
                  const bool               checkDeck,
                  const Opm::Parser&       parser,
                  const Opm::ParseContext& parseContext,
+                 const bool               treatCriticalAsNonCritical,
                  Opm::ErrorGuard&         errorGuard)
     {
-        auto deck = std::make_shared<Opm::Deck>
-            (parser.parseFile(deckFilename, parseContext, errorGuard));
+        Opm::Deck deck(parser.parseFile(deckFilename, parseContext, errorGuard));
 
         auto keyword_validator = Opm::KeywordValidation::KeywordValidator {
             Opm::FlowKeywordValidation::unsupportedKeywords(),
@@ -199,10 +204,10 @@ namespace {
             Opm::KeywordValidation::specialValidation()
         };
 
-        keyword_validator.validateDeck(*deck, parseContext, errorGuard);
+        keyword_validator.validateDeck(deck, parseContext, treatCriticalAsNonCritical, errorGuard);
 
         if (checkDeck) {
-            Opm::checkDeck(*deck, parser, parseContext, errorGuard);
+            Opm::checkDeck(deck, parser, parseContext, errorGuard);
         }
 
         return deck;
@@ -222,7 +227,6 @@ namespace {
     void readOnIORank(Opm::Parallel::Communication         comm,
                       const std::string&                   deckFilename,
                       const Opm::ParseContext*             parseContext,
-                      std::shared_ptr<Opm::Deck>&          deck,
                       std::shared_ptr<Opm::EclipseState>&  eclipseState,
                       std::shared_ptr<Opm::Schedule>&      schedule,
                       std::unique_ptr<Opm::UDQState>&      udqState,
@@ -232,41 +236,42 @@ namespace {
                       std::shared_ptr<Opm::Python>         python,
                       const bool                           initFromRestart,
                       const bool                           checkDeck,
+                      const bool                           treatCriticalAsNonCritical,
                       const std::optional<int>&            outputInterval,
                       Opm::ErrorGuard&                     errorGuard)
     {
-        if (((deck == nullptr) || (schedule == nullptr) || (summaryConfig == nullptr)) &&
+        if (((schedule == nullptr) || (summaryConfig == nullptr)) &&
             (parseContext == nullptr))
         {
             OPM_THROW(std::logic_error,
-                      "We need a parse context if deck, schedule, "
+                      "We need a parse context if schedule "
                       "or summaryConfig are not initialized");
         }
 
         auto parser = Opm::Parser{};
-        if (deck == nullptr) {
-            deck = readDeckFile(deckFilename, checkDeck, parser,
-                                *parseContext, errorGuard);
-        }
+        const auto deck = readDeckFile(deckFilename, checkDeck, parser,
+                                       *parseContext, treatCriticalAsNonCritical, errorGuard);
 
         if (eclipseState == nullptr) {
-            eclipseState = createEclipseState(comm, *deck);
+            eclipseState = createEclipseState(comm, deck);
         }
 
         if (eclipseState->getInitConfig().restartRequested()) {
-            loadObjectsFromRestart(*deck, parser, *parseContext,
+            loadObjectsFromRestart(deck, parser, *parseContext,
                                    initFromRestart, outputInterval,
                                    *eclipseState, std::move(python),
                                    schedule, udqState, actionState, wtestState,
                                    errorGuard);
         }
         else {
-            createNonRestartDynamicObjects(*deck, *eclipseState,
+            createNonRestartDynamicObjects(deck, *eclipseState,
                                            *parseContext, std::move(python),
                                            schedule, udqState, actionState, wtestState,
                                            errorGuard);
         }
 
+        eclipseState->appendAqufluxSchedule(schedule->getAquiferFluxSchedule());
+
         if (Opm::OpmLog::hasBackend("STDOUT_LOGGER")) {
             // loggers might not be set up!
             setupMessageLimiter((*schedule)[0].message_limits(), "STDOUT_LOGGER");
@@ -274,7 +279,7 @@ namespace {
 
         if (summaryConfig == nullptr) {
             summaryConfig = std::make_shared<Opm::SummaryConfig>
-                (*deck, *schedule, eclipseState->fieldProps(),
+                (deck, *schedule, eclipseState->fieldProps(),
                  eclipseState->aquifer(), *parseContext, errorGuard);
         }
 
@@ -480,33 +485,37 @@ Opm::setupLogging(const int          mpi_rank_,
 
 void Opm::readDeck(Opm::Parallel::Communication    comm,
                    const std::string&              deckFilename,
-                   std::shared_ptr<Deck>&          deck,
                    std::shared_ptr<EclipseState>&  eclipseState,
                    std::shared_ptr<Schedule>&      schedule,
                    std::unique_ptr<UDQState>&      udqState,
                    std::unique_ptr<Action::State>& actionState,
                    std::unique_ptr<WellTestState>& wtestState,
                    std::shared_ptr<SummaryConfig>& summaryConfig,
-                   std::unique_ptr<ErrorGuard>     errorGuard,
                    std::shared_ptr<Python>         python,
-                   std::unique_ptr<ParseContext>   parseContext,
+                   const std::string&              parsingStrictness,
                    const bool                      initFromRestart,
                    const bool                      checkDeck,
                    const std::optional<int>&       outputInterval)
 {
-    if (errorGuard == nullptr) {
-        errorGuard = std::make_unique<ErrorGuard>();
-    }
+    auto errorGuard = std::make_unique<ErrorGuard>();
 
     int parseSuccess = 1; // > 0 is success
     std::string failureMessage;
 
+    if (parsingStrictness != "high" && parsingStrictness != "normal" && parsingStrictness != "low") {
+        OPM_THROW(std::runtime_error,
+                  fmt::format("Incorrect value {} for parameter ParsingStrictness, must be 'high', 'normal', or 'low'", parsingStrictness));
+    }
+
     if (comm.rank() == 0) { // Always true when !HAVE_MPI
+        const bool exitOnAllErrors = (parsingStrictness == "high");
+        const bool treatCriticalAsNonCritical = (parsingStrictness == "low");
         try {
-            readOnIORank(comm, deckFilename, parseContext.get(), deck,
+            auto parseContext = setupParseContext(exitOnAllErrors);
+            readOnIORank(comm, deckFilename, parseContext.get(),
                          eclipseState, schedule, udqState, actionState, wtestState,
                          summaryConfig, std::move(python), initFromRestart,
-                         checkDeck, outputInterval, *errorGuard);
+                         checkDeck, treatCriticalAsNonCritical, outputInterval, *errorGuard);
         }
         catch (const OpmInputError& input_error) {
             failureMessage = input_error.what();
@@ -576,3 +585,17 @@ void Opm::verifyValidCellGeometry(Parallel::Communication comm,
 Please check geometry keywords, especially if grid is imported through GDFILE)"
     };
 }
+
+std::unique_ptr<Opm::ParseContext> Opm::setupParseContext(const bool strictParsing)
+{
+    auto parseContext =
+        std::make_unique<ParseContext>(std::vector<std::pair<std::string , InputErrorAction>>
+                                       {{ParseContext::PARSE_RANDOM_SLASH, InputErrorAction::IGNORE},
+                                       {ParseContext::PARSE_MISSING_DIMS_KEYWORD, InputErrorAction::WARN},
+                                       {ParseContext::SUMMARY_UNKNOWN_WELL, InputErrorAction::WARN},
+                                       {ParseContext::SUMMARY_UNKNOWN_GROUP, InputErrorAction::WARN}});
+    if (strictParsing)
+        parseContext->update(InputErrorAction::DELAYED_EXIT1);
+
+    return parseContext;
+}
