@@ -29,7 +29,6 @@
 #include <string>
 
 namespace Opm {
-    class Deck;
     class EclipseState;
     class ErrorGuard;
     class ParseContext;
@@ -61,6 +60,8 @@ enum class FileOutputMode {
 void
 ensureOutputDirExists(const std::string& cmdline_output_dir);
 
+std::unique_ptr<ParseContext> setupParseContext(const bool exitOnAllErrors);
+
 // Setup the OpmLog backends
 FileOutputMode
 setupLogging(int                mpi_rank_,
@@ -77,16 +78,14 @@ setupLogging(int                mpi_rank_,
 /// are created and can be used outside later.
 void readDeck(Parallel::Communication         comm,
               const std::string&              deckFilename,
-              std::shared_ptr<Deck>&          deck,
               std::shared_ptr<EclipseState>&  eclipseState,
               std::shared_ptr<Schedule>&      schedule,
               std::unique_ptr<UDQState>&      udqState,
               std::unique_ptr<Action::State>& actionState,
               std::unique_ptr<WellTestState>& wtestState,
               std::shared_ptr<SummaryConfig>& summaryConfig,
-              std::unique_ptr<ErrorGuard>     errorGuard,
               std::shared_ptr<Python>         python,
-              std::unique_ptr<ParseContext>   parseContext,
+              const std::string&              parsingStrictness,
               bool                            initFromRestart,
               bool                            checkDeck,
               const std::optional<int>&       outputInterval);
