@@ -28,6 +28,7 @@
 #include <opm/common/OpmLog/OpmLog.hpp>
 
 #include <cassert>
+#include <cstddef>
 #include <map>
 #include <memory>
 #include <optional>
@@ -37,10 +38,6 @@
 #include <unordered_map>
 #include <vector>
 
-#include <stddef.h>
-
-#include <opm/input/eclipse/EclipseState/Runspec.hpp>
-
 #include <opm/input/eclipse/Schedule/Schedule.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellTestState.hpp>
 #include <opm/input/eclipse/Schedule/Group/GuideRate.hpp>
@@ -49,6 +46,7 @@
 #include <opm/simulators/timestepping/SimulatorReport.hpp>
 #include <opm/simulators/flow/countGlobalCells.hpp>
 #include <opm/simulators/wells/BlackoilWellModelGeneric.hpp>
+#include <opm/simulators/wells/BlackoilWellModelGuideRates.hpp>
 #include <opm/simulators/wells/GasLiftSingleWell.hpp>
 #include <opm/simulators/wells/GasLiftWellState.hpp>
 #include <opm/simulators/wells/GasLiftSingleWellGeneric.hpp>
@@ -185,6 +183,7 @@ namespace Opm {
 
             void beginEpisode()
             {
+                OPM_TIMEBLOCK(beginEpsiode);
                 beginReportStep(ebosSimulator_.episodeIndex());
             }
 
@@ -192,6 +191,7 @@ namespace Opm {
 
             void beginIteration()
             {
+                OPM_TIMEBLOCK(beginIteration);
                 assemble(ebosSimulator_.model().newtonMethod().numIterations(),
                          ebosSimulator_.timeStepSize());
             }
@@ -201,6 +201,7 @@ namespace Opm {
 
             void endTimeStep()
             {
+                OPM_TIMEBLOCK(endTimeStep);
                 timeStepSucceeded(ebosSimulator_.time(), ebosSimulator_.timeStepSize());
             }
 
@@ -230,6 +231,13 @@ namespace Opm {
                                     param_.use_multisegment_well_);
             }
 
+            using BlackoilWellModelGeneric::prepareDeserialize;
+            void prepareDeserialize(const int report_step)
+            {
+                prepareDeserialize(report_step, grid().size(0),
+                                   param_.use_multisegment_well_);
+            }
+
             data::Wells wellData() const
             {
                 auto wsrpt = this->wellState()
@@ -241,7 +249,7 @@ namespace Opm {
 
                 this->assignWellTracerRates(wsrpt);
 
-                this->assignWellGuideRates(wsrpt, this->reportStepIndex());
+                BlackoilWellModelGuideRates(*this).assignWellGuideRates(wsrpt, this->reportStepIndex());
                 this->assignShutConnections(wsrpt, this->reportStepIndex());
 
                 return wsrpt;
@@ -266,6 +274,10 @@ namespace Opm {
 
             void addWellContributions(SparseMatrixAdapter& jacobian) const;
 
+            // add source from wells to the reservoir matrix
+            void addReservoirSourceTerms(GlobalEqVector& residual,
+                                         std::vector<typename SparseMatrixAdapter::MatrixBlock*>& diagMatAddress) const;
+
             // called at the beginning of a report step
             void beginReportStep(const int time_step);
 
@@ -280,7 +292,6 @@ namespace Opm {
             // at the beginning of each time step (Not report step)
             void prepareTimeStep(DeferredLogger& deferred_logger);
             void initPrimaryVariablesEvaluation() const;
-            bool shouldBalanceNetwork(const int reportStepIndex, const int iterationIdx) const;
             std::tuple<bool, bool, double> updateWellControls(DeferredLogger& deferred_logger);
 
             void updateAndCommunicate(const int reportStepIdx,
@@ -319,7 +330,7 @@ namespace Opm {
             Simulator& ebosSimulator_;
 
             // a vector of all the wells.
-            std::vector<WellInterfacePtr > well_container_{};
+            std::vector<WellInterfacePtr> well_container_{};
 
             std::vector<bool> is_cell_perforated_{};
 
@@ -342,15 +353,15 @@ namespace Opm {
 
 
             const ModelParameters param_;
-            size_t global_num_cells_{};
+            std::size_t global_num_cells_{};
             // the number of the cells in the local grid
-            size_t local_num_cells_{};
+            std::size_t local_num_cells_{};
             double gravity_{};
             std::vector<double> depth_{};
             bool alternative_well_rate_init_{};
 
             std::unique_ptr<RateConverterType> rateConverter_{};
-            std::unique_ptr<AverageRegionalPressureType> regionalAveragePressureCalculator_{};
+            std::map<std::string, std::unique_ptr<AverageRegionalPressureType>> regionalAveragePressureCalculator_{};
 
 
             SimulatorReportSingle last_report_{};
@@ -437,6 +448,7 @@ namespace Opm {
 
             void calcRates(const int fipnum,
                            const int pvtreg,
+                           const std::vector<double>& production_rates,
                            std::vector<double>& resv_coeff) override;
 
             void calcInjRates(const int fipnum,
