@@ -47,6 +47,7 @@ namespace Opm {
     class EclipseState;
     class GasLiftSingleWellGeneric;
     class GasLiftWellState;
+    class GasLiftGroupInfo;
     class Group;
     class GuideRateConfig;
     class ParallelWellInfo;
@@ -89,7 +90,7 @@ public:
 
     /// return true if wells are available in the reservoir
     bool wellsActive() const;
-    bool hasWell(const std::string& wname);
+    bool hasWell(const std::string& wname) const;
 
     // whether there exists any multisegment well open on this process
     bool anyMSWellOpenLocal() const;
@@ -99,6 +100,8 @@ public:
     const Schedule& schedule() const { return schedule_; }
     const PhaseUsage& phaseUsage() const { return phase_usage_; }
     const GroupState& groupState() const { return this->active_wgstate_.group_state; }
+    std::vector<const WellInterfaceGeneric*> genericWells() const
+    { return {well_container_generic_.begin(), well_container_generic_.end()}; }
 
     /*
       Immutable version of the currently active wellstate.
@@ -130,18 +133,15 @@ public:
                         const std::unordered_set<std::string>& wells,
                         const SummaryState& st);
 
-
-    void loadRestartData(const data::Wells& rst_wells,
-                         const data::GroupAndNetworkValues& grpNwrkValues,
-                         const PhaseUsage& phases,
-                         const bool handle_ms_well,
-                         WellState& well_state);
-
     void initFromRestartFile(const RestartValue& restartValues,
                              WellTestState wtestState,
                              const size_t numCells,
                              bool handle_ms_well);
 
+    void prepareDeserialize(int report_step,
+                            const size_t numCells,
+                            bool handle_ms_well);
+
     /*
       Will assign the internal member last_valid_well_state_ to the
       current value of the this->active_well_state_. The state stored
@@ -163,6 +163,58 @@ public:
     bool forceShutWellByName(const std::string& wellname,
                              const double simulation_time);
 
+    const std::vector<PerforationData>& perfData(const int well_idx) const
+    { return well_perf_data_[well_idx]; }
+
+    const Parallel::Communication& comm() const { return comm_; }
+
+    const SummaryState& summaryState() const { return summaryState_; }
+
+    const GuideRate& guideRate() const { return guideRate_; }
+
+    bool reportStepStarts() const { return report_step_starts_; }
+
+    bool shouldBalanceNetwork(const int reportStepIndex,
+                              const int iterationIdx) const;
+
+    bool shouldIterateNetwork(const int reportStepIndex,
+                              const std::size_t recursion_level,
+                              const double network_imbalance) const;
+
+    template<class Serializer>
+    void serializeOp(Serializer& serializer)
+    {
+        serializer(initial_step_);
+        serializer(report_step_starts_);
+        serializer(last_run_wellpi_);
+        serializer(local_shut_wells_);
+        serializer(closed_this_step_);
+        serializer(guideRate_);
+        serializer(node_pressures_);
+        serializer(active_wgstate_);
+        serializer(last_valid_wgstate_);
+        serializer(nupcol_wgstate_);
+        serializer(last_glift_opt_time_);
+        serializer(switched_prod_groups_);
+        serializer(switched_inj_groups_);
+    }
+
+    bool operator==(const BlackoilWellModelGeneric& rhs) const
+    {
+        return this->initial_step_ == rhs.initial_step_ &&
+               this->report_step_starts_ == rhs.report_step_starts_ &&
+               this->last_run_wellpi_ == rhs.last_run_wellpi_ &&
+               this->local_shut_wells_ == rhs.local_shut_wells_ &&
+               this->closed_this_step_ == rhs.closed_this_step_ &&
+               this->node_pressures_ == rhs.node_pressures_ &&
+               this->active_wgstate_ == rhs.active_wgstate_ &&
+               this->last_valid_wgstate_ == rhs.last_valid_wgstate_ &&
+               this->nupcol_wgstate_ == rhs.nupcol_wgstate_ &&
+               this->last_glift_opt_time_ == rhs.last_glift_opt_time_ &&
+               this->switched_prod_groups_ == rhs.switched_prod_groups_ &&
+               this->switched_inj_groups_ == rhs.switched_inj_groups_;
+    }
+
 protected:
 
     /*
@@ -260,76 +312,22 @@ protected:
                      double wsolvent);
     virtual void calcRates(const int fipnum,
                            const int pvtreg,
+                           const std::vector<double>& production_rates,
                            std::vector<double>& resv_coeff) = 0;
     virtual void calcInjRates(const int fipnum,
                            const int pvtreg,
                            std::vector<double>& resv_coeff) = 0;
 
-    data::GuideRateValue getGuideRateValues(const Group& group) const;
-    data::GuideRateValue getGuideRateValues(const Well& well) const;
-    data::GuideRateValue getGuideRateInjectionGroupValues(const Group& group) const;
-    void getGuideRateValues(const GuideRate::RateVector& qs,
-                            const bool                   is_inj,
-                            const std::string&           wgname,
-                            data::GuideRateValue&        grval) const;
-
-    void assignWellGuideRates(data::Wells& wsrpt,
-                              const int reportStepIdx) const;
     void assignShutConnections(data::Wells& wsrpt,
                                const int reportStepIndex) const;
     void assignGroupControl(const Group& group,
                             data::GroupData& gdata) const;
-    void assignGroupGuideRates(const Group& group,
-                               const std::unordered_map<std::string, data::GroupGuideRates>& groupGuideRates,
-                               data::GroupData& gdata) const;
     void assignGroupValues(const int reportStepIdx,
                            std::map<std::string, data::GroupData>& gvalues) const;
     void assignNodeValues(std::map<std::string, data::NodeData>& nodevalues) const;
 
-    void loadRestartConnectionData(const std::vector<data::Rates::opt>& phs,
-                                   const data::Well&                    rst_well,
-                                   const std::vector<PerforationData>&  old_perf_data,
-                                   SingleWellState&                     ws);
-
-    void loadRestartSegmentData(const std::string&                   well_name,
-                                const std::vector<data::Rates::opt>& phs,
-                                const data::Well&                    rst_well,
-                                SingleWellState&                     ws);
-
-    void loadRestartWellData(const std::string&                   well_name,
-                             const bool                           handle_ms_well,
-                             const std::vector<data::Rates::opt>& phs,
-                             const data::Well&                    rst_well,
-                             const std::vector<PerforationData>&  old_perf_data,
-                             SingleWellState&                     ws);
-
-    void loadRestartGroupData(const std::string&     group,
-                              const data::GroupData& value);
-
-    void loadRestartGuideRates(const int                    report_step,
-                               const GuideRateModel::Target target,
-                               const data::Wells&           rst_wells);
-
-    void loadRestartGuideRates(const int                                     report_step,
-                               const GuideRateConfig&                        config,
-                               const std::map<std::string, data::GroupData>& rst_groups);
-
-    std::unordered_map<std::string, data::GroupGuideRates>
-    calculateAllGroupGuiderates(const int reportStepIdx) const;
-
     void calculateEfficiencyFactors(const int reportStepIdx);
 
-    bool checkGroupConstraints(const Group& group,
-                               const int reportStepIdx,
-                               DeferredLogger& deferred_logger) const;
-
-    std::pair<Group::InjectionCMode, double> checkGroupInjectionConstraints(const Group& group,
-                                                         const int reportStepIdx,
-                                                         const Phase& phase) const;
-    std::pair<Group::ProductionCMode, double> checkGroupProductionConstraints(const Group& group,
-                                                           const int reportStepIdx,
-                                                           DeferredLogger& deferred_logger) const;
-
     void checkGconsaleLimits(const Group& group,
                              WellState& well_state,
                              const int reportStepIdx,
@@ -339,19 +337,6 @@ protected:
                                      DeferredLogger& deferred_logger,
                                      const int reportStepIdx);
 
-    bool updateGroupIndividualControl(const Group& group,
-                                      DeferredLogger& deferred_logger,
-                                      const int reportStepIdx);
-
-    void actionOnBrokenConstraints(const Group& group,
-                                   const Group::ExceedAction& exceed_action,
-                                   const Group::ProductionCMode& newControl,
-                                   DeferredLogger& deferred_logger);
-    void actionOnBrokenConstraints(const Group& group,
-                                   const Group::InjectionCMode& newControl,
-                                   const Phase& controlPhase,
-                                   DeferredLogger& deferred_logger);
-
     void updateAndCommunicateGroupData(const int reportStepIdx,
                                        const int iterationIdx);
 
@@ -367,6 +352,7 @@ protected:
     void gasLiftOptimizationStage2(DeferredLogger& deferred_logger,
                                    GLiftProdWells& prod_wells,
                                    GLiftOptWells& glift_wells,
+                                   GasLiftGroupInfo& group_info,
                                    GLiftWellStateMap& map,
                                    const int episodeIndex);
 
@@ -382,8 +368,6 @@ protected:
                               const SummaryConfig& summaryConfig,
                               DeferredLogger& deferred_logger);
 
-    bool guideRateUpdateIsNeeded(const int reportStepIdx) const;
-
     // create the well container
     virtual void createWellContainer(const int time_step) = 0;
     virtual void initWellContainer(const int reportStepIdx) = 0;
@@ -398,6 +382,10 @@ protected:
     /// \brief get compressed index for interior cells (-1, otherwise
     virtual int compressedIndexForInterior(int cartesian_cell_idx) const = 0;
 
+    std::vector<int> getCellsForConnections(const Well& well) const;
+
+    std::vector<std::string> getWellsForTesting(const int timeStepIdx,
+                                                const double simulationTime);
 
     Schedule& schedule_;
     const SummaryState& summaryState_;
