@@ -23,8 +23,10 @@
 #include <opm/simulators/utils/DeferredLoggingErrorHelpers.hpp>
 #include <opm/core/props/phaseUsageFromDeck.hpp>
 #include <opm/grid/utility/cartesianToCompressed.hpp>
+
 #include <opm/input/eclipse/Units/UnitSystem.hpp>
 
+#include <opm/simulators/wells/BlackoilWellModelConstraints.hpp>
 #include <opm/simulators/wells/VFPProperties.hpp>
 #include <opm/simulators/utils/MPIPacker.hpp>
 #include <opm/simulators/linalg/bda/WellContributions.hpp>
@@ -126,20 +128,7 @@ namespace Opm {
         // initialize the additional cell connections introduced by wells.
         for (const auto& well : schedule_wells)
         {
-            std::vector<int> wellCells;
-            // All possible connections of the well
-            const auto& connectionSet = well.getConnections();
-            wellCells.reserve(connectionSet.size());
-
-            for ( size_t c=0; c < connectionSet.size(); c++ )
-            {
-                const auto& connection = connectionSet.get(c);
-                int compressed_idx = compressedIndexForInterior(connection.global_index());
-                if ( compressed_idx >= 0 ) { // Ignore connections in inactive/remote cells.
-                    wellCells.push_back(compressed_idx);
-                }
-            }
-
+            std::vector<int> wellCells = this->getCellsForConnections(well);
             for (int cellIdx : wellCells) {
                 neighbors[cellIdx].insert(wellCells.begin(),
                                           wellCells.end());
@@ -213,21 +202,22 @@ namespace Opm {
             WellGroupHelpers::setCmodeGroup(fieldGroup, schedule(), summaryState, timeStepIdx, this->wellState(), this->groupState());
 
             // Compute reservoir volumes for RESV controls.
-            rateConverter_.reset(new RateConverterType (phase_usage_,
-                                                        std::vector<int>(local_num_cells_, 0)));
+            rateConverter_ = std::make_unique<RateConverterType>(phase_usage_,
+                                                                 std::vector<int>(local_num_cells_, 0));
             rateConverter_->template defineState<ElementContext>(ebosSimulator_);
 
             // Compute regional average pressures used by gpmaint
             if (schedule_[timeStepIdx].has_gpmaint()) {
-                const auto& fp = this->eclState_.fieldProps();
-                const auto& fipnum = fp.get_int("FIPNUM");
-                regionalAveragePressureCalculator_.reset(new AverageRegionalPressureType (phase_usage_,fipnum));
+                WellGroupHelpers::setRegionAveragePressureCalculator(fieldGroup, schedule(),
+                        timeStepIdx, this->eclState_.fieldProps(), phase_usage_, regionalAveragePressureCalculator_);
             }
 
             {
                 const auto& sched_state = this->schedule()[timeStepIdx];
                 // update VFP properties
-                vfp_properties_.reset(new VFPProperties( sched_state.vfpinj(), sched_state.vfpprod(), this->prevWellState()));
+                vfp_properties_ = std::make_unique<VFPProperties>(sched_state.vfpinj(),
+                                                                  sched_state.vfpprod(),
+                                                                  this->prevWellState());
                 this->initializeWellProdIndCalculators();
                 if (sched_state.events().hasEvent(ScheduleEvents::Events::WELL_PRODUCTIVITY_INDEX)) {
                     this->runWellPIScaling(timeStepIdx, local_deferredLogger);
@@ -247,6 +237,7 @@ namespace Opm {
     BlackoilWellModel<TypeTag>::
     beginTimeStep()
     {
+        OPM_TIMEBLOCK(beginTimeStep);
         updatePerforationIntensiveQuantities();
         updateAverageFormationFactor();
         DeferredLogger local_deferredLogger;
@@ -341,10 +332,12 @@ namespace Opm {
         auto exc_type = ExceptionType::NONE;
         // update gpmaint targets
         if (schedule_[reportStepIdx].has_gpmaint()) {
-            regionalAveragePressureCalculator_->template defineState<ElementContext>(ebosSimulator_);
+            for (auto& calculator : regionalAveragePressureCalculator_) {
+                calculator.second->template defineState<ElementContext>(ebosSimulator_);
+            }
             const double dt = ebosSimulator_.timeStepSize();
             WellGroupHelpers::updateGpMaintTargetForGroups(fieldGroup,
-                                                           schedule_, *regionalAveragePressureCalculator_, reportStepIdx, dt, this->wellState(), this->groupState());
+                                                           schedule_, regionalAveragePressureCalculator_, reportStepIdx, dt, this->wellState(), this->groupState());
         }
         try {
             // Compute initial well solution for new wells and injectors that change injection type i.e. WAG.
@@ -390,30 +383,24 @@ namespace Opm {
                                             const double simulationTime,
                                             DeferredLogger& deferred_logger)
     {
-        const auto& wtest_config = schedule()[timeStepIdx].wtest_config();
-        if (!wtest_config.empty()) { // there is a WTEST request
-            const std::vector<std::string> wellsForTesting = wellTestState()
-                .test_wells(wtest_config, simulationTime);
-            for (const std::string& well_name : wellsForTesting) {
+        for (const std::string& well_name : this->getWellsForTesting(timeStepIdx, simulationTime)) {
+            const Well& wellEcl = schedule().getWell(well_name, timeStepIdx);
+            if (wellEcl.getStatus() == Well::Status::SHUT)
+                continue;
 
-                const Well& wellEcl = schedule().getWell(well_name, timeStepIdx);
-                if (wellEcl.getStatus() == Well::Status::SHUT)
-                    continue;
+            WellInterfacePtr well = createWellForWellTest(well_name, timeStepIdx, deferred_logger);
+            // some preparation before the well can be used
+            well->init(&phase_usage_, depth_, gravity_, local_num_cells_, B_avg_, true);
 
-                WellInterfacePtr well = createWellForWellTest(well_name, timeStepIdx, deferred_logger);
-                // some preparation before the well can be used
-                well->init(&phase_usage_, depth_, gravity_, local_num_cells_, B_avg_, true);
+            double well_efficiency_factor = wellEcl.getEfficiencyFactor();
+            WellGroupHelpers::accumulateGroupEfficiencyFactor(schedule().getGroup(wellEcl.groupName(), timeStepIdx),
+                                                              schedule(), timeStepIdx, well_efficiency_factor);
 
-                double well_efficiency_factor = wellEcl.getEfficiencyFactor();
-                WellGroupHelpers::accumulateGroupEfficiencyFactor(schedule().getGroup(wellEcl.groupName(), timeStepIdx),
-                                                                  schedule(), timeStepIdx, well_efficiency_factor);
+            well->setWellEfficiencyFactor(well_efficiency_factor);
+            well->setVFPProperties(vfp_properties_.get());
+            well->setGuideRate(&guideRate_);
 
-                well->setWellEfficiencyFactor(well_efficiency_factor);
-                well->setVFPProperties(vfp_properties_.get());
-                well->setGuideRate(&guideRate_);
-
-                well->wellTesting(ebosSimulator_, simulationTime, this->wellState(), this->groupState(), wellTestState(), deferred_logger);
-            }
+            well->wellTesting(ebosSimulator_, simulationTime, this->wellState(), this->groupState(), wellTestState(), deferred_logger);
         }
     }
 
@@ -628,7 +615,7 @@ namespace Opm {
             for (int w = 0; w < nw; ++w) {
                 const Well& well_ecl = wells_ecl_[w];
 
-                if (well_ecl.getConnections().empty()) {
+                if (!well_ecl.hasConnections()) {
                     // No connections in this well.  Nothing to do.
                     continue;
                 }
@@ -833,7 +820,9 @@ namespace Opm {
         }
         // It should be able to find in wells_ecl.
         if (index_well_ecl == nw_wells_ecl) {
-            OPM_DEFLOG_THROW(std::logic_error, "Could not find well " << well_name << " in wells_ecl ", deferred_logger);
+            OPM_DEFLOG_THROW(std::logic_error,
+                             fmt::format("Could not find well {} in wells_ecl ", well_name),
+                             deferred_logger);
         }
 
         return this->createWellPointer(index_well_ecl, report_step);
@@ -876,8 +865,9 @@ namespace Opm {
                 calculateExplicitQuantities(local_deferredLogger);
                 prepareTimeStep(local_deferredLogger);
             }
-            OPM_END_PARALLEL_TRY_CATCH_LOG(local_deferredLogger, "assemble() failed (It=0): ",
-                                               terminal_output_, grid().comm());
+            OPM_END_PARALLEL_TRY_CATCH_LOG(local_deferredLogger,
+                                           "assemble() failed (It=0): ",
+                                           terminal_output_, grid().comm());
         }
 
         const bool well_group_control_changed = assembleImpl(iterationIdx, dt, 0, local_deferredLogger);
@@ -917,7 +907,8 @@ namespace Opm {
 
         //update guide rates
         const int reportStepIdx = ebosSimulator_.episodeIndex();
-        if (alq_updated || guideRateUpdateIsNeeded(reportStepIdx)) {
+        if (alq_updated || BlackoilWellModelGuideRates(*this).
+                           guideRateUpdateIsNeeded(reportStepIdx)) {
             const double simulationTime = ebosSimulator_.time();
             const auto& comm = ebosSimulator_.vanguard().grid().comm();
             const auto& summaryState = ebosSimulator_.vanguard().summaryState();
@@ -929,12 +920,9 @@ namespace Opm {
 
         // Maybe do a recursive call to iterate network and well controls.
         if (network_changed) {
-            if (shouldBalanceNetwork(reportStepIdx, iterationIdx)) {
-                const auto& balance = schedule()[reportStepIdx].network_balance();
-                // Iterate if not converged, and number of iterations is not yet max (NETBALAN item 3).
-                if (recursion_level < balance.pressure_max_iter() && network_imbalance > balance.pressure_tolerance()) {
-                    well_group_control_changed = assembleImpl(iterationIdx, dt, recursion_level + 1, local_deferredLogger);
-                }
+            if (shouldBalanceNetwork(reportStepIdx, iterationIdx) &&
+                shouldIterateNetwork(reportStepIdx, recursion_level, network_imbalance)) {
+                well_group_control_changed = assembleImpl(iterationIdx, dt, recursion_level + 1, local_deferredLogger);
             }
         }
         return well_group_control_changed;
@@ -984,6 +972,7 @@ namespace Opm {
                 phase_usage_,
                 deferred_logger,
                 this->wellState(),
+                this->groupState(),
                 ebosSimulator_.vanguard().grid().comm(),
                 this->glift_debug
             };
@@ -991,7 +980,7 @@ namespace Opm {
             gasLiftOptimizationStage1(
                 deferred_logger, prod_wells, glift_wells, group_info, state_map);
             gasLiftOptimizationStage2(
-                deferred_logger, prod_wells, glift_wells, state_map,
+                deferred_logger, prod_wells, glift_wells, group_info, state_map,
                 ebosSimulator_.episodeIndex());
             if (this->glift_debug) gliftDebugShowALQ(deferred_logger);
             num_wells_changed = glift_wells.size();
@@ -1050,10 +1039,6 @@ namespace Opm {
                 }
                 num_rates_to_sync = groups_to_sync.size();
             }
-            // Since "group_info" is not used in stage2, there is no need to
-            //   communicate rates if this is the last iteration...
-            if (i == (num_procs - 1))
-                break;
             num_rates_to_sync = comm.sum(num_rates_to_sync);
             if (num_rates_to_sync > 0) {
                 std::vector<int> group_indexes;
@@ -1082,14 +1067,6 @@ namespace Opm {
                     group_water_rates.resize(num_rates_to_sync);
                     group_alq_rates.resize(num_rates_to_sync);
                 }
-                // TODO: We only need to broadcast to processors with index
-                //   j > i since we do not use the "group_info" in stage 2. In
-                //   this case we should use comm.send() and comm.receive()
-                //   instead of comm.broadcast() to communicate with specific
-                //   processes, and these processes only need to receive the
-                //   data if they are going to check the group rates in stage1
-                //   Another similar idea is to only communicate the rates to
-                //   process j = i + 1
 #if HAVE_MPI
                 EclMpiSerializer ser(comm);
                 ser.broadcast(i, group_indexes, group_oil_rates,
@@ -1200,9 +1177,7 @@ namespace Opm {
             auto& well = well_container_[i];
             std::shared_ptr<StandardWell<TypeTag> > derived = std::dynamic_pointer_cast<StandardWell<TypeTag> >(well);
             if (derived) {
-                unsigned int numBlocks;
-                derived->getNumBlocks(numBlocks);
-                wellContribs.addNumBlocks(numBlocks);
+                wellContribs.addNumBlocks(derived->linSys().getNumBlocks());
             }
         }
 
@@ -1212,13 +1187,13 @@ namespace Opm {
         for(unsigned int i = 0; i < well_container_.size(); i++){
             auto& well = well_container_[i];
             // maybe WellInterface could implement addWellContribution()
-            auto derived_std = std::dynamic_pointer_cast<StandardWell<TypeTag> >(well);
+            auto derived_std = std::dynamic_pointer_cast<StandardWell<TypeTag>>(well);
             if (derived_std) {
-                derived_std->addWellContribution(wellContribs);
+                derived_std->linSys().extract(derived_std->numStaticWellEq, wellContribs);
             } else {
                 auto derived_ms = std::dynamic_pointer_cast<MultisegmentWell<TypeTag> >(well);
                 if (derived_ms) {
-                    derived_ms->addWellContribution(wellContribs);
+                    derived_ms->linSys().extract(wellContribs);
                 } else {
                     OpmLog::warning("Warning unknown type of well");
                 }
@@ -1274,6 +1249,35 @@ namespace Opm {
         }
     }
 
+    template <typename TypeTag>
+    void BlackoilWellModel<TypeTag>::
+    addReservoirSourceTerms(GlobalEqVector& residual,
+                            std::vector<typename SparseMatrixAdapter::MatrixBlock*>& diagMatAddress) const
+    {
+        // NB this loop may write multiple times to the same element
+        // if a cell is perforated by more than one well, so it should
+        // not be OpenMP-parallelized.
+        for (const auto& well : well_container_) {
+            if (!well->isOperableAndSolvable() && !well->wellIsStopped()) {
+                continue;
+            }
+            const auto& cells = well->cells();
+            const auto& rates = well->connectionRates();
+            for (unsigned perfIdx = 0; perfIdx < rates.size(); ++perfIdx) {
+                unsigned cellIdx = cells[perfIdx];
+                auto rate = rates[perfIdx];
+                rate *= -1.0;
+                VectorBlockType res(0.0);
+                using MatrixBlockType = typename SparseMatrixAdapter::MatrixBlock;
+                MatrixBlockType bMat(0.0);
+                ebosSimulator_.model().linearizer().setResAndJacobi(res, bMat, rate);
+                residual[cellIdx] += res;
+                *diagMatAddress[cellIdx] += bMat;
+            }
+        }
+    }
+
+
     template<typename TypeTag>
     int
     BlackoilWellModel<TypeTag>::
@@ -1298,19 +1302,7 @@ namespace Opm {
         // initialize the additional cell connections introduced by wells.
         for ( const auto& well : schedule_wells )
         {
-            std::vector<int> compressed_well_perforations;
-            // All possible completions of the well
-            const auto& completionSet = well.getConnections();
-            compressed_well_perforations.reserve(completionSet.size());
-
-            for (const auto& connection: well.getConnections())
-            {
-                const int compressed_idx = compressedIndexForInterior(connection.global_index());
-                if ( compressed_idx >= 0 ) // Ignore completions in inactive/remote cells.
-                {
-                    compressed_well_perforations.push_back(compressed_idx);
-                }
-            }
+            std::vector<int> compressed_well_perforations = this->getCellsForConnections(well);
 
             // also include wells with no perforations in case
             std::sort(compressed_well_perforations.begin(),
@@ -1361,7 +1353,8 @@ namespace Opm {
         OPM_BEGIN_PARALLEL_TRY_CATCH();
         {
             for (auto& well : well_container_) {
-                well->recoverWellSolutionAndUpdateWellState(x, this->wellState(), local_deferredLogger);
+                const auto& summary_state = ebosSimulator_.vanguard().summaryState();
+                well->recoverWellSolutionAndUpdateWellState(summary_state, x, this->wellState(), local_deferredLogger);
             }
 
         }
@@ -1454,30 +1447,6 @@ namespace Opm {
 
 
 
-    template<typename TypeTag>
-    bool
-    BlackoilWellModel<TypeTag>::
-    shouldBalanceNetwork(const int reportStepIdx, const int iterationIdx) const
-    {
-        const auto& balance = schedule()[reportStepIdx].network_balance();
-        if (balance.mode() == Network::Balance::CalcMode::TimeStepStart) {
-            return iterationIdx == 0;
-        } else if (balance.mode() == Network::Balance::CalcMode::NUPCOL) {
-            const int nupcol = schedule()[reportStepIdx].nupcol();
-            return iterationIdx < nupcol;
-        } else {
-            // We do not support any other rebalancing modes,
-            // i.e. TimeInterval based rebalancing is not available.
-            // This should be warned about elsewhere, so we choose to
-            // avoid spamming with a warning here.
-            return false;
-        }
-    }
-
-
-
-
-
     template<typename TypeTag>
     std::tuple<bool, bool, double>
     BlackoilWellModel<TypeTag>::
@@ -1576,7 +1545,15 @@ namespace Opm {
             changed = true;
             updateAndCommunicate(reportStepIdx, iterationIdx, deferred_logger);
         }
-        bool changed_individual =  updateGroupIndividualControl( group, deferred_logger, reportStepIdx);
+        bool changed_individual =
+            BlackoilWellModelConstraints(*this).
+                updateGroupIndividualControl(group,
+                                             reportStepIdx,
+                                             this->switched_inj_groups_,
+                                             this->switched_prod_groups_,
+                                             this->groupState(),
+                                             this->wellState(),
+                                             deferred_logger);
         if (changed_individual) {
             changed = true;
             updateAndCommunicate(reportStepIdx, iterationIdx, deferred_logger);
@@ -1668,7 +1645,7 @@ namespace Opm {
         // corresponding "this->wells_ecl_[shutWell]".
 
         for (const auto& shutWell : this->local_shut_wells_) {
-            if (this->wells_ecl_[shutWell].getConnections().empty()) {
+            if (!this->wells_ecl_[shutWell].hasConnections()) {
                 // No connections in this well.  Nothing to do.
                 continue;
             }
@@ -1712,7 +1689,8 @@ namespace Opm {
             auto& events = this->wellState().well(well->indexOfWell()).events;
             if (events.hasEvent(WellState::event_mask)) {
                 well->updateWellStateWithTarget(ebosSimulator_, this->groupState(), this->wellState(), deferred_logger);
-                well->updatePrimaryVariables(this->wellState(), deferred_logger);
+                const auto& summary_state = ebosSimulator_.vanguard().summaryState();
+                well->updatePrimaryVariables(summary_state, this->wellState(), deferred_logger);
                 well->initPrimaryVariablesEvaluation();
                 // There is no new well control change input within a report step,
                 // so next time step, the well does not consider to have effective events anymore.
@@ -1786,7 +1764,8 @@ namespace Opm {
     updatePrimaryVariables(DeferredLogger& deferred_logger)
     {
         for (const auto& well : well_container_) {
-            well->updatePrimaryVariables(this->wellState(), deferred_logger);
+            const auto& summary_state = ebosSimulator_.vanguard().summaryState();
+            well->updatePrimaryVariables(summary_state, this->wellState(), deferred_logger);
         }
     }
 
@@ -1909,9 +1888,10 @@ namespace Opm {
     BlackoilWellModel<TypeTag>::
     calcRates(const int fipnum,
               const int pvtreg,
+              const std::vector<double>& production_rates,
               std::vector<double>& resv_coeff)
     {
-        rateConverter_->calcCoeff(fipnum, pvtreg, resv_coeff);
+        rateConverter_->calcCoeff(fipnum, pvtreg, production_rates, resv_coeff);
     }
 
     template<typename TypeTag>
@@ -1960,7 +1940,7 @@ namespace Opm {
 
             for (int perf = 0; perf < num_perf_this_well; ++perf) {
                 const int cell_idx = well_perf_data_[wellID][perf].cell_index;
-                const auto& intQuants = *(ebosSimulator_.model().cachedIntensiveQuantities(cell_idx, /*timeIdx=*/0));
+                const auto& intQuants = ebosSimulator_.model().intensiveQuantities(cell_idx, /*timeIdx=*/0);
                 const auto& fs = intQuants.fluidState();
 
                 // we on only have one temperature pr cell any phaseIdx will do
