@@ -30,7 +30,6 @@
 #include <optional>
 #include <vector>
 #include <utility>
-#include <fmt/format.h>
 
 namespace Opm
 {
@@ -56,10 +55,10 @@ namespace Opm
         const WellInterfaceGeneric &getWell() const override { return well_; }
 
     private:
-        std::optional<double> computeBhpAtThpLimit_(double alq) const override;
+        std::optional<double> computeBhpAtThpLimit_(double alq, bool debug_ouput=true) const override;
         BasicRates computeWellRates_(
             double bhp, bool bhp_is_limited, bool debug_output=true) const override;
-        void setAlqMaxRate_(const GasLiftOpt::Well& well);
+        void setAlqMaxRate_(const GasLiftWell& well);
         void setupPhaseVariables_();
         bool checkThpControl_() const override;
 
