@@ -25,10 +25,10 @@
 
 #include <opm/core/props/BlackoilPhases.hpp>
 
-#include <opm/input/eclipse/Schedule/GasLiftOpt.hpp>
-#include <opm/input/eclipse/Schedule/Well/Well.hpp>
+#include <opm/input/eclipse/Schedule/Well/WellProductionControls.hpp>
 #include <opm/simulators/wells/GasLiftGroupInfo.hpp>
 #include <opm/simulators/wells/GasLiftCommon.hpp>
+#include <opm/simulators/wells/GroupState.hpp>
 
 #include <functional>
 #include <optional>
@@ -41,6 +41,7 @@ namespace Opm
 {
 
 class DeferredLogger;
+class GasLiftWell;
 class GasLiftWellState;
 class Schedule;
 class SummaryState;
@@ -93,7 +94,12 @@ public:
     const std::string& name() const { return well_name_; }
 
     std::optional<GradInfo> calcIncOrDecGradient(double oil_rate, double gas_rate,
-                                                 double alq, bool increase) const;
+                                                 double water_rate,
+                                                 double alq,
+                                                 const std::string& gr_name_dont_limit,
+                                                 bool increase,
+                                                 bool debug_output = true
+                                                 ) const;
 
     std::unique_ptr<GasLiftWellState> runOptimize(const int iteration_idx);
 
@@ -237,7 +243,7 @@ protected:
         double getBhpWithLimit();
         void warn_(std::string msg) {parent.displayWarning_(msg);}
     };
-    bool checkGroupALQrateExceeded(double delta_alq) const;
+    bool checkGroupALQrateExceeded(double delta_alq, const std::string& gr_name_dont_limit = "") const;
     bool checkGroupTotalRateExceeded(double delta_alq, double delta_gas_rate) const;
 
     std::pair<std::optional<double>, bool> addOrSubtractAlqIncrement_(
@@ -249,11 +255,11 @@ protected:
                       const BasicRates& rates, const BasicRates& new_rates) const;
     bool checkInitialALQmodified_(double alq, double initial_alq) const;
     virtual bool checkThpControl_() const = 0;
-    virtual std::optional<double> computeBhpAtThpLimit_(double alq) const = 0;
+    virtual std::optional<double> computeBhpAtThpLimit_(double alq, bool debug_output = true) const = 0;
     std::pair<std::optional<double>,double> computeConvergedBhpAtThpLimitByMaybeIncreasingALQ_() const;
     std::pair<std::optional<BasicRates>,double> computeInitialWellRates_() const;
     std::optional<LimitedRates> computeLimitedWellRatesWithALQ_(double alq) const;
-    virtual BasicRates computeWellRates_(double bhp, bool bhp_is_limited,                                                             bool debug_output = true) const = 0;
+    virtual BasicRates computeWellRates_(double bhp, bool bhp_is_limited, bool debug_output = true) const = 0;
     std::optional<BasicRates> computeWellRatesWithALQ_(double alq) const;
     void debugCheckNegativeGradient_(double grad, double alq, double new_alq,
                                      double oil_rate, double new_oil_rate,
@@ -272,14 +278,14 @@ protected:
     std::pair<double, bool> getGasRateWithLimit_(
                            const BasicRates& rates) const;
     std::pair<double, bool> getGasRateWithGroupLimit_(
-                           double new_gas_rate, double gas_rate) const;
+                           double new_gas_rate, double gas_rate, const std::string& gr_name_dont_limit) const;
     std::pair<std::optional<LimitedRates>,double> getInitialRatesWithLimit_() const;
     LimitedRates getLimitedRatesFromRates_(const BasicRates& rates) const;
     std::tuple<double,double,bool,bool> getLiquidRateWithGroupLimit_(
                            const double new_oil_rate, const double oil_rate,
-                           const double new_water_rate, const double water_rate) const;
+                           const double new_water_rate, const double water_rate, const std::string& gr_name_dont_limit) const;
     std::pair<double, bool> getOilRateWithGroupLimit_(
-                           double new_oil_rate, double oil_rate) const;
+                           double new_oil_rate, double oil_rate, const std::string& gr_name_dont_limit) const;
     std::pair<double, bool> getOilRateWithLimit_(const BasicRates& rates) const;
     std::pair<double, std::optional<Rate>> getOilRateWithLimit2_(
                            const BasicRates& rates) const;
@@ -288,9 +294,9 @@ protected:
     std::pair<double, std::optional<Rate>> getRateWithLimit_(
                     Rate rate_type, const BasicRates& rates) const;
     std::tuple<double, const std::string*, double> getRateWithGroupLimit_(
-                 Rate rate_type, const double new_rate, const double old_rate) const;
+                 Rate rate_type, const double new_rate, const double old_rate, const std::string& gr_name_dont_limit) const;
     std::pair<double, bool> getWaterRateWithGroupLimit_(
-                           double new_water_rate, double water_rate) const;
+                           double new_water_rate, double water_rate, const std::string& gr_name_dont_limit) const;
     std::pair<double, bool> getWaterRateWithLimit_(const BasicRates& rates) const;
     std::pair<double, std::optional<Rate>> getWaterRateWithLimit2_(
                            const BasicRates& rates) const;
@@ -313,7 +319,7 @@ protected:
     std::unique_ptr<GasLiftWellState> runOptimize1_();
     std::unique_ptr<GasLiftWellState> runOptimize2_();
     std::unique_ptr<GasLiftWellState> runOptimizeLoop_(bool increase);
-    void setAlqMinRate_(const GasLiftOpt::Well& well);
+    void setAlqMinRate_(const GasLiftWell& well);
     std::unique_ptr<GasLiftWellState> tryIncreaseLiftGas_();
     std::unique_ptr<GasLiftWellState> tryDecreaseLiftGas_();
     void updateGroupRates_(
@@ -321,20 +327,19 @@ protected:
         const LimitedRates& new_rates,
         double delta_alq) const;
     LimitedRates updateRatesToGroupLimits_(
-        const BasicRates& rates, const LimitedRates& new_rates) const;
-    void updateWellStateAlqFixedValue_(const GasLiftOpt::Well& well);
-    bool useFixedAlq_(const GasLiftOpt::Well& well);
+        const BasicRates& rates, const LimitedRates& new_rates, const std::string& gr_name = "") const;
+    void updateWellStateAlqFixedValue_(const GasLiftWell& well);
+    bool useFixedAlq_(const GasLiftWell& well);
     void debugInfoGroupRatesExceedTarget(
         Rate rate_type, const std::string& gr_name, double rate, double target) const;
     void warnMaxIterationsExceeded_();
 
-    const GroupState& group_state_;
     const Well& ecl_well_;
     const SummaryState& summary_state_;
     GasLiftGroupInfo& group_info_;
     const PhaseUsage& phase_usage_;
     GLiftSyncGroups& sync_groups_;
-    const Well::ProductionControls controls_;
+    const WellProductionControls controls_;
 
     double increment_;
     double max_alq_;
@@ -353,7 +358,7 @@ protected:
 
     std::string well_name_;
 
-    const GasLiftOpt::Well* gl_well_;
+    const GasLiftWell* gl_well_;
 
     bool optimize_;
     bool debug_limit_increase_decrease_;
