@@ -17,6 +17,9 @@
   along with OPM.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include <opm/input/eclipse/Schedule/GasLiftOpt.hpp>
+#include <fmt/format.h>
+
 namespace Opm {
 
 template<typename TypeTag>
@@ -125,20 +128,23 @@ computeWellRates_( double bhp, bool bhp_is_limited, bool debug_output ) const
 template<typename TypeTag>
 std::optional<double>
 GasLiftSingleWell<TypeTag>::
-computeBhpAtThpLimit_(double alq) const
+computeBhpAtThpLimit_(double alq, bool debug_output) const
 {
     auto bhp_at_thp_limit = this->well_.computeBhpAtThpLimitProdWithAlq(
         this->ebos_simulator_,
         this->summary_state_,
-        this->deferred_logger_,
-        alq);
+        alq,
+        this->deferred_logger_);
     if (bhp_at_thp_limit) {
         if (*bhp_at_thp_limit < this->controls_.bhp_limit) {
-            const std::string msg = fmt::format(
-                "Computed bhp ({}) from thp limit is below bhp limit ({}), (ALQ = {})."
-                " Using bhp limit instead",
-                *bhp_at_thp_limit, this->controls_.bhp_limit, alq);
-            displayDebugMessage_(msg);
+            if (debug_output && this->debug) {
+                const std::string msg = fmt::format(
+                    "Computed bhp ({}) from thp limit is below bhp limit ({}), (ALQ = {})."
+                    " Using bhp limit instead",
+                    *bhp_at_thp_limit, this->controls_.bhp_limit, alq
+                );
+                displayDebugMessage_(msg);
+            }
             bhp_at_thp_limit = this->controls_.bhp_limit;
         }
         //bhp_at_thp_limit = std::max(*bhp_at_thp_limit, this->controls_.bhp_limit);
@@ -157,7 +163,9 @@ GasLiftSingleWell<TypeTag>::
 setupPhaseVariables_()
 {
     const auto& pu = this->phase_usage_;
+#ifndef NDEBUG
     bool num_phases_ok = (pu.num_phases == 3);
+#endif
     if (pu.num_phases == 2) {
         // NOTE: We support two-phase oil-water flow, by setting the gas flow rate
         //   to zero. This is done by initializing the potential vector to zero:
@@ -173,7 +181,9 @@ setupPhaseVariables_()
              && pu.phase_used[BlackoilPhases::Liquid] == 1
              && pu.phase_used[BlackoilPhases::Vapour] == 0)
         {
+#ifndef NDEBUG
             num_phases_ok = true;  // two-phase oil-water is also supported
+#endif
         }
         else {
             throw std::logic_error("Two-phase gas lift optimization only supported"
@@ -189,7 +199,7 @@ setupPhaseVariables_()
 template<typename TypeTag>
 void
 GasLiftSingleWell<TypeTag>::
-setAlqMaxRate_(const GasLiftOpt::Well &well)
+setAlqMaxRate_(const GasLiftWell &well)
 {
     auto& max_alq_optional = well.max_rate();
     if (max_alq_optional) {
