@@ -21,6 +21,11 @@
 
 #include <opm/common/ErrorMacros.hpp>
 #include <opm/input/eclipse/Schedule/Well/Well.hpp>
+#include <opm/input/eclipse/Schedule/Well/WellConnections.hpp>
+
+#include <cassert>
+#include <iterator>
+#include <numeric>
 
 namespace Dune
 {
@@ -231,6 +236,68 @@ int CommunicateAboveBelow::endReset()
     return num_local_perfs_;
 }
 
+template<class RAIterator>
+void CommunicateAboveBelow::partialSumPerfValues(RAIterator begin, RAIterator end) const
+{
+    if (this->comm_.size() < 2)
+    {
+        std::partial_sum(begin, end, begin);
+    }
+    else
+    {
+#if HAVE_MPI
+        // The global index used in the index set current_indices
+        // is the index of the perforation in ECL Schedule definition.
+        // This is assumed to give the topological order that is used
+        // when doing the partial sum.
+        // allgather the index of the perforation in ECL schedule and the value.
+        using Value = typename std::iterator_traits<RAIterator>::value_type;
+        std::vector<int> sizes(comm_.size());
+        std::vector<int> displ(comm_.size() + 1, 0);
+        using GlobalIndex = typename IndexSet::IndexPair::GlobalIndex;
+        using Pair = std::pair<GlobalIndex,Value>;
+        std::vector<Pair> my_pairs;
+        my_pairs.reserve(current_indices_.size());
+        for (const auto& pair: current_indices_)
+        {
+            if (pair.local().attribute() == owner)
+            {
+                my_pairs.emplace_back(pair.global(), begin[pair.local()]);
+            }
+        }
+        int mySize = my_pairs.size();
+        comm_.allgather(&mySize, 1, sizes.data());
+        std::partial_sum(sizes.begin(), sizes.end(), displ.begin()+1);
+        std::vector<Pair> global_pairs(displ.back());
+        comm_.allgatherv(my_pairs.data(), my_pairs.size(), global_pairs.data(), sizes.data(), displ.data());
+        // sort the complete range to get the correct ordering
+        std::sort(global_pairs.begin(), global_pairs.end(),
+                  [](const Pair& p1, const Pair& p2){ return p1.first < p2.first; } );
+        std::vector<Value> sums(global_pairs.size());
+        std::transform(global_pairs.begin(), global_pairs.end(), sums.begin(),
+                       [](const Pair& p) { return p.second; });
+        std::partial_sum(sums.begin(), sums.end(),sums.begin());
+        // assign the values (both ranges are sorted by the ecl index)
+        auto global_pair = global_pairs.begin();
+        for (const auto& pair: current_indices_)
+        {
+            global_pair = std::lower_bound(global_pair, global_pairs.end(),
+                                           pair.global(),
+                                           [](const Pair& val1, const GlobalIndex& val2)
+                                           { return val1.first < val2; });
+            assert(global_pair != global_pairs.end());
+            assert(global_pair->first == pair.global());
+            begin[pair.local()] = sums[global_pair - global_pairs.begin()];
+        }
+#else
+        OPM_THROW(std::logic_error, "In a sequential run the size of the communicator should be 1!");
+#endif
+    }
+}
+
+using dIter = typename std::vector<double>::iterator;
+template void CommunicateAboveBelow::partialSumPerfValues<dIter>(dIter begin, dIter end) const;
+
 struct CopyGatherScatter
 {
     static const double& gather(const double* a, std::size_t i)
@@ -408,12 +475,49 @@ void ParallelWellInfo::endReset()
                                               local_num_perfs));
 }
 
+template<typename It>
+typename It::value_type
+ParallelWellInfo::sumPerfValues(It begin, It end) const
+{
+    using V = typename It::value_type;
+    /// \todo cater for overlap later. Currently only owner
+    auto local = std::accumulate(begin, end, V());
+    return communication().sum(local);
+}
+
+using cdIter = typename std::vector<double>::const_iterator;
+template typename cdIter::value_type ParallelWellInfo::sumPerfValues<cdIter>(cdIter,cdIter) const;
+template typename dIter::value_type ParallelWellInfo::sumPerfValues<dIter>(dIter,dIter) const;
+
 void ParallelWellInfo::clear()
 {
     commAboveBelow_->clear();
     globalPerfCont_.reset();
 }
 
+
+template<class T>
+T ParallelWellInfo::broadcastFirstPerforationValue(const T& t) const
+{
+    T res = t;
+    if (rankWithFirstPerf_ >= 0) {
+#ifndef NDEBUG
+        assert(rankWithFirstPerf_ < comm_->size());
+        // At least on some OpenMPI version this might broadcast might interfere
+        // with other communication if there are bugs
+        comm_->barrier();
+#endif
+        comm_->broadcast(&res, 1, rankWithFirstPerf_);
+#ifndef NDEBUG
+        comm_->barrier();
+#endif
+    }
+    return res;
+}
+
+template int ParallelWellInfo::broadcastFirstPerforationValue<int>(const int&) const;
+template double ParallelWellInfo::broadcastFirstPerforationValue<double>(const double&) const;
+
 std::vector<double> ParallelWellInfo::communicateAboveValues(double zero_value,
                                                              const double* current_values,
                                                              std::size_t size) const
