@@ -25,7 +25,6 @@
 
 #include <opm/simulators/timestepping/ConvergenceReport.hpp>
 #include <opm/simulators/wells/RateConverter.hpp>
-#include <opm/simulators/wells/StandardWellGeneric.hpp>
 #include <opm/simulators/wells/VFPInjProperties.hpp>
 #include <opm/simulators/wells/VFPProdProperties.hpp>
 #include <opm/simulators/wells/WellInterface.hpp>
@@ -39,8 +38,7 @@
 #include <opm/models/blackoil/blackoilbrinemodules.hh>
 #include <opm/models/blackoil/blackoilmicpmodules.hh>
 
-#include <opm/material/densead/DynamicEvaluation.hpp>
-#include <opm/input/eclipse/EclipseState/Runspec.hpp>
+#include <opm/material/densead/Evaluation.hpp>
 #include <opm/input/eclipse/Schedule/ScheduleTypes.hpp>
 
 #include <opm/simulators/wells/StandardWellEval.hpp>
@@ -50,7 +48,6 @@
 
 #include <memory>
 #include <optional>
-#include <fmt/format.h>
 
 namespace Opm
 {
@@ -63,7 +60,7 @@ namespace Opm
     {
 
     public:
-        typedef WellInterface<TypeTag> Base;
+        using Base = WellInterface<TypeTag>;
         using StdWellEval = StandardWellEval<GetPropType<TypeTag, Properties::FluidSystem>,
                                              GetPropType<TypeTag, Properties::Indices>,
                                              GetPropType<TypeTag, Properties::Scalar>>;
@@ -109,6 +106,8 @@ namespace Opm
         // TODO: we should have indices for the well equations and well primary variables separately
         static constexpr int Bhp = numStaticWellEq - numWellControlEq;
 
+        using StdWellEval::WQTotal;
+
         using typename Base::Scalar;
 
 
@@ -122,7 +121,6 @@ namespace Opm
         using Eval = typename StdWellEval::Eval;
         using EvalWell = typename StdWellEval::EvalWell;
         using BVectorWell = typename StdWellEval::BVectorWell;
-        using DiagMatrixBlockWellType = typename StdWellEval::DiagMatrixBlockWellType;
 
         StandardWell(const Well& well,
                      const ParallelWellInfo& pw_info,
@@ -143,7 +141,7 @@ namespace Opm
                           const bool changed_to_open_this_step) override;
 
 
-        virtual void initPrimaryVariablesEvaluation() const override;
+        void initPrimaryVariablesEvaluation() override;
 
         /// check whether the well equations get converged for this well
         virtual ConvergenceReport getWellConvergence(const WellState& well_state,
@@ -158,9 +156,10 @@ namespace Opm
 
         /// using the solution x to recover the solution xw for wells and applying
         /// xw to update Well State
-        virtual void recoverWellSolutionAndUpdateWellState(const BVector& x,
-                                                           WellState& well_state,
-                                                           DeferredLogger& deferred_logger) const override;
+        void recoverWellSolutionAndUpdateWellState(const SummaryState& summary_state,
+                                                   const BVector& x,
+                                                   WellState& well_state,
+                                                   DeferredLogger& deferred_logger) override;
 
         /// computing the well potentials for group control
         virtual void computeWellPotentials(const Simulator& ebosSimulator,
@@ -168,9 +167,13 @@ namespace Opm
                                            std::vector<double>& well_potentials,
                                            DeferredLogger& deferred_logger) /* const */ override;
 
-        virtual void updatePrimaryVariables(const WellState& well_state, DeferredLogger& deferred_logger) const override;
+        void updatePrimaryVariables(const SummaryState& summary_state,
+                                    const WellState& well_state,
+                                    DeferredLogger& deferred_logger) override;
 
-        virtual void solveEqAndUpdateWellState(WellState& well_state, DeferredLogger& deferred_logger) override;
+        void solveEqAndUpdateWellState(const SummaryState& summary_state,
+                                       WellState& well_state,
+                                       DeferredLogger& deferred_logger);
 
         virtual void calculateExplicitQuantities(const Simulator& ebosSimulator,
                                                  const WellState& well_state,
@@ -218,11 +221,11 @@ namespace Opm
             std::vector<double> &potentials,
             double alq) const;
 
-        virtual std::optional<double> computeBhpAtThpLimitProdWithAlq(
+        std::optional<double> computeBhpAtThpLimitProdWithAlq(
             const Simulator& ebos_simulator,
             const SummaryState& summary_state,
-            DeferredLogger& deferred_logger,
-            double alq_value) const override;
+            const double alq_value,
+            DeferredLogger& deferred_logger) const override;
 
         virtual void computeWellRatesWithBhp(
             const Simulator& ebosSimulator,
@@ -253,13 +256,11 @@ namespace Opm
     protected:
         bool regularize_;
 
-        // xw = inv(D)*(rw - C*x)
-        void recoverSolutionWell(const BVector& x, BVectorWell& xw) const;
-
         // updating the well_state based on well solution dwells
-        void updateWellState(const BVectorWell& dwells,
+        void updateWellState(const SummaryState& summary_state,
+                             const BVectorWell& dwells,
                              WellState& well_state,
-                             DeferredLogger& deferred_logger) const;
+                             DeferredLogger& deferred_logger);
 
         // calculate the properties for the well connections
         // to calulate the pressure difference between well connections.
@@ -269,6 +270,7 @@ namespace Opm
                                                          std::vector<double>& rsmax_perf,
                                                          std::vector<double>& rvmax_perf,
                                                          std::vector<double>& rvwmax_perf,
+                                                         std::vector<double>& rswmax_perf,
                                                          std::vector<double>& surf_dens_perf) const;
 
         void computeWellConnectionDensitesPressures(const Simulator& ebosSimulator,
@@ -277,6 +279,7 @@ namespace Opm
                                                     const std::vector<double>& rsmax_perf,
                                                     const std::vector<double>& rvmax_perf,
                                                     const std::vector<double>& rvwmax_perf,
+                                                    const std::vector<double>& rswmax_perf,
                                                     const std::vector<double>& surf_dens_perf,
                                                     DeferredLogger& deferred_logger);
 
@@ -292,6 +295,7 @@ namespace Opm
                                  const bool allow_cf,
                                  std::vector<EvalWell>& cq_s,
                                  double& perf_dis_gas_rate,
+                                 double& perf_dis_gas_rate_in_water,
                                  double& perf_vap_oil_rate,
                                  double& perf_vap_wat_rate,
                                  DeferredLogger& deferred_logger) const;
@@ -312,6 +316,7 @@ namespace Opm
                              const Value& rs,
                              const Value& rv,
                              const Value& rvw,
+                             const Value& rsw,
                              std::vector<Value>& b_perfcells_dense,
                              const double Tw,
                              const int perf,
@@ -320,6 +325,7 @@ namespace Opm
                              const std::vector<Value>& cmix_s,
                              std::vector<Value>& cq_s,
                              double& perf_dis_gas_rate,
+                             double& perf_dis_gas_rate_in_water,
                              double& perf_vap_oil_rate,
                              double& perf_vap_wat_rate,
                              DeferredLogger& deferred_logger) const;
@@ -334,6 +340,9 @@ namespace Opm
             DeferredLogger& deferred_logger,
             const WellState &well_state) const;
 
+        bool updateWellStateWithTHPTargetProd(const Simulator& ebos_simulator,
+                                              WellState& well_state,
+                                              DeferredLogger& deferred_logger) const override;
 
         virtual double getRefDensity() const override;
 
@@ -356,14 +365,12 @@ namespace Opm
                                             DeferredLogger& deferred_logger) const;
 
         void updatePrimaryVariablesNewton(const BVectorWell& dwells,
-                                          const WellState& well_state,
-                                          DeferredLogger& deferred_logger) const;
+                                          const bool stop_or_zero_rate_target,
+                                          DeferredLogger& deferred_logger);
 
-        // update extra primary vriables if there are any
-        void updateExtraPrimaryVariables(const BVectorWell& dwells) const;
-
-
-        void updateWellStateFromPrimaryVariables(WellState& well_state, DeferredLogger& deferred_logger) const;
+        void updateWellStateFromPrimaryVariables(const bool stop_or_zero_rate_target,
+                                                 WellState& well_state,
+                                                 DeferredLogger& deferred_logger) const;
 
         virtual void assembleWellEqWithoutIteration(const Simulator& ebosSimulator,
                                                     const double dt,
@@ -375,6 +382,8 @@ namespace Opm
 
         void assembleWellEqWithoutIterationImpl(const Simulator& ebosSimulator,
                                                 const double dt,
+                                                const Well::InjectionControls& inj_controls,
+                                                const Well::ProductionControls& prod_controls,
                                                 WellState& well_state,
                                                 const GroupState& group_state,
                                                 DeferredLogger& deferred_logger);
