@@ -26,6 +26,8 @@
 #include <opm/input/eclipse/Schedule/Group/GConSale.hpp>
 #include <opm/input/eclipse/Schedule/Group/GPMaint.hpp>
 #include <opm/input/eclipse/Schedule/Group/Group.hpp>
+#include <opm/input/eclipse/Schedule/Network/ExtNetwork.hpp>
+#include <opm/input/eclipse/Schedule/Well/Well.hpp>
 
 #include <opm/material/fluidsystems/BlackOilFluidSystem.hpp>
 
@@ -349,32 +351,47 @@ namespace WellGroupHelpers
             // accumulate group contribution from sub group
             if (isInjector) {
                 const Phase all[] = {Phase::WATER, Phase::OIL, Phase::GAS};
-                bool individual_control = false;
-                int num_group_controlled_wells = 0;
                 for (Phase phase : all) {
-                    const Group::InjectionCMode& currentGroupControl
-                            = group_state.injection_control(subGroup.name(), phase);
-                    individual_control = individual_control || (currentGroupControl != Group::InjectionCMode::FLD
-                            && currentGroupControl != Group::InjectionCMode::NONE);
-                    num_group_controlled_wells
-                            += groupControlledWells(schedule, wellState, group_state, reportStepIdx, subGroupName, "", !isInjector, phase);
-                }
-                if (individual_control || num_group_controlled_wells == 0) {
-                    for (int phase = 0; phase < np; phase++) {
-                        groupTargetReduction[phase]
-                            += subGroupEfficiency * sumWellSurfaceRates(subGroup, schedule, wellState, reportStepIdx, phase, isInjector);
-                    }
-                } else {
-                    // The subgroup may participate in group control.
-                    bool has_guide_rate = false;
-                    for (Phase phase : all) {
-                        has_guide_rate = has_guide_rate || guide_rate.has(subGroupName, phase);
+
+                    int phase_pos = -1;
+                    switch (phase) {
+                    case Phase::GAS:
+                        if (pu.phase_used[BlackoilPhases::Vapour]) {
+                            phase_pos = pu.phase_pos[BlackoilPhases::Vapour];
+                        }
+                        break;
+                    case Phase::OIL:
+                        if (pu.phase_used[BlackoilPhases::Liquid]) {
+                            phase_pos = pu.phase_pos[BlackoilPhases::Liquid];
+                        }
+                        break;
+                    case Phase::WATER:
+                        if (pu.phase_used[BlackoilPhases::Aqua]) {
+                            phase_pos = pu.phase_pos[BlackoilPhases::Aqua];
+                        }
+                        break;
+                    default:
+                        // just to avoid warning
+                        throw std::invalid_argument("unhandled phase enum");
                     }
 
-                    if (!has_guide_rate) {
+                    // the phase is not present
+                    if (phase_pos == -1)
+                        continue;
+
+                    const Group::InjectionCMode& currentGroupControl
+                            = group_state.injection_control(subGroup.name(), phase);
+                    const bool individual_control = (currentGroupControl != Group::InjectionCMode::FLD
+                            && currentGroupControl != Group::InjectionCMode::NONE);
+                    const int num_group_controlled_wells
+                            = groupControlledWells(schedule, wellState, group_state, reportStepIdx, subGroupName, "", !isInjector, phase);
+                    if (individual_control || num_group_controlled_wells == 0) {
+                        groupTargetReduction[phase_pos]
+                            += subGroupEfficiency * sumWellSurfaceRates(subGroup, schedule, wellState, reportStepIdx, phase_pos, isInjector);
+                    } else {
                         // Accumulate from this subgroup only if no group guide rate is set for it.
-                        for (int phase = 0; phase < np; phase++) {
-                            groupTargetReduction[phase] += subGroupEfficiency * subGroupTargetReduction[phase];
+                        if (!guide_rate.has(subGroupName, phase)) {
+                            groupTargetReduction[phase_pos] += subGroupEfficiency * subGroupTargetReduction[phase_pos];
                         }
                     }
                 }
@@ -682,8 +699,12 @@ namespace WellGroupHelpers
         if (!gpm)
             return;
 
-        const auto [name, number] = *gpm->region();
-        const double error = gpm->pressure_target() - regional_values.pressure(number);
+        const auto& region = gpm->region();
+        if (!region)
+            return;
+
+        const auto [name, number] = *region;
+        const double error = gpm->pressure_target() - regional_values.at(name)->pressure(number);
         double current_rate = 0.0;
         const auto& pu = well_state.phaseUsage();
         double sign = 1.0;
@@ -738,7 +759,6 @@ namespace WellGroupHelpers
             default:
                 throw std::invalid_argument("Invalid Flow target type in GPMAINT");
         }
-
         auto& gpmaint_state = group_state.gpmaint(group.name());
         double rate = gpm->rate(gpmaint_state, current_rate, error, dt);
         group_state.update_gpmaint_target(group.name(), std::max(0.0, sign * rate));
@@ -903,7 +923,11 @@ namespace WellGroupHelpers
                         const PhaseUsage& pu)
     {
         if (schedule.hasWell(name, reportStepIdx)) {
-            return guideRate->get(name, target, getWellRateVector(wellState, pu, name));
+            if (guideRate->has(name) || guideRate->hasPotentials(name)) {
+                return guideRate->get(name, target, getWellRateVector(wellState, pu, name));
+            } else {
+                return 0.0;
+            }
         }
 
         if (guideRate->has(name)) {
@@ -935,7 +959,9 @@ namespace WellGroupHelpers
             if (!wellState.isProductionGrup(wellName))
                 continue;
 
-            totalGuideRate += guideRate->get(wellName, target, getWellRateVector(wellState, pu, wellName));
+            totalGuideRate += getGuideRate(wellName, schedule, wellState, group_state,
+                                           reportStepIdx, guideRate, target, pu);
+
         }
         return totalGuideRate;
     }
@@ -952,7 +978,8 @@ namespace WellGroupHelpers
                            const PhaseUsage& pu)
     {
         if (schedule.hasWell(name, reportStepIdx)) {
-            return guideRate->get(name, target, getWellRateVector(wellState, pu, name));
+            return getGuideRate(name, schedule, wellState, group_state,
+                                reportStepIdx, guideRate, target, pu);
         }
 
         if (guideRate->has(name, injectionPhase)) {
@@ -1122,7 +1149,8 @@ namespace WellGroupHelpers
     double FractionCalculator::guideRate(const std::string& name, const std::string& always_included_child)
     {
         if (schedule_.hasWell(name, report_step_)) {
-            return guide_rate_->get(name, target_, getWellRateVector(well_state_, pu_, name));
+            return getGuideRate(name, schedule_, well_state_, group_state_,
+                                report_step_, guide_rate_, target_, pu_);
         } else {
             if (groupControlledWells(name, always_included_child) > 0) {
                 if (is_producer_ && guide_rate_->has(name)) {
@@ -1262,7 +1290,12 @@ namespace WellGroupHelpers
             const std::vector<double>& groupSurfaceRates = group_state.production_rates(group_name);
             return tcalc.calcModeRateFromRates(groupSurfaceRates);
         };
-        const double orig_target = tcalc.groupTarget(group.productionControls(summaryState));
+
+        std::optional<Group::ProductionControls> ctrl;
+        if (!group.has_gpmaint_control(currentGroupControl))
+            ctrl = group.productionControls(summaryState);
+
+        const double orig_target = tcalc.groupTarget(ctrl, deferred_logger);
         // Assume we have a chain of groups as follows: BOTTOM -> MIDDLE -> TOP.
         // Then ...
         // TODO finish explanation.
@@ -1404,7 +1437,12 @@ namespace WellGroupHelpers
             return tcalc.calcModeRateFromRates(groupSurfaceRates);
         };
 
-        const double orig_target = tcalc.groupTarget(group.injectionControls(injectionPhase, summaryState), deferred_logger);
+        std::optional<Group::InjectionControls> ctrl;
+        if (!group.has_gpmaint_control(injectionPhase, currentGroupControl))
+            ctrl = group.injectionControls(injectionPhase, summaryState);
+
+
+        const double orig_target = tcalc.groupTarget(ctrl, deferred_logger);
         // Assume we have a chain of groups as follows: BOTTOM -> MIDDLE -> TOP.
         // Then ...
         // TODO finish explanation.
@@ -1471,6 +1509,32 @@ namespace WellGroupHelpers
         return std::make_pair(current_rate > target_rate, scale);
     }
 
+    template <class AverageRegionalPressureType>
+    void setRegionAveragePressureCalculator(const Group& group,
+                                            const Schedule& schedule,
+                                            const int reportStepIdx,
+                                            const FieldPropsManager& fp,
+                                            const PhaseUsage& pu,
+                                            std::map<std::string, std::unique_ptr<AverageRegionalPressureType>>& regionalAveragePressureCalculator) 
+    {
+        for (const std::string& groupName : group.groups()) {
+            setRegionAveragePressureCalculator( schedule.getGroup(groupName, reportStepIdx), schedule, 
+                                                reportStepIdx, fp, pu, regionalAveragePressureCalculator);
+        }
+        const auto& gpm = group.gpmaint();
+        if (!gpm)
+            return;
+
+        const auto& reg = gpm->region();
+        if (!reg)
+            return;
+
+        if (regionalAveragePressureCalculator.count(reg->first) == 0) {
+            const std::string name = (reg->first.rfind("FIP", 0) == 0) ? reg->first : "FIP" + reg->first;
+            const auto& fipnum = fp.get_int(name);
+            regionalAveragePressureCalculator[reg->first] = std::make_unique<AverageRegionalPressureType>(pu,fipnum);
+        }
+    }
 
     template <class Comm>
     void updateGuideRates(const Group& group,
@@ -1609,13 +1673,20 @@ namespace WellGroupHelpers
     }
 
  using AvgP = RegionAverageCalculator::AverageRegionalPressure<BlackOilFluidSystem<double>,std::vector<int>>;
- template void WellGroupHelpers::updateGpMaintTargetForGroups<AvgP>(const Group&,
+ using AvgPMap = std::map<std::string, std::unique_ptr<AvgP>>;
+ template void WellGroupHelpers::updateGpMaintTargetForGroups<AvgPMap>(const Group&,
                                                                     const Schedule&,
-                                                                    const AvgP&,
+                                                                    const AvgPMap&,
                                                                     int,
                                                                     double,
                                                                     const WellState&,
                                                                     GroupState&);
+ template void WellGroupHelpers::setRegionAveragePressureCalculator<AvgP>(const Group&,
+                                            const Schedule&,
+                                            const int,
+                                            const FieldPropsManager&,
+                                            const PhaseUsage&,
+                                            AvgPMap&);
 
 template
 void updateGuideRateForProductionGroups<Parallel::Communication>(const Group& group,
