@@ -28,7 +28,6 @@
 #include <opm/common/ErrorMacros.hpp>
 #include <opm/common/Exceptions.hpp>
 
-#include <opm/input/eclipse/Schedule/Well/Well.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellTestState.hpp>
 
 #include <opm/core/props/BlackoilPhases.hpp>
@@ -65,13 +64,15 @@ namespace Opm {
 namespace Opm
 {
 
+class WellInjectionProperties;
+class WellProductionProperties;
+
 template<typename TypeTag>
 class WellInterface : public WellInterfaceIndices<GetPropType<TypeTag, Properties::FluidSystem>,
                                                   GetPropType<TypeTag, Properties::Indices>,
                                                   GetPropType<TypeTag, Properties::Scalar>>
 {
 public:
-
     using ModelParameters = BlackoilModelParametersEbos<TypeTag>;
 
     using Grid = GetPropType<TypeTag, Properties::Grid>;
@@ -103,7 +104,6 @@ public:
     using WellInterfaceFluidSystem<FluidSystem>::Gas;
     using WellInterfaceFluidSystem<FluidSystem>::Oil;
     using WellInterfaceFluidSystem<FluidSystem>::Water;
-    using RatioLimitCheckReport = typename WellInterfaceFluidSystem<FluidSystem>::RatioLimitCheckReport;
 
     static constexpr bool has_solvent = getPropValue<TypeTag, Properties::EnableSolvent>();
     static constexpr bool has_zFraction = getPropValue<TypeTag, Properties::EnableExtbo>();
@@ -115,6 +115,7 @@ public:
     static constexpr bool has_foam = getPropValue<TypeTag, Properties::EnableFoam>();
     static constexpr bool has_brine = getPropValue<TypeTag, Properties::EnableBrine>();
     static constexpr bool has_watVapor = getPropValue<TypeTag, Properties::EnableEvaporation>();
+    static constexpr bool has_disgas_in_water = getPropValue<TypeTag, Properties::EnableDisgasInWater>();
     static constexpr bool has_saltPrecip = getPropValue<TypeTag, Properties::EnableSaltPrecipitation>();
     static constexpr bool has_micp = getPropValue<TypeTag, Properties::EnableMICP>();
 
@@ -127,6 +128,7 @@ public:
                                           has_watVapor,
                                           has_brine,
                                           has_saltPrecip,
+                                          has_disgas_in_water,
                                           Indices::numPhases >;
     /// Constructor
     WellInterface(const Well& well,
@@ -150,12 +152,10 @@ public:
                       const std::vector< Scalar >& B_avg,
                       const bool changed_to_open_this_step);
 
-    virtual void initPrimaryVariablesEvaluation() const = 0;
+    virtual void initPrimaryVariablesEvaluation() = 0;
 
     virtual ConvergenceReport getWellConvergence(const WellState& well_state, const std::vector<double>& B_avg, DeferredLogger& deferred_logger, const bool relax_tolerance) const = 0;
 
-    virtual void solveEqAndUpdateWellState(WellState& well_state, DeferredLogger& deferred_logger) = 0;
-
     void assembleWellEq(const Simulator& ebosSimulator,
                         const double dt,
                         WellState& well_state,
@@ -172,15 +172,16 @@ public:
     virtual std::optional<double> computeBhpAtThpLimitProdWithAlq(
         const Simulator& ebos_simulator,
         const SummaryState& summary_state,
-        DeferredLogger& deferred_logger,
-        double alq_value
+        const double alq_value,
+        DeferredLogger& deferred_logger
     ) const = 0;
 
     /// using the solution x to recover the solution xw for wells and applying
     /// xw to update Well State
-    virtual void recoverWellSolutionAndUpdateWellState(const BVector& x,
+    virtual void recoverWellSolutionAndUpdateWellState(const SummaryState& summary_state,
+                                                       const BVector& x,
                                                        WellState& well_state,
-                                                       DeferredLogger& deferred_logger) const = 0;
+                                                       DeferredLogger& deferred_logger) = 0;
 
     /// Ax = Ax - C D^-1 B x
     virtual void apply(const BVector& x, BVector& Ax) const = 0;
@@ -199,6 +200,10 @@ public:
                                            WellState& well_state,
                                            DeferredLogger& deferred_logger) const;
 
+    virtual bool updateWellStateWithTHPTargetProd(const Simulator& ebos_simulator,
+                                                  WellState& well_state,
+                                                  DeferredLogger& deferred_logger) const = 0;
+
     enum class IndividualOrGroup { Individual, Group, Both };
     bool updateWellControl(const Simulator& ebos_simulator,
                            const IndividualOrGroup iog,
@@ -206,7 +211,9 @@ public:
                            const GroupState& group_state,
                            DeferredLogger& deferred_logger) /* const */;
 
-    virtual void updatePrimaryVariables(const WellState& well_state, DeferredLogger& deferred_logger) const = 0;
+    virtual void updatePrimaryVariables(const SummaryState& summary_state,
+                                        const WellState& well_state,
+                                        DeferredLogger& deferred_logger) = 0;
 
     virtual void calculateExplicitQuantities(const Simulator& ebosSimulator,
                                              const WellState& well_state,
@@ -226,8 +233,6 @@ public:
     // Add well contributions to matrix
     virtual void addWellContributions(SparseMatrixAdapter&) const = 0;
 
-    virtual bool isPressureControlled(const WellState& well_state) const;
-    
     virtual void addWellPressureEquations(PressureMatrix& mat,
                                           const BVector& x,
                                           const int pressureVarIndex,
@@ -289,6 +294,11 @@ public:
                            const GroupState& group_state,
                            DeferredLogger& deferred_logger);
 
+    const std::vector<RateVector>& connectionRates() const
+    {
+        return connectionRates_;
+    }
+
 protected:
 
     // simulation parameters
@@ -329,8 +339,8 @@ protected:
 
     virtual void assembleWellEqWithoutIteration(const Simulator& ebosSimulator,
                                                 const double dt,
-                                                const Well::InjectionControls& inj_controls,
-                                                const Well::ProductionControls& prod_controls,
+                                                const WellInjectionControls& inj_controls,
+                                                const WellProductionControls& prod_controls,
                                                 WellState& well_state,
                                                 const GroupState& group_state,
                                                 DeferredLogger& deferred_logger) = 0;
@@ -338,8 +348,8 @@ protected:
     // iterate well equations with the specified control until converged
     virtual bool iterateWellEqWithControl(const Simulator& ebosSimulator,
                                           const double dt,
-                                          const Well::InjectionControls& inj_controls,
-                                          const Well::ProductionControls& prod_controls,
+                                          const WellInjectionControls& inj_controls,
+                                          const WellProductionControls& prod_controls,
                                           WellState& well_state,
                                           const GroupState& group_state,
                                           DeferredLogger& deferred_logger) = 0;
@@ -355,7 +365,6 @@ protected:
 
     Eval getPerfCellPressure(const FluidState& fs) const;
 
-
 };
 
 }
