@@ -50,9 +50,6 @@ class Schedule;
 
 class WellInterfaceGeneric {
 public:
-
-    static constexpr bool extraBhpAtThpLimitProdOutput = false;
-
     WellInterfaceGeneric(const Well& well,
                          const ParallelWellInfo& parallel_well_info,
                          const int time_step,
@@ -172,6 +169,10 @@ public:
         return well_index_;
     }
 
+    const std::map<int,std::vector<int>>& getCompletions() const {
+        return completions_;
+    }
+
     double getTHPConstraint(const SummaryState& summaryState) const;
     double getALQ(const WellState& well_state) const;
     double wsolvent() const;
@@ -185,48 +186,33 @@ public:
     bool changedToOpenThisStep() const {
         return this->changed_to_open_this_step_;
     }
-    std::optional<double> computeBhpAtThpLimitProdCommon(const std::function<std::vector<double>(const double)>& frates,
-                                                         const SummaryState& summary_state,
-                                                         const double maxPerfPress,
-                                                         const double rho,
-                                                         const double alq_value,
-                                                         DeferredLogger& deferred_logger
-                                                         ) const;
 
+    void updateWellTestState(const SingleWellState& ws,
+                             const double& simulationTime,
+                             const bool& writeMessageToOPMLog,
+                             WellTestState& wellTestState,
+                             DeferredLogger& deferred_logger) const;
 
+    bool isPressureControlled(const WellState& well_state) const;
 
+    bool stopppedOrZeroRateTarget(const SummaryState& summary_state,
+                                  const WellState& well_state) const;
 protected:
     bool getAllowCrossFlow() const;
-    double mostStrictBhpFromBhpLimits(const SummaryState& summaryState) const;
-    void updateWellTestStatePhysical(const double simulation_time,
-                                     const bool write_message_to_opmlog,
-                                     WellTestState& well_test_state,
-                                     DeferredLogger& deferred_logger) const;
 
-    std::optional<double> bhpMax(const std::function<double(const double)>& fflo,
-                                 const double bhp_limit,
-                                 const double maxPerfPress,
-                                 const double vfp_flo_front,
-                                 DeferredLogger& deferred_logger) const;
+    double wmicrobes_() const;
+    double wfoam_() const;
+    double woxygen_() const;
+    double wpolymer_() const;
+    double wsalt_() const;
+    double wurea_() const;
 
-    std::optional<double> computeBhpAtThpLimitCommon(
-                           const std::function<std::vector<double>(const double)>& frates,
-                           const std::function<double(const std::vector<double>)>& fbhp,
-                           const std::array<double, 2>& range,
-                           DeferredLogger& deferred_logger) const;
-
-
-    bool bruteForceBracket(const std::function<double(const double)>& eq,
-                           const std::array<double, 2>& range,
-                           double& low, double& high,
-                           DeferredLogger& deferred_logger) const;
-
-    bool bisectBracket(const std::function<double(const double)>& eq,
-                       const std::array<double, 2>& range,
-                       double& low, double& high,
-                       std::optional<double>& approximate_solution,
-                       DeferredLogger& deferred_logger) const;
+    int polymerTable_() const;
+    int polymerInjTable_() const;
+    int polymerWaterTable_() const;
 
+    bool wellUnderZeroRateTarget(const SummaryState& summary_state,
+                                 const WellState& well_state) const;
 
     // definition of the struct OperabilityStatus
     struct OperabilityStatus {
