@@ -19,13 +19,19 @@
   along with OPM.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include <opm/common/Exceptions.hpp>
+
 #include <opm/input/eclipse/Schedule/ScheduleTypes.hpp>
 #include <opm/simulators/utils/DeferredLoggingErrorHelpers.hpp>
 #include <opm/simulators/wells/GroupState.hpp>
 #include <opm/simulators/wells/TargetCalculator.hpp>
+#include <opm/simulators/wells/WellBhpThpCalculator.hpp>
+#include <opm/simulators/wells/WellHelpers.hpp>
 
 #include <dune/common/version.hh>
 
+#include <fmt/format.h>
+
 namespace Opm
 {
 
@@ -91,16 +97,7 @@ namespace Opm
     wpolymer() const
     {
         if constexpr (has_polymer) {
-            auto injectorType = this->well_ecl_.injectorType();
-
-            if (injectorType == InjectorType::WATER) {
-                WellPolymerProperties polymer = this->well_ecl_.getPolymerProperties();
-                const double polymer_injection_concentration = polymer.m_polymerConcentration;
-                return polymer_injection_concentration;
-            } else {
-                // Not a water injection well => no polymer.
-                return 0.0;
-            }
+            return this->wpolymer_();
         }
 
         return 0.0;
@@ -116,15 +113,7 @@ namespace Opm
     wfoam() const
     {
         if constexpr (has_foam) {
-            auto injectorType = this->well_ecl_.injectorType();
-
-            if (injectorType == InjectorType::GAS) {
-                WellFoamProperties fprop = this->well_ecl_.getFoamProperties();
-                return fprop.m_foamConcentration;
-            } else {
-                // Not a gas injection well => no foam.
-                return 0.0;
-            }
+            return this->wfoam_();
         }
 
         return 0.0;
@@ -138,15 +127,7 @@ namespace Opm
     wsalt() const
     {
         if constexpr (has_brine) {
-            auto injectorType = this->well_ecl_.injectorType();
-
-            if (injectorType == InjectorType::WATER) {
-                WellBrineProperties fprop = this->well_ecl_.getBrineProperties();
-                return fprop.m_saltConcentration;
-            } else {
-                // Not a water injection well => no salt (?).
-                return 0.0;
-            }
+            return this->wsalt_();
         }
 
         return 0.0;
@@ -158,16 +139,7 @@ namespace Opm
     wmicrobes() const
     {
       if constexpr (has_micp) {
-          auto injectorType = this->well_ecl_.injectorType();
-
-          if (injectorType == InjectorType::WATER) {
-              WellMICPProperties microbes = this->well_ecl_.getMICPProperties();
-              const double microbial_injection_concentration = microbes.m_microbialConcentration;
-              return microbial_injection_concentration;
-          } else {
-              // Not a water injection well => no microbes.
-              return 0.0;
-          }
+          return this->wmicrobes_();
       }
 
       return 0.0;
@@ -179,16 +151,7 @@ namespace Opm
     woxygen() const
     {
       if constexpr (has_micp) {
-          auto injectorType = this->well_ecl_.injectorType();
-
-          if (injectorType == InjectorType::WATER) {
-              WellMICPProperties oxygen = this->well_ecl_.getMICPProperties();
-              const double oxygen_injection_concentration = oxygen.m_oxygenConcentration;
-              return oxygen_injection_concentration;
-          } else {
-              // Not a water injection well => no oxygen.
-              return 0.0;
-          }
+          return this->woxygen_();
       }
 
       return 0.0;
@@ -206,16 +169,7 @@ namespace Opm
     wurea() const
     {
       if constexpr (has_micp) {
-          auto injectorType = this->well_ecl_.injectorType();
-
-          if (injectorType == InjectorType::WATER) {
-              WellMICPProperties urea = this->well_ecl_.getMICPProperties();
-              const double urea_injection_concentration = urea.m_ureaConcentration / 10.; //Dividing by scaling factor 10
-              return urea_injection_concentration;
-          } else {
-              // Not a water injection well => no urea.
-              return 0.0;
-          }
+          return this->wurea_();
       }
 
       return 0.0;
@@ -230,7 +184,8 @@ namespace Opm
                       const GroupState& group_state,
                       DeferredLogger& deferred_logger) /* const */
     {
-        if (this->wellIsStopped()) {
+        const auto& summary_state = ebos_simulator.vanguard().summaryState();
+        if (this->stopppedOrZeroRateTarget(summary_state, well_state)) {
             return false;
         }
 
@@ -240,9 +195,9 @@ namespace Opm
         auto& ws = well_state.well(this->index_of_well_);
         std::string from;
         if (well.isInjector()) {
-            from = Well::InjectorCMode2String(ws.injection_cmode);
+            from = WellInjectorCMode2String(ws.injection_cmode);
         } else {
-            from = Well::ProducerCMode2String(ws.production_cmode);
+            from = WellProducerCMode2String(ws.production_cmode);
         }
         bool oscillating = std::count(this->well_control_log_.begin(), this->well_control_log_.end(), from) >= param_.max_number_of_well_switches_;
 
@@ -276,9 +231,9 @@ namespace Opm
         if (changed) {
             std::string to;
             if (well.isInjector()) {
-                to = Well::InjectorCMode2String(ws.injection_cmode);
+                to = WellInjectorCMode2String(ws.injection_cmode);
             } else {
-                to = Well::ProducerCMode2String(ws.production_cmode);
+                to = WellProducerCMode2String(ws.production_cmode);
             }
             std::ostringstream ss;
             ss << "    Switching control mode for well " << this->name()
@@ -291,7 +246,7 @@ namespace Opm
 
             this->well_control_log_.push_back(from);
             updateWellStateWithTarget(ebos_simulator, group_state, well_state, deferred_logger);
-            updatePrimaryVariables(well_state, deferred_logger);
+            updatePrimaryVariables(summaryState, well_state, deferred_logger);
         }
 
         return changed;
@@ -316,7 +271,8 @@ namespace Opm
 
         updateWellStateWithTarget(simulator, group_state, well_state_copy, deferred_logger);
         calculateExplicitQuantities(simulator, well_state_copy, deferred_logger);
-        updatePrimaryVariables(well_state_copy, deferred_logger);
+        const auto& summary_state = simulator.vanguard().summaryState();
+        updatePrimaryVariables(summary_state, well_state_copy, deferred_logger);
         initPrimaryVariablesEvaluation();
 
         if (this->isProducer()) {
@@ -345,7 +301,6 @@ namespace Opm
                 deferred_logger.debug(msg);
                 return;
             }
-
             std::vector<double> potentials;
             try {
                 computeWellPotentials(simulator, well_state_copy, potentials, deferred_logger);
@@ -407,7 +362,7 @@ namespace Opm
         bool converged = false;
         try {
             converged = this->iterateWellEqWithControl(ebosSimulator, dt, inj_controls, prod_controls, well_state, group_state, deferred_logger);
-        } catch (NumericalIssue& e ) {
+        } catch (NumericalProblem& e ) {
             const std::string msg = "Inner well iterations failed for well " + this->name() + " Treat the well as unconverged. ";
             deferred_logger.warning("INNER_ITERATION_FAILED", msg);
             converged = false;
@@ -475,13 +430,13 @@ namespace Opm
                 thp_control = ws.injection_cmode == Well::InjectorCMode::THP;
                 if (thp_control) {
                     ws.injection_cmode = Well::InjectorCMode::BHP;
-                    this->well_control_log_.push_back(Well::InjectorCMode2String(Well::InjectorCMode::THP));
+                    this->well_control_log_.push_back(WellInjectorCMode2String(Well::InjectorCMode::THP));
                 }
             } else {
                 thp_control = ws.production_cmode == Well::ProducerCMode::THP;
                 if (thp_control) {
                     ws.production_cmode = Well::ProducerCMode::BHP;
-                    this->well_control_log_.push_back(Well::ProducerCMode2String(Well::ProducerCMode::THP));
+                    this->well_control_log_.push_back(WellProducerCMode2String(Well::ProducerCMode::THP));
                 }
             }
             if (thp_control) {
@@ -570,34 +525,6 @@ namespace Opm
         assembleWellEqWithoutIteration(ebosSimulator, dt, inj_controls, prod_controls, well_state, group_state, deferred_logger);
     }
 
-    template<typename TypeTag>
-    bool
-    WellInterface<TypeTag>::isPressureControlled(const WellState& well_state) const
-    {
-         bool thp_controlled_well = false;
-        bool bhp_controlled_well = false;
-        const auto& ws = well_state.well(this->index_of_well_);
-        if (this->isInjector()) {
-            const Well::InjectorCMode& current = ws.injection_cmode;
-            if (current == Well::InjectorCMode::THP) {
-                thp_controlled_well = true;
-            }
-            if (current == Well::InjectorCMode::BHP) {
-                bhp_controlled_well = true;
-            }
-        } else {
-            const Well::ProducerCMode& current = ws.production_cmode;
-            if (current == Well::ProducerCMode::THP) {
-                thp_controlled_well = true;
-            }
-            if (current == Well::ProducerCMode::BHP) {
-                bhp_controlled_well = true;
-            }
-        }
-        bool ispressureControlled =  (bhp_controlled_well || thp_controlled_well);
-        return ispressureControlled;
-    }
-
     template<typename TypeTag>
     void
     WellInterface<TypeTag>::addCellRates(RateVector& rates, int cellIdx) const
@@ -811,7 +738,12 @@ namespace Opm
             case Well::InjectorCMode::THP:
             {
                 auto rates = ws.surface_rates;
-                double bhp = this->calculateBhpFromThp(well_state, rates, well, summaryState, this->getRefDensity(), deferred_logger);
+                double bhp = WellBhpThpCalculator(*this).calculateBhpFromThp(well_state,
+                                                                             rates,
+                                                                             well,
+                                                                             summaryState,
+                                                                             this->getRefDensity(),
+                                                                             deferred_logger);
                 ws.bhp = bhp;
                 ws.thp = this->getTHPConstraint(summaryState);
 
@@ -962,12 +894,14 @@ namespace Opm
             }
             case Well::ProducerCMode::CRAT:
             {
-                OPM_DEFLOG_THROW(std::runtime_error, "CRAT control not supported " << this->name(), deferred_logger);
+                OPM_DEFLOG_THROW(std::runtime_error,
+                                 fmt::format("CRAT control not supported, well {}", this->name()),
+                                 deferred_logger);
             }
             case Well::ProducerCMode::RESV:
             {
                 std::vector<double> convert_coeff(this->number_of_phases_, 1.0);
-                this->rateConverter_.calcCoeff(/*fipreg*/ 0, this->pvtRegionIdx_, convert_coeff);
+                this->rateConverter_.calcCoeff(/*fipreg*/ 0, this->pvtRegionIdx_, ws.surface_rates, convert_coeff);
                 double total_res_rate = 0.0;
                 for (int p = 0; p<np; ++p) {
                     total_res_rate -= ws.surface_rates[p] * convert_coeff[p];
@@ -1034,19 +968,26 @@ namespace Opm
             }
             case Well::ProducerCMode::THP:
             {
-                auto rates = ws.surface_rates;
-                this->adaptRatesForVFP(rates);
-                double bhp = this->calculateBhpFromThp(well_state, rates, well, summaryState, this->getRefDensity(), deferred_logger);
-                ws.bhp = bhp;
-                ws.thp = this->getTHPConstraint(summaryState);
+                const bool update_success = updateWellStateWithTHPTargetProd(ebos_simulator, well_state, deferred_logger);
 
-                // if the total rates are negative or zero
-                // we try to provide a better intial well rate
-                // using the well potentials
-                double total_rate = -std::accumulate(rates.begin(), rates.end(), 0.0);
-                if (total_rate <= 0.0){
-                    for (int p = 0; p<np; ++p) {
-                        ws.surface_rates[p] = -ws.well_potentials[p];
+                if (!update_success) {
+                    // the following is the original way of initializing well state with THP constraint
+                    // keeping it for robust reason in case that it fails to get a bhp value with THP constraint
+                    // more sophisticated design might be needed in the future
+                    auto rates = ws.surface_rates;
+                    this->adaptRatesForVFP(rates);
+                    const double bhp = WellBhpThpCalculator(*this).calculateBhpFromThp(
+                        well_state, rates, well, summaryState, this->getRefDensity(), deferred_logger);
+                    ws.bhp = bhp;
+                    ws.thp = this->getTHPConstraint(summaryState);
+                    // if the total rates are negative or zero
+                    // we try to provide a better initial well rate
+                    // using the well potentials
+                    const double total_rate = -std::accumulate(rates.begin(), rates.end(), 0.0);
+                    if (total_rate <= 0.0) {
+                        for (int p = 0; p < this->number_of_phases_; ++p) {
+                            ws.surface_rates[p] = -ws.well_potentials[p];
+                        }
                     }
                 }
                 break;
@@ -1061,7 +1002,8 @@ namespace Opm
                                                           group_state,
                                                           schedule,
                                                           summaryState,
-                                                          efficiencyFactor);
+                                                          efficiencyFactor,
+                                                          deferred_logger);
 
                 // we don't want to scale with zero and get zero rates.
                 if (scale > 0) {
@@ -1113,7 +1055,7 @@ namespace Opm
         }
         for (int perf = 0; perf < nperf; ++perf) {
             const int cell_idx = this->well_cells_[perf];
-            const auto& intQuants = *(ebosSimulator.model().cachedIntensiveQuantities(cell_idx, /*timeIdx=*/0));
+            const auto& intQuants = ebosSimulator.model().intensiveQuantities(cell_idx, /*timeIdx=*/0);
             const auto& fs = intQuants.fluidState();
             const double well_tw_fraction = this->well_index_[perf] / total_tw;
             double total_mobility = 0.0;
@@ -1181,16 +1123,13 @@ namespace Opm
     typename WellInterface<TypeTag>::Eval
     WellInterface<TypeTag>::getPerfCellPressure(const typename WellInterface<TypeTag>::FluidState& fs) const
     {
-        Eval pressure;
-        if (Indices::oilEnabled) {
-            pressure = fs.pressure(FluidSystem::oilPhaseIdx);
+        if constexpr (Indices::oilEnabled) {
+            return fs.pressure(FluidSystem::oilPhaseIdx);
+        } else if constexpr (Indices::waterEnabled) {
+            return fs.pressure(FluidSystem::waterPhaseIdx);
         } else {
-            if (Indices::waterEnabled) {
-                pressure = fs.pressure(FluidSystem::waterPhaseIdx);
-            } else {
-                pressure = fs.pressure(FluidSystem::gasPhaseIdx);
-            }
+            return fs.pressure(FluidSystem::gasPhaseIdx);
         }
-        return pressure;
     }
+
 } // namespace Opm
