@@ -18,22 +18,130 @@
   along with OPM.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include <fmt/format.h>
 #include <config.h>
+
 #include <opm/simulators/wells/WellState.hpp>
 
 #include <opm/common/ErrorMacros.hpp>
+#include <opm/input/eclipse/Schedule/MSW/WellSegments.hpp>
 #include <opm/input/eclipse/Schedule/Schedule.hpp>
-#include <opm/simulators/wells/ParallelWellInfo.hpp>
+#include <opm/input/eclipse/Schedule/Well/Well.hpp>
+#include <opm/input/eclipse/Schedule/Well/WellConnections.hpp>
 
 #include <opm/simulators/utils/ParallelCommunication.hpp>
+#include <opm/simulators/wells/ParallelWellInfo.hpp>
+#include <opm/grid/common/p2pcommunicator.hh>
+#include <opm/output/data/Wells.hpp>
 
 #include <algorithm>
 #include <cassert>
 #include <numeric>
+#include <set>
+#include <stdexcept>
+#include <vector>
 
-namespace Opm
+#include <fmt/format.h>
+
+namespace {
+
+using P2PCommunicatorType = Dune::Point2PointCommunicator<Dune::SimpleMessageBuffer>;
+using MessageBufferType = P2PCommunicatorType::MessageBufferType;
+
+class PackUnpackXConn : public P2PCommunicatorType::DataHandleInterface
 {
+public:
+    using XConn = std::vector<Opm::data::Connection>;
+
+    explicit PackUnpackXConn(const bool   isOwner,
+                             const XConn& local,
+                             XConn&       global);
+
+    // Pack all data associated with link.
+    void pack(const int link, MessageBufferType& buffer);
+
+    // Unpack all data associated with link.
+    void unpack([[maybe_unused]] const int link,
+                MessageBufferType&         buffer);
+
+private:
+    const XConn& local_;
+    XConn& global_;
+};
+
+PackUnpackXConn::PackUnpackXConn(const bool   isOwner,
+                                 const XConn& local,
+                                 XConn&       global)
+    : local_ (local)
+    , global_(global)
+{
+    if (! isOwner) {
+        return;
+    }
+
+    this->global_.insert(this->global_.end(),
+                         this->local_.begin(),
+                         this->local_.end());
+}
+
+void PackUnpackXConn::pack(const int          link,
+                           MessageBufferType& buffer)
+{
+    // We should only get one link
+    if (link != 0) {
+        throw std::logic_error {
+            "link in pack() does not match expected value 0"
+        };
+    }
+
+    // Write all local connection results
+    {
+        const auto nconn = this->local_.size();
+        buffer.write(nconn);
+    }
+
+    for (const auto& conn : this->local_) {
+        conn.write(buffer);
+    }
+}
+
+void PackUnpackXConn::unpack([[maybe_unused]] const int link,
+                             MessageBufferType&         buffer)
+{
+    const auto nconn = [this, &buffer]()
+    {
+        auto nc = 0 * this->local_.size();
+        buffer.read(nc);
+
+        return nc;
+    }();
+
+    this->global_.reserve(this->global_.size() + nconn);
+
+    for (auto conn = 0*nconn; conn < nconn; ++conn) {
+        this->global_.emplace_back().read(buffer);
+    }
+}
+
+} // Anonymous namespace
+
+namespace Opm {
+
+WellState::WellState(const ParallelWellInfo& pinfo)
+    : phase_usage_{}
+{
+    wells_.add("test4",
+               SingleWellState{"dummy", pinfo, false, 0.0, {}, phase_usage_, 0.0});
+}
+
+WellState WellState::serializationTestObject(const ParallelWellInfo& pinfo)
+{
+    WellState result(PhaseUsage{});
+    result.alq_state = ALQState::serializationTestObject();
+    result.well_rates = {{"test2", {true, {1.0}}}, {"test3", {false, {2.0}}}};
+    result.wells_.add("test4", SingleWellState::serializationTestObject(pinfo));
+
+    return result;
+}
 
 void WellState::base_init(const std::vector<double>& cellPressures,
                           const std::vector<Well>& wells_ecl,
@@ -64,7 +172,14 @@ void WellState::initSingleProducer(const Well& well,
     const auto& pu = this->phase_usage_;
     const double temp = 273.15 + 15.56;
 
-    auto& ws = this->wells_.add(well.name(), SingleWellState{well.name(), well_info, true, pressure_first_connection, well_perf_data, pu, temp});
+    auto& ws = this->wells_.add(well.name(),
+                                SingleWellState{well.name(),
+                                                well_info,
+                                                true,
+                                                pressure_first_connection,
+                                                well_perf_data,
+                                                pu,
+                                                temp});
 
     // the rest of the code needs to executed even if ws.perf_data is empty
     // as this does not say anything for the whole well if it is distributed.
@@ -76,7 +191,6 @@ void WellState::initSingleProducer(const Well& well,
     ws.update_producer_targets(well, summary_state);
 }
 
-
 void WellState::initSingleInjector(const Well& well,
                                    const ParallelWellInfo& well_info,
                                    double pressure_first_connection,
@@ -84,10 +198,15 @@ void WellState::initSingleInjector(const Well& well,
                                    const SummaryState& summary_state) {
 
     const auto& pu = this->phase_usage_;
-    const auto& inj_controls = well.injectionControls(summary_state);
-    const double temp = inj_controls.temperature;
+    const double temp = well.temperature();
 
-    auto& ws = this->wells_.add(well.name(), SingleWellState{well.name(), well_info, false, pressure_first_connection, well_perf_data, pu, temp});
+    auto& ws = this->wells_.add(well.name(), SingleWellState{well.name(),
+                                                             well_info,
+                                                             false,
+                                                             pressure_first_connection,
+                                                             well_perf_data,
+                                                             pu,
+                                                             temp});
 
     // the rest of the code needs to executed even if ws.perf_data is empty
     // as this does not say anything for the whole well if it is distributed.
@@ -99,8 +218,6 @@ void WellState::initSingleInjector(const Well& well,
     ws.update_injector_targets(well, summary_state);
 }
 
-
-
 void WellState::initSingleWell(const std::vector<double>& cellPressures,
                                const Well& well,
                                const std::vector<PerforationData>& well_perf_data,
@@ -112,13 +229,15 @@ void WellState::initSingleWell(const std::vector<double>& cellPressures,
         pressure_first_connection = cellPressures[well_perf_data[0].cell_index];
     pressure_first_connection = well_info.broadcastFirstPerforationValue(pressure_first_connection);
 
-    if (well.isInjector())
-        this->initSingleInjector(well, well_info, pressure_first_connection, well_perf_data, summary_state);
-    else
-        this->initSingleProducer(well, well_info, pressure_first_connection, well_perf_data, summary_state);
+    if (well.isInjector()) {
+        this->initSingleInjector(well, well_info, pressure_first_connection,
+                                 well_perf_data, summary_state);
+    } else {
+        this->initSingleProducer(well, well_info, pressure_first_connection,
+                                 well_perf_data, summary_state);
+    }
 }
 
-
 void WellState::init(const std::vector<double>& cellPressures,
                      const Schedule& schedule,
                      const std::vector<Well>& wells_ecl,
@@ -129,8 +248,11 @@ void WellState::init(const std::vector<double>& cellPressures,
                      const SummaryState& summary_state)
 {
     // call init on base class
-    this->base_init(cellPressures, wells_ecl, parallel_well_info, well_perf_data, summary_state);
-    this->global_well_info = std::make_optional<GlobalWellInfo>( schedule, report_step, wells_ecl );
+    this->base_init(cellPressures, wells_ecl, parallel_well_info,
+                    well_perf_data, summary_state);
+    this->global_well_info = std::make_optional<GlobalWellInfo>(schedule,
+                                                                report_step,
+                                                                wells_ecl);
     for (const auto& wname : schedule.wellNames(report_step))
     {
         well_rates.insert({wname, std::make_pair(false, std::vector<double>(this->numPhases()))});
@@ -307,7 +429,8 @@ WellState::currentWellRates(const std::string& wellName) const
     auto it = well_rates.find(wellName);
 
     if (it == well_rates.end())
-        OPM_THROW(std::logic_error, "Could not find any rates for well  " << wellName);
+        OPM_THROW(std::logic_error,
+                  "Could not find any rates for well " + wellName);
 
     return it->second.second;
 }
@@ -317,35 +440,39 @@ void WellState::gatherVectorsOnRoot(const std::vector<data::Connection>& from_co
                                                          std::vector<data::Connection>& to_connections,
                                                          const Communication& comm) const
 {
-    int size = from_connections.size();
-    std::vector<int> sizes;
-    std::vector<int> displ;
-    if (comm.rank()==0){
-        sizes.resize(comm.size());
-    }
-    comm.gather(&size, sizes.data(), 1, 0);
+    auto send = std::set<int>{};
+    auto recv = std::set<int>{};
 
-    if (comm.rank()==0){
-        displ.resize(comm.size()+1, 0);
-        std::partial_sum(sizes.begin(), sizes.end(), displ.begin()+1);
-        to_connections.resize(displ.back());
+    const auto isOwner = comm.rank() == 0;
+    if (isOwner) {
+        for (auto other = 0*comm.size() + 1; other < comm.size(); ++other) {
+            recv.insert(other);
+        }
     }
-    comm.gatherv(from_connections.data(), size, to_connections.data(),
-                 sizes.data(), displ.data(), 0);
+    else {
+        send.insert(0);
+    }
+
+    auto toOwnerComm = P2PCommunicatorType{ comm };
+    toOwnerComm.insertRequest(send, recv);
+
+    PackUnpackXConn lineariser { isOwner, from_connections, to_connections };
+    toOwnerComm.exchange(lineariser);
 }
 
 data::Wells
 WellState::report(const int* globalCellIdxMap,
                   const std::function<bool(const int)>& wasDynamicallyClosed) const
 {
-    if (this->numWells() == 0)
+    if (this->numWells() == 0) {
         return {};
+    }
 
     using rt = data::Rates::opt;
     const auto& pu = this->phaseUsage();
 
     data::Wells res;
-    for( std::size_t well_index = 0; well_index < this->size(); well_index++) {
+    for (std::size_t well_index = 0; well_index < this->size(); ++well_index) {
         const auto& ws = this->well(well_index);
         if ((ws.status == Well::Status::SHUT) && !wasDynamicallyClosed(well_index))
         {
@@ -358,20 +485,21 @@ WellState::report(const int* globalCellIdxMap,
         const auto& wv = ws.surface_rates;
         const auto& wname = this->name(well_index);
 
+        auto dummyWell = data::Well{};
+        auto& well = ws.parallel_info.get().isOwner() ? res[wname] : dummyWell;
 
-        data::Well well;
         well.bhp = ws.bhp;
         well.thp = ws.thp;
         well.temperature = ws.temperature;
 
-        if( pu.phase_used[BlackoilPhases::Aqua] ) {
+        if (pu.phase_used[BlackoilPhases::Aqua]) {
             well.rates.set(rt::wat, wv[ pu.phase_pos[BlackoilPhases::Aqua] ] );
             well.rates.set(rt::reservoir_water, reservoir_rates[pu.phase_pos[BlackoilPhases::Aqua]]);
             well.rates.set(rt::productivity_index_water, wpi[pu.phase_pos[BlackoilPhases::Aqua]]);
             well.rates.set(rt::well_potential_water, well_potentials[pu.phase_pos[BlackoilPhases::Aqua]]);
         }
 
-        if( pu.phase_used[BlackoilPhases::Liquid] ) {
+        if (pu.phase_used[BlackoilPhases::Liquid]) {
             well.rates.set(rt::oil, wv[ pu.phase_pos[BlackoilPhases::Liquid] ] );
             well.rates.set(rt::reservoir_oil, reservoir_rates[pu.phase_pos[BlackoilPhases::Liquid]]);
             well.rates.set(rt::productivity_index_oil, wpi[pu.phase_pos[BlackoilPhases::Liquid]]);
@@ -404,7 +532,7 @@ WellState::report(const int* globalCellIdxMap,
             well.rates.set(rt::alq, 0.0);
         }
 
-        well.rates.set(rt::dissolved_gas, ws.dissolved_gas_rate);
+        well.rates.set(rt::dissolved_gas, ws.dissolved_gas_rate + ws.dissolved_gas_rate_in_water);
         well.rates.set(rt::vaporized_oil, ws.vaporized_oil_rate);
         well.rates.set(rt::vaporized_water, ws.vaporized_wat_rate);
 
@@ -417,12 +545,10 @@ WellState::report(const int* globalCellIdxMap,
         }
 
         const auto& pwinfo = ws.parallel_info.get();
-        if (pwinfo.communication().size()==1)
-        {
+        if (pwinfo.communication().size() == 1) {
             reportConnections(well.connections, pu, well_index, globalCellIdxMap);
         }
-        else
-        {
+        else {
             std::vector<data::Connection> connections;
 
             reportConnections(connections, pu, well_index, globalCellIdxMap);
@@ -434,13 +560,11 @@ WellState::report(const int* globalCellIdxMap,
             const auto seg_no = ws.segments.segment_number()[seg_ix];
             well.segments[seg_no] = this->reportSegmentResults(well_index, seg_ix, seg_no);
         }
-
-        res.insert( {wname, well} );
     }
+
     return res;
 }
 
-
 void WellState::reportConnections(std::vector<data::Connection>& connections,
                                   const PhaseUsage &pu,
                                   std::size_t well_index,
@@ -452,7 +576,7 @@ void WellState::reportConnections(std::vector<data::Connection>& connections,
     connections.resize(num_perf_well);
     const auto& perf_rates = perf_data.rates;
     const auto& perf_pressure = perf_data.pressure;
-    for( int i = 0; i < num_perf_well; ++i ) {
+    for (int i = 0; i < num_perf_well; ++i) {
       const auto active_index = perf_data.cell_index[i];
         auto& connection = connections[ i ];
         connection.index = globalCellIdxMap[active_index];
@@ -462,50 +586,52 @@ void WellState::reportConnections(std::vector<data::Connection>& connections,
     }
 
     const int np = pu.num_phases;
-    size_t local_comp_index = 0;
     std::vector< rt > phs( np );
     std::vector<rt> pi(np);
-    if( pu.phase_used[Water] ) {
+    if (pu.phase_used[Water]) {
         phs.at( pu.phase_pos[Water] ) = rt::wat;
         pi .at( pu.phase_pos[Water] ) = rt::productivity_index_water;
     }
 
-    if( pu.phase_used[Oil] ) {
+    if (pu.phase_used[Oil]) {
         phs.at( pu.phase_pos[Oil] ) = rt::oil;
         pi .at( pu.phase_pos[Oil] ) = rt::productivity_index_oil;
     }
 
-    if( pu.phase_used[Gas] ) {
+    if (pu.phase_used[Gas]) {
         phs.at( pu.phase_pos[Gas] ) = rt::gas;
         pi .at( pu.phase_pos[Gas] ) = rt::productivity_index_gas;
     }
-    for( auto& comp : connections) {
-        const auto * rates = &perf_data.phase_rates[np*local_comp_index];
-        const auto& connPI  = perf_data.prod_index;
 
-        for( int i = 0; i < np; ++i ) {
+    size_t local_conn_index = 0;
+    for (auto& comp : connections) {
+        const auto * rates = &perf_data.phase_rates[np * local_conn_index];
+        const auto * connPI = &perf_data.prod_index[np * local_conn_index];
+
+
+        for (int i = 0; i < np; ++i) {
             comp.rates.set( phs[ i ], rates[i] );
             comp.rates.set( pi [ i ], connPI[i] );
         }
-        if ( pu.has_polymer ) {
+        if (pu.has_polymer) {
             const auto& perf_polymer_rate = perf_data.polymer_rates;
-            comp.rates.set( rt::polymer, perf_polymer_rate[local_comp_index]);
+            comp.rates.set( rt::polymer, perf_polymer_rate[local_conn_index]);
         }
-        if ( pu.has_brine ) {
+        if (pu.has_brine) {
             const auto& perf_brine_rate = perf_data.brine_rates;
-            comp.rates.set( rt::brine, perf_brine_rate[local_comp_index]);
+            comp.rates.set( rt::brine, perf_brine_rate[local_conn_index]);
         }
-        if ( pu.has_solvent ) {
+        if (pu.has_solvent) {
             const auto& perf_solvent_rate = perf_data.solvent_rates;
-            comp.rates.set( rt::solvent, perf_solvent_rate[local_comp_index] );
+            comp.rates.set( rt::solvent, perf_solvent_rate[local_conn_index] );
         }
 
-        ++local_comp_index;
+        ++local_conn_index;
     }
 }
 
 void WellState::initWellStateMSWell(const std::vector<Well>& wells_ecl,
-                                                         const WellState* prev_well_state)
+                                    const WellState* prev_well_state)
 {
     // still using the order in wells
     const int nw = wells_ecl.size();
@@ -521,7 +647,7 @@ void WellState::initWellStateMSWell(const std::vector<Well>& wells_ecl,
         const auto& well_ecl = wells_ecl[w];
         auto& ws = this->well(w);
 
-        if ( well_ecl.isMultiSegment() ) {
+        if (well_ecl.isMultiSegment()) {
             const WellSegments& segment_set = well_ecl.getSegments();
             // assuming the order of the perforations in well_ecl is the same with Wells
             const WellConnections& completion_set = well_ecl.getConnections();
@@ -537,6 +663,14 @@ void WellState::initWellStateMSWell(const std::vector<Well>& wells_ecl,
                 const Connection& connection = completion_set.get(perf);
                 if (connection.state() == Connection::State::OPEN) {
                     const int segment_index = segment_set.segmentNumberToIndex(connection.segment());
+                    if (segment_index == -1) {
+                        OPM_THROW(std::logic_error,
+                                  fmt::format("COMPSEGS: Well {} has connection in cell {}, {}, {} "
+                                              "without associated segment.", well_ecl.name(),
+                                              connection.getI() + 1 , connection.getJ() + 1,
+                                              connection.getK() + 1 ));
+                    }
+
                     segment_perforations[segment_index].push_back(n_activeperf);
                     n_activeperf++;
                 }
@@ -554,7 +688,6 @@ void WellState::initWellStateMSWell(const std::vector<Well>& wells_ecl,
                 }
             }
 
-
             auto& perf_data = ws.perf_data;
             // for the seg_rates_, now it becomes a recursive solution procedure.
             if (pu.phase_used[Gas]) {
@@ -584,7 +717,7 @@ void WellState::initWellStateMSWell(const std::vector<Well>& wells_ecl,
                 segment_pressure[0] = ws.bhp;
                 const auto& perf_press = perf_data.pressure;
                 for (int seg = 1; seg < well_nseg; ++seg) {
-                    if ( !segment_perforations[seg].empty() ) {
+                    if (!segment_perforations[seg].empty()) {
                         const int first_perf = segment_perforations[seg][0];
                         segment_pressure[seg] = perf_press[first_perf];
                     } else {
@@ -605,7 +738,7 @@ void WellState::initWellStateMSWell(const std::vector<Well>& wells_ecl,
             if (well.getStatus() == Well::Status::SHUT)
                 continue;
 
-            if ( !well.isMultiSegment() )
+            if (!well.isMultiSegment())
                 continue;
 
             const auto& wname = well.name();
@@ -616,10 +749,11 @@ void WellState::initWellStateMSWell(const std::vector<Well>& wells_ecl,
                     continue;
                 }
 
-                // TODO: the well with same name can change a lot, like they might not have same number of segments
-                // we need to handle that later.
-                // for now, we just copy them.
-                ws.segments = prev_ws.segments;
+                // we do not copy the segment information if the number of segments have changed
+                // safer way should be to check the relevant Event
+                if (ws.segments.size() == prev_ws.segments.size()) {
+                    ws.segments = prev_ws.segments;
+                }
             }
         }
     }
@@ -627,10 +761,10 @@ void WellState::initWellStateMSWell(const std::vector<Well>& wells_ecl,
 
 void
 WellState::calculateSegmentRates(const std::vector<std::vector<int>>& segment_inlets,
-                                                      const std::vector<std::vector<int>>&segment_perforations,
-                                                      const std::vector<double>& perforation_rates,
-                                                      const int np, const int segment,
-                                                      std::vector<double>& segment_rates)
+                                 const std::vector<std::vector<int>>& segment_perforations,
+                                 const std::vector<double>& perforation_rates,
+                                 const int np, const int segment,
+                                 std::vector<double>& segment_rates)
 {
     // the rate of the segment equals to the sum of the contribution from the perforations and inlet segment rates.
     // the first segment is always the top segment, its rates should be equal to the well rates.
@@ -671,14 +805,12 @@ void WellState::shutWell(int well_index)
     ws.shut();
 }
 
-void WellState::updateStatus(int well_index, Well::Status status)
+void WellState::updateStatus(int well_index, WellStatus status)
 {
     auto& ws = this->well(well_index);
     ws.updateStatus(status);
 }
 
-
-
 template<class Comm>
 void WellState::communicateGroupRates(const Comm& comm)
 {
@@ -724,7 +856,6 @@ void WellState::communicateGroupRates(const Comm& comm)
     assert(pos == sz);
 }
 
-
 template<class Comm>
 void WellState::updateGlobalIsGrup(const Comm& comm)
 {
@@ -740,17 +871,21 @@ void WellState::updateGlobalIsGrup(const Comm& comm)
 }
 
 data::Segment
-WellState::reportSegmentResults(const int         well_id,
-                                const int         seg_ix,
-                                const int         seg_no) const
+WellState::reportSegmentResults(const int well_id,
+                                const int seg_ix,
+                                const int seg_no) const
 {
+    using PhaseQuant = data::SegmentPhaseQuantity::Item;
+
     const auto& segments = this->well(well_id).segments;
-    if (segments.empty())
+    if (segments.empty()) {
         return {};
+    }
 
     auto seg_res = data::Segment{};
     {
         using Value = data::SegmentPressures::Value;
+
         auto& segpress = seg_res.pressures;
         segpress[Value::Pressure] = segments.pressure[seg_ix];
         segpress[Value::PDrop] = segments.pressure_drop(seg_ix);
@@ -760,20 +895,42 @@ WellState::reportSegmentResults(const int         well_id,
     }
 
     const auto& pu = this->phaseUsage();
-    const auto rate = &segments.rates[seg_ix * pu.num_phases];
+    const auto* rate = &segments.rates[seg_ix * pu.num_phases];
+    const auto* resv = &segments.phase_resv_rates[seg_ix * pu.num_phases];
+    const auto* velocity = &segments.phase_velocity[seg_ix * pu.num_phases];
+    const auto* holdup = &segments.phase_holdup[seg_ix * pu.num_phases];
+    const auto* viscosity = &segments.phase_viscosity[seg_ix * pu.num_phases];
+
     if (pu.phase_used[Water]) {
-        seg_res.rates.set(data::Rates::opt::wat,
-                          rate[pu.phase_pos[Water]]);
+        const auto iw = pu.phase_pos[Water];
+
+        seg_res.rates.set(data::Rates::opt::wat, rate[iw]);
+        seg_res.rates.set(data::Rates::opt::reservoir_water, resv[iw]);
+        seg_res.velocity.set(PhaseQuant::Water, velocity[iw]);
+        seg_res.holdup.set(PhaseQuant::Water, holdup[iw]);
+        seg_res.viscosity.set(PhaseQuant::Water, viscosity[iw]);
     }
 
     if (pu.phase_used[Oil]) {
-        seg_res.rates.set(data::Rates::opt::oil,
-                          rate[pu.phase_pos[Oil]]);
+        const auto io = pu.phase_pos[Oil];
+
+        seg_res.rates.set(data::Rates::opt::oil, rate[io]);
+        seg_res.rates.set(data::Rates::opt::vaporized_oil, segments.vaporized_oil_rate[seg_ix]);
+        seg_res.rates.set(data::Rates::opt::reservoir_oil, resv[io]);
+        seg_res.velocity.set(PhaseQuant::Oil, velocity[io]);
+        seg_res.holdup.set(PhaseQuant::Oil, holdup[io]);
+        seg_res.viscosity.set(PhaseQuant::Oil, viscosity[io]);
     }
 
     if (pu.phase_used[Gas]) {
-        seg_res.rates.set(data::Rates::opt::gas,
-                          rate[pu.phase_pos[Gas]]);
+        const auto ig = pu.phase_pos[Gas];
+
+        seg_res.rates.set(data::Rates::opt::gas, rate[ig]);
+        seg_res.rates.set(data::Rates::opt::dissolved_gas, segments.dissolved_gas_rate[seg_ix]);
+        seg_res.rates.set(data::Rates::opt::reservoir_gas, resv[ig]);
+        seg_res.velocity.set(PhaseQuant::Gas, velocity[ig]);
+        seg_res.holdup.set(PhaseQuant::Gas, holdup[ig]);
+        seg_res.viscosity.set(PhaseQuant::Gas, viscosity[ig]);
     }
 
     seg_res.segNumber = seg_no;
@@ -782,7 +939,7 @@ WellState::reportSegmentResults(const int         well_id,
 }
 
 bool WellState::wellIsOwned(std::size_t well_index,
-                                                 [[maybe_unused]] const std::string& wellName) const
+                            [[maybe_unused]] const std::string& wellName) const
 {
     const auto& well_info = this->parallelWellInfo(well_index);
     assert(well_info.name() == wellName);
@@ -793,15 +950,15 @@ bool WellState::wellIsOwned(std::size_t well_index,
 bool WellState::wellIsOwned(const std::string& wellName) const
 {
     const auto& well_index = this->index(wellName);
-    if (!well_index.has_value())
-        OPM_THROW(std::logic_error, "Could not find well " << wellName << " in well map");
+    if (!well_index.has_value()) {
+        OPM_THROW(std::logic_error,
+                  fmt::format("Could not find well {} in well map", wellName));
+    }
 
     return wellIsOwned(well_index.value(), wellName);
 }
 
-
-
-void WellState::updateWellsDefaultALQ( const std::vector<Well>& wells_ecl )
+void WellState::updateWellsDefaultALQ(const std::vector<Well>& wells_ecl)
 {
     const int nw = wells_ecl.size();
     for (int i = 0; i<nw; i++) {
@@ -814,6 +971,12 @@ void WellState::updateWellsDefaultALQ( const std::vector<Well>& wells_ecl )
     }
 }
 
+bool WellState::operator==(const WellState& rhs) const
+{
+    return this->alq_state == rhs.alq_state &&
+           this->well_rates == rhs.well_rates &&
+           this->wells_ == rhs.wells_;
+}
 
 const ParallelWellInfo&
 WellState::parallelWellInfo(std::size_t well_index) const
@@ -824,6 +987,5 @@ WellState::parallelWellInfo(std::size_t well_index) const
 
 template void WellState::updateGlobalIsGrup<Parallel::Communication>(const Parallel::Communication& comm);
 template void WellState::communicateGroupRates<Parallel::Communication>(const Parallel::Communication& comm);
+
 } // namespace Opm
-
-
