@@ -31,6 +31,7 @@
 #include <ebos/ebos.hh>
 #include <opm/models/utils/start.hh>
 
+#include <opm/input/eclipse/EclipseState/SummaryConfig/SummaryConfig.hpp>
 #include <opm/input/eclipse/Schedule/Schedule.hpp>
 #include <opm/input/eclipse/Schedule/Well/Well.hpp>
 #include <opm/simulators/utils/DeferredLogger.hpp>
@@ -181,6 +182,7 @@ BOOST_AUTO_TEST_CASE(G1)
         well_model.phaseUsage(),
         deferred_logger,
         well_state,
+        group_state,
         comm,
         /*glift_debug=*/false
     };
