@@ -56,7 +56,7 @@ declare -A tests
 #    tests[cmake_testname]="binary dirname SIMULATION_CASE
 #
 # The key in the dictionary must agree with the name given to the test when
-# registering it with cmake in compareEclFiles.cmake. The SIMULATION_CASE should
+# registering it with cmake in regressionTests.cmake. The SIMULATION_CASE should
 # be the basename of the .DATA file used for the simulation.
 
 tests[spe1]="flow spe1 SPE1CASE1"
@@ -77,10 +77,13 @@ tests[spe1_water]="flow spe1 SPE1CASE1_WATER"
 tests[spe1_spider]="flow radial_grid SPIDER_CAKESLICE"
 tests[spe1_radial]="flow radial_grid RADIAL_CAKESLICE"
 tests[spe1_import]="flow spe1 SPE1CASE1_IMPORT"
+tests[jfunc_01]="flow jfunc JFUNC-01"
+tests[pinch_nopinch_1x1x10]="flow pinch PINCH10_NOPINCH"
 tests[ctaquifer_2d_oilwater]="flow aquifer-oilwater 2D_OW_CTAQUIFER"
 tests[fetkovich_2d]="flow aquifer-fetkovich 2D_FETKOVICHAQUIFER"
 tests[numerical_aquifer_3d_1aqu]="flow aquifer-num 3D_1AQU_3CELLS"
 tests[numerical_aquifer_3d_2aqu]="flow aquifer-num 3D_2AQU_NUM"
+tests[aquflux_01]="flow aquifers AQUFLUX-01"
 tests[msw_2d_h]="flow msw_2d_h 2D_H__"
 tests[msw_3d_hfa]="flow msw_3d_hfa 3D_MSW"
 tests[polymer_oilwater]="flow polymer_oilwater 2D_OILWATER_POLYMER"
@@ -97,6 +100,7 @@ tests[base_model_1]="flow model1 BASE_MODEL_1"
 tests[msw_model_1]="flow model1 MSW_MODEL_1"
 tests[faults_model_1]="flow model1 FAULTS_MODEL_1"
 tests[base_model2]="flow model2 0_BASE_MODEL2"
+tests[base_model2_welpi]="flow model2 0B_WELPI_MODEL2"
 tests[0a1_grpctl_stw_model2]="flow model2 0A1_GRCTRL_LRAT_ORAT_BASE_MODEL2_STW"
 tests[0a1_grpctl_msw_model2]="flow model2 0A1_GRCTRL_LRAT_ORAT_BASE_MODEL2_MSW"
 tests[0a2_grpctl_stw_model2]="flow model2 0A2_GRCTRL_LRAT_ORAT_GGR_BASE_MODEL2_STW"
@@ -108,6 +112,7 @@ tests[0a4_grpctl_msw_model2]="flow model2 0A4_GRCTRL_LRAT_LRAT_GGR_BASE_MODEL2_M
 tests[multregt_model2]="flow model2 1_MULTREGT_MODEL2"
 tests[multxyz_model2]="flow model2 2_MULTXYZ_MODEL2"
 tests[multflt_model2]="flow model2 3_MULTFLT_MODEL2"
+tests[multflt_sched_model2]="flow model2 3_A_MPI_MULTFLT_SCHED_MODEL2"
 tests[multpvv_model2]="flow model2 4_MINPVV_MODEL2"
 tests[swatinit_model2]="flow model2 5_SWATINIT_MODEL2"
 tests[endscale_model2]="flow model2 6_ENDSCALE_MODEL2"
@@ -158,7 +163,9 @@ tests[wsegaicd]="flow wsegaicd BASE_MSW_WSEGAICD"
 tests[wsegvalv]="flow wsegvalv BASE_MSW_WSEGVALV"
 tests[bc_lab]="flow bc_lab BC_LAB"
 tests[pinch_multz_all]="flow pinch PINCH_MULTZ_ALL"
+tests[pinch_multzm_all]="flow pinch PINCH_MULTZ-_ALL"
 tests[pinch_multz_all_barrier]="flow pinch PINCH_MULTZ_ALL_BARRIER"
+tests[pinch_multzm_all_barrier]="flow pinch PINCH_MULTZ-_ALL_BARRIER"
 tests[model6_msw]="flow model6 1_MSW_MODEL6"
 tests[norne_reperf]="flow norne NORNE_ATW2013_B1H_RE-PERF"
 tests[compl_smry]="flow compl_smry COMPL_SMRY"
@@ -195,6 +202,18 @@ tests[fpr_nonhc]="flow water-1ph WATER2F"
 tests[actionx_wpimult]="flow actionx ACTIONX_WPIMULT"
 tests[wvfpexp_02]="flow wvfpexp WVFPEXP-02"
 tests[spe1case2_krnum]="flow spe1 SPE1CASE2_KRNUM"
+tests[krnum_02x]="flow krnum KRNUM-02X"
+tests[krnum_02y]="flow krnum KRNUM-02Y"
+tests[krnum_02z]="flow krnum KRNUM-02Z"
+tests[krnum_03x]="flow krnum KRNUM-03X"
+tests[krnum_03y]="flow krnum KRNUM-03Y"
+tests[krnum_03z]="flow krnum KRNUM-03Z"
+tests[model_field_gridunit_cm]="flow gridunit M_FIELD_GRID_CM"
+tests[model_field_gridunit_feet]="flow gridunit M_FIELD_GRID_FEET"
+tests[model_field_gridunit_metres]="flow gridunit M_FIELD_GRID_METRES"
+tests[model_metric_gridunit_cm]="flow gridunit M_METRIC_GRID_CM"
+tests[model_metric_gridunit_feet]="flow gridunit M_METRIC_GRID_FEET"
+tests[model_metric_gridunit_metres]="flow gridunit M_METRIC_GRID_METRES"
 
 changed_tests=""
 
@@ -259,7 +278,7 @@ else
 fi
 if [ -n "$CONVERT_ECL" ]
 then
-  for dep in opm-common opm-grid opm-material opm-models
+  for dep in opm-common opm-grid opm-models
   do
     pushd $WORKSPACE/deps/$dep > /dev/null
     name=`printf "%-14s" $dep`
